subroutine transcoef(np,npbig,coefsmall,coefbig,notest,expcoefs,poscoefs,negcoefs,coefout)

  use params
  implicit none

  ! this routine takes a vector of parameters being estimated and transforms to a full parameter vector

  integer np, npbig
  integer i, j
  doubleprecision coefsmall(np), coefbig(npbig), negcoefs(npbig), coefout(npbig)
  integer notest(npbig), expcoefs(npbig), poscoefs(npbig)

  j=0

  do i=1,npbig

    if(notest(i) == 1) then
      coefout(i) = coefbig(i)
    else
      j=j+1
      coefout(i) = coefsmall(j)
    endif

    if(expcoefs(i) == 1) coefout(i) = exp(coefout(i))*negcoefs(i)
    if(poscoefs(i) == 1 .and. coefout(i) .le. 0.d0) coefout(i) = 0.d0

  enddo

end subroutine transcoef


! importance draws for new inclusive values

subroutine newhngrids

  use params
  implicit none

  integer hh

  integer j,k
  doubleprecision u(1)

  if(pdrawtype .eq. 1) then

     do hh=1,nhhs

        do k=1,nrpts

           call DRNUN(1,u)

           do j=1,maxpurchases
              if(u(1) .lt. primpdistp(hh,j,2)) then
                 savedpricesi(hh,k,mod(itnum-1,nsavs)+1) = j
                 exit
              endif
           enddo

        enddo

     enddo

  else

     do hh=1,nhhs

        do j=1,niv
           call DRNNOA(nrpts,savedprices(hh,j,:,mod(itnum-1,nsavs)+1))

           savedprices(hh,j,:,mod(itnum-1,nsavs)+1) = savedprices(hh,j,:,mod(itnum-1,nsavs)+1)*impdisthh(2,hh,j)+impdisthh(1,hh,j)
        enddo
     enddo

  endif

end subroutine newhngrids



! hendel nevo transition process

! sum of splines for multiple indexes

doubleprecision function ivpred(x,spcoef,cut,n)

  use params
  implicit none

  integer n
  doubleprecision x(niv), spcoef(n+2,niv), cut(2,niv)

  integer i,j
  doubleprecision y(n+2,niv)

  ivpred = 0.d0

  do j=1,niv

     call bsplinei(x(j),cut(:,j),n,y(:,j))

     do i=1,n+2
        ivpred = ivpred + spcoef(i,j)*y(i,j)
     enddo

  enddo

end function ivpred



subroutine ptprobshn(plast,pnext,w,hh)

  use params
  implicit none

  doubleprecision plast(niv), pnext(niv), w, mu(niv)
  integer indx, hh, i, j
  doubleprecision mvnormpdf, ivpred

  logical pflag
  integer outfile

  if(hhcutiv(hh) .eq. 0) then
     do j=1,niv
        mu(j) = ivcoef(1,1,j,hh)
        do i=1,niv
           mu(j) = mu(j) + ivcoef(2,i,j,hh)*plast(i)
        enddo
     enddo
  else
     do j=1,niv
        mu(j) = ivpred(plast,ivcoef(1:(hhcutiv(hh)+4),:,j,hh),cutmat(1:2,:,hh),hhcutiv(hh)+2)
     enddo
  endif

  w = mvnormpdf(pnext,mu,ivsd(:,:,hh),dsighh(hh),niv)

end subroutine ptprobshn



! function to compute ivs at a given point

subroutine ivcalc(cbig,n,pvec,fvec,dvec,ivout)

  use params
  implicit none

  integer n
  doubleprecision cbig(n),pvec(numprods),fvec(numprods),dvec(numprods)
  doubleprecision ivout(niv)

  integer j,k,l,pindx

  doubleprecision denom, u

  logical dontskip

  do k=1,niv

     denom = 0

     do j=1,csetsize(k)

        dontskip = .true.

        do l=1,csetnels(k)
           pindx = choicesetmat(l,j,k)
           dontskip = dontskip .and. pvec(pindx) .gt. 0
        enddo

        if(dontskip) then

           u = 0
           do l=1,csetnels(k)
              pindx = choicesetmat(l,j,k)
              u = u + alphabig(l,j,k)*(cbig(pindx) - cbig(numprods+4)*pvec(pindx) + cbig(numprods+5)*fvec(pindx) + cbig(numprods+6)*dvec(pindx))
           enddo

           denom = denom + exp(u)

        endif

     enddo

     ivout(k) = log(denom)

  enddo


end subroutine ivcalc


! compute inclusive values

subroutine getivs(bpass,n,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,rflag,rstart,hhlimits,dopar)

  use params
  implicit none

  integer n, npbig
  doubleprecision coefbig(npbig), negcoefs(npbig)
  integer notest(npbig), expcoefs(npbig), poscoefs(npbig), rstart, hhlimits(2)
  logical rflag, dopar

  integer, parameter :: ncheck = 10

  integer i,hh,j,k,l,m,ii1

  doubleprecision bpass(n,nhhs), bigx(wkstart+maxpurchases,niv,niv*(ncutiv+4))

  doubleprecision c(n), cbig(npbig)

  integer visits(wksample)

  doubleprecision SST, SSE, pvec(numprods), fvec(numprods), dvec(numprods), h, maxiv, miniv, stddev
  doubleprecision resid(niv,wkstart+maxpurchases-1), yhat(wkstart+maxpurchases-1,1)
  doubleprecision b1pass(niv*(ncutiv+4),1)

  doubleprecision s2(niv), s3(niv), ivpred, pvcheck(ncheck,ncheck), ivtemp(niv)

  doubleprecision xsave(numprods*3,wkstart+maxpurchases), maxcoef

  integer nx(niv)

  doubleprecision dummyb(niv*(ncutiv+4),niv),v1(niv,niv),det

  logical bad, getout, rgrid

  bigx = 0.d0

  if(dopar) hhcutiv = ncutiv
  
  !$OMP PARALLEL IF(dopar) DEFAULT(SHARED) PRIVATE(hh,c,cbig,visits,i,pvec,fvec,dvec,j,xsave,getout,k,miniv,maxiv,h,nx,l,dummyb,SST,SSE,m,b1pass,yhat,resid,v1,ivtemp,bad,s2,s3,pvcheck,bigx,maxcoef) NUM_THREADS(nthread)
  !$OMP DO
  do hh=hhlimits(1),hhlimits(2)

     c = bpass(:,hh)

     call transcoef(n,npbig,c,coefbig,notest,expcoefs,poscoefs,negcoefs,cbig)

     visits = visitsave(:,hh)

     do i=1,wkstart+maxpurchases

        ivweek(i,hh,:) = 0.d0

        if(i .le. wksample) then

           k = abs(visits(i))

           pvec = initprice(1:numprods,k,i)

           if(visits(i) .lt. 0) then
              fvec = 0.d0
              dvec = 0.d0
           else
              fvec = initprice((numprods+1):(2*numprods),k,i)
              dvec = initprice((2*numprods+1):(3*numprods),k,i)
           endif

        else if(i .le. wkstart) then

           if(initweek(sthhind,i,hh) .gt. 0) then

              ! when you visit the store, use the store price
              do j=1,numprods

                 pvec(j) = initweek(sthhind+j,i,hh)
                 fvec(j) = initweek(sthhind+numprods+j,i,hh)
                 dvec(j) = initweek(sthhind+2*numprods+j,i,hh)

              enddo

           else
              !when you didn't, use last week's

              pvec = sasprices(hhdatatemp(3,i,hh),i,1:numprods)

              ! assume no feature or display if you don't go to the store

              fvec = 0.d0
              dvec = 0.d0

           endif

        else

           if(hhweek(sthhind,i-wkstart,hh) .gt. 0) then

              ! when you visit the store, use the store price
              do j=1,numprods

                 pvec(j) = hhweek(sthhind+j,i-wkstart,hh)
                 fvec(j) = hhweek(sthhind+numprods+j,i-wkstart,hh)
                 dvec(j) = hhweek(sthhind+2*numprods+j,i-wkstart,hh)

              enddo

           else
              !when you didn't, use last week's

              pvec = sasprices(hhdatatemp(3,i,hh),i,1:numprods)

              ! assume no feature or display if you don't go to the store

              fvec = 0.d0
              dvec = 0.d0

           endif

        endif
        xsave(1:numprods,i) = pvec
        xsave((numprods+1):(2*numprods),i) = fvec
        xsave((2*numprods+1):(3*numprods),i) = dvec

        call ivcalc(cbig,npbig,pvec,fvec,dvec,ivweek(i,hh,:),.false.)

        if(i .gt. wkstart) then

           if(int(hhweek(nhhvars-1,i-wkstart,hh)) .gt. 0) then

              ivsave(pstateptr(int(hhweek(nhhvars-1,i-wkstart,hh)),hh),hh,:) = ivweek(i,hh,:)

           endif

        endif

     enddo

     ! calculate cut points

     getout = .false.

     do while (.not. getout)

        do k=1,niv

           call pmin(ivweek(:,hh,k),wkstart+maxpurchases,miniv,j)
           call pmax(ivweek(:,hh,k),wkstart+maxpurchases,maxiv,j)

           h = (maxiv-miniv)/dble(hhcutiv(hh)+1)

           do i=1,hhcutiv(hh)+1

              cutmat(i,k,hh) = miniv + dble(i-1)*h

           enddo

        enddo

        ! compute b spline basis points

        if(hhcutiv(hh) .eq. 0) then

           bigx(:,:,1) = 1.d0
           do k=1,niv
              nx(k) = 1
              do l=1,niv
                 if(ivinclude(hh,k,l)) then
                    nx(k) = nx(k) + 1
                    bigx(:,k,nx(k)) = ivweek(:,hh,l)
                 endif
              enddo
           enddo

        else

           do i=1,wkstart+maxpurchases

              nx = 0

              do k=1,niv

                 do l=1,niv

                    if(ivinclude(hh,k,l)) then

                       call bsplinei(ivweek(i,hh,l),cutmat(1:2,l,hh),hhcutiv(hh)+2,bigx(i,k,(nx(k)+1):(nx(k)+hhcutiv(hh)+4)))

                       nx(k)=nx(k)+hhcutiv(hh)+4

                    endif

                 enddo

              enddo

           enddo

        endif

        ! do ols to get inclusive value autocorrelation coefficients

        if(hhcutiv(hh) .eq. 0) then
           do k=1,niv
              call DRLSE(wkstart+maxpurchases-1, ivweek(2:(wkstart+maxpurchases),hh,k), nx(k), bigx(1:(wkstart+maxpurchases-1),k,1:nx(k)), wkstart+maxpurchases-1, 0, dummyb(1:nx(k),k), SST, SSE)
              ivcoef(:,:,k,hh) = 0.d0
              ivcoef(1,1,k,hh) = dummyb(1,k)
              m=0
              do l=1,niv
                 if(ivinclude(hh,k,l)) then
                    m=m+1
                    ivcoef(2,l,k,hh) = dummyb(1+m,k)
                 else
                    ivcoef(2,l,k,hh) = 0
                 endif
              enddo
           enddo
        else
           maxcoef = 0.d0
           do k=1,niv
              call DRLSE(wkstart+maxpurchases-1, ivweek(2:(wkstart+maxpurchases),hh,k), nx(k), bigx(1:(wkstart+maxpurchases-1),k,1:nx(k)), wkstart+maxpurchases-1, 0, dummyb(1:nx(k),k), SST, SSE)
              ivcoef(:,:,k,hh) = 0.d0
              m=0
              do l=1,niv
                 if(ivinclude(hh,k,l)) then
                    m=m+1
                    ivcoef(1:(hhcutiv(hh)+4),l,k,hh) = dummyb((1+(hhcutiv(hh)+4)*(m-1)):((hhcutiv(hh)+4)*m),k)
                    do i=1,hhcutiv(hh)+4
                       if(abs(ivcoef(i,l,k,hh)) .gt. maxcoef) maxcoef = abs(ivcoef(i,l,k,hh))
                    enddo
                 endif
              enddo
           enddo
        endif

        do k=1,niv

           b1pass(1:nx(k),1) = dummyb(1:nx(k),k)

           yhat = matmul(bigx(1:(wkstart+maxpurchases-1),k,1:nx(k)),b1pass(1:nx(k),:))

           resid(k,:) = ivweek(2:(wkstart+maxpurchases),hh,k) - yhat(:,1)

        enddo

        if(diagsd) then
           v1 = 0.d0
           do k=1,niv
              v1(k,k) = stddev(resid(k,:),wkstart+maxpurchases-1)
              v1(k,k) = v1(k,k)*v1(k,k)
           enddo
        else
           call varmat(resid,niv,wkstart+maxpurchases-1,v1)
        endif

        if(diagsd) then
           ivsd(:,:,hh) = 0.d0
           do k=1,niv
              ivsd(k,k,hh) = 1.d0/v1(k,k)
           enddo
        else
           call invertmat(v1,ivsd(:,:,hh),niv)
        endif
        if(diagsd) then
           dsighh(hh) = 1.d0
           do k=1,niv
              dsighh(hh) = dsighh(hh)*sqrt(v1(k,k))
           enddo
        else
           dsighh(hh) = sqrt(det(v1,niv))
        endif

        if(rflag .and. itnum .gt. 1) then

           do i=rstart,min(itnum,nsavs)-1
              do k=1,nrpts

                 pvec = hhprices(1:numprods,savedpricesi(hh,k,mod(itnum-i-1,nsavs)+1))
                 fvec = hhprices((numprods+1):(2*numprods),savedpricesi(hh,k,mod(itnum-i-1,nsavs)+1))
                 dvec = hhprices((2*numprods+1):(3*numprods),savedpricesi(hh,k,mod(itnum-i-1,nsavs)+1))

                 call ivcalc(cbig,npbig,pvec,fvec,dvec,ivtemp,.false.)

                 savedprices(hh,k,:,mod(itnum-i-1,nsavs)+1) = ivtemp

              enddo
           enddo

        endif

        if(hhcutiv(hh) .gt. 0) then
           bad = .false.
           do i=1,ncheck
              do j=1,ncheck
                s2 = impdisthh(1,hh,:)+(-1.d0 + 2.d0*dble(i-1)/dble(ncheck-1))*3.d0*impdisthh(2,hh,:)
                s3 = impdisthh(1,hh,:)+(-1.d0 + 2.d0*dble(j-1)/dble(ncheck-1))*3.d0*impdisthh(2,hh,:)
                call ptprobshn(s2,s3,pvcheck(i,j),hh,.false.,-1)
                bad = bad .or. (pvcheck(i,j) .le. 1e-40)
              enddo
            enddo

            if(bad .or. maxcoef .gt. 100.d0) then
               hhcutiv(hh) = hhcutiv(hh) - 1
            else
               getout = .true.
            endif

        else
           getout = .true.
        endif

     enddo

  enddo
  !$OMP END DO
  !$OMP END PARALLEL

end subroutine getivs


! vf interpolation routines

subroutine getbell(accnum,bflag,hh,sstate,pstate,invstate,vf,erflag)

  !solve for expected value function

  use params
  implicit none

  integer hh,n,hh2,t
  integer br !brand - 1,cheer, 2,surf, 3, dash
  integer pstate      !price state
  integer invstate    !inventory state
  integer sstate      !sale state
  integer ind   !vfsav index we're looking at
  integer ds, erflag
  doubleprecision vf

  doubleprecision sum1,sum2,v,kern,vfk,sum3,sum4
  integer i,it,st,st1
  integer pstate1,pstate2

  integer accnum   !which hh-purch record are we in
  integer bflag    !are we updating bflag

  !get probability of all price grid points given pm

  sum1 = 0.d0
  sum2 = 0.d0

  !note: shouldn't need to keep track of this at every purchase,
  !price state space is pretty small.  However, they still need to be
  !separated, because in vfupdate we are storing kernel differences
  !that have draws on the posterior true taste, which get integrated over

  erflag = 0

  if (accnum .gt. 0 .and. itnum .gt. 1)  then

     !get it at a particular purchase event

     it = 1

     hh2 = hh

     do it=1,ntil

        if(kernsavhh(it,hh2) .gt. 0.d0) then
           hh2 = hh
           sum3 = 0.d0
           sum4 = 0.d0
           do i=1,nrpts
              sum3 = sum3 + vsav1(mod(itnum-int(kernsavhh(it,hh2))-1,nsavs)+1,hh2,invstate,i,sstate)&
                   &*wtsavhhbig(mod(itnum-int(kernsavhh(it,hh2))-1,nsavs)+1,hh,pstate,i)

              sum4 = sum4 + wtsavhhbig(mod(itnum-int(kernsavhh(it,hh2))-1,nsavs)+1,hh,pstate,i)

           enddo
           sum1 = sum1 + sum3/sum4
           if(sum4 .eq. 0.d0) erflag = 1
           sum2 = sum2 + 1.d0
        endif

     enddo

  else if(accnum .eq. 0 .and. itnum .gt. 1) then

     !state space point, used by vfupdate

     it = 1

     hh2 = hh
     do it=1,ntil

        if(kernsavhh(it,hh2) .gt. 0.d0) then
           hh2 = hh
           sum3 = 0.d0
           sum4 = 0.d0
           do i=1,nrpts

              sum3 = sum3 + vsav1(mod(itnum-int(kernsavhh(it,hh2))-1,nsavs)+1,hh2,invstate,i,sstate)&
                   &*wtsavbig(mod(itnum-int(kernsavhh(it,hh2))-1,nsavs)+1,hh2,1,pstate,i)

              sum4 = sum4 + wtsavbig(mod(itnum-int(kernsavhh(it,hh2))-1,nsavs)+1,hh2,1,pstate,i)

           enddo
           sum1 = sum1 + sum3/sum4
           if(sum4 .eq. 0.d0) erflag = 1
           sum2 = sum2 + 1.d0
	endif

     enddo

  endif



  if (sum2 .gt. 0.d0) then

     vf = sum1/sum2

  else

     vf = 0.d0

  endif

end subroutine getbell


! find closest indexes of past draws

subroutine getclosest(n,ntilde,m,ck,kshh)

  use params
  implicit none

  integer n,ntilde,m,m2

  doubleprecision kshh(nsavs)

  doubleprecision ck(2,n)

  integer i,j,k,s

  integer smallestinds(ntilde)        !indexes of the nearest neighbors
  doubleprecision smallest(ntilde)    !distances of the nearest neighbors

  doubleprecision dist


  smallestinds = 0
  smallest = 1.0e10


  do i=1,min(itnum,nsavs)-1

     m2 = m

     ck(2,1:n) = bsavebig(:,m2,mod(itnum-i-1,nsavs)+1)

     dist = sqrt(dot_product(ck(2,:)-ck(1,:),ck(2,:)-ck(1,:)))

     if (dist .lt. smallest(ntilde)) then

        !put it in place

        do k=1,ntilde

           if(dist .lt. smallest(k)) then

              if (k .lt. ntilde) then
                 smallest(k+1:ntilde) = smallest(k:ntilde-1)
                 smallestinds(k+1:ntilde) = smallestinds(k:ntilde-1)
              endif

              smallest(k) = dist
              smallestinds(k) = i

              exit

           endif

        enddo

     endif

  enddo

  kshh(:) = 0.d0

  kshh(1:ntilde) = dble(smallestinds)

end subroutine getclosest





! update value function

subroutine vfupdate(n,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs)

  !draw the coupons and simulations used for expected value function updates
  !then calculate the utility at each draw, average it using nearest neighbor, and plug it in as the
  !potential expected value function update

  !new version - only needs to use 1 value function at each iteration

  !definitions of value function learning states


  use params
  implicit none

  integer n, npbig
  doubleprecision coefbig(npbig), negcoefs(npbig)
  integer notest(npbig), expcoefs(npbig), poscoefs(npbig)

  integer i,j,k,hh,s,st,stn,l

  integer st1,st2,sp1,sp2,erflag,maxerr

  logical doloop

  doubleprecision diff, sumw

  doubleprecision, parameter :: tol = 1.0

  doubleprecision, parameter :: bigM = 0.5d0

  integer ri,m,loop1(nipts,nrpts),stt(2,numprods),hhlimits(2),hhlim2(2)

  integer, parameter :: ncheck = 10

  doubleprecision a01,a11,b01,b11

  doubleprecision c(n), cbig(npbig), ctemp(npbig)

  doubleprecision v1(nmdraws), v2(nmdraws), x(1), s2(niv)

  doubleprecision utildisc, normpdf, utq

  doubleprecision u0

  integer uc(numprods,2,nmdraws), ctype

  doubleprecision c2(n), ck(2,n), v, ut(2), maxutil,  eutil

  doubleprecision x1,x2,inv1,inv2,p1,p2,c1d,c2d,util,uttemp(bigj+1)

  doubleprecision s3(niv), pvcheck(ncheck,ncheck)

  doubleprecision pvecnext(niv), pveclast(niv)

  doubleprecision, allocatable :: vfpass(:,:,:)
  doubleprecision dbj, ivpred

  integer nvfit

  real(4) t1, tdelta

  external DRNUN, DRNNOA, utildisc, utilqf

  dbj = dble(bigj)
  allocate(vfpass(nipts,nrpts,nmulti))

  call newhngrids

  hhlimits(1) = 1
  hhlimits(2) = nhhs

  call getivs(bsavebig(:,:,mod(itnum-1,nsavs)+1),n,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,.true.,0,hhlimits,.true.)

  ! draw hendel nevo inclusive values

  !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(hh,j,sumw,i,k,pveclast,pvecnext,v,l,s2,s3,pvcheck,doloop,hhlim2) NUM_THREADS(nthread)
  !$OMP DO

  do hh=1,nhhs

     doloop = .true.

     do while(doloop)

        do j=1,nrpts
           sumw = 0.d0
           do i=1,min(itnum,nsavs)-1
              do k=1,nrpts
                 pveclast = savedprices(hh,j,:,mod(itnum-1,nsavs)+1)

                 pvecnext = savedprices(hh,k,:,mod(itnum-i-1,nsavs)+1)

                 call ptprobshn(pveclast,pvecnext,wtsavbig(mod(itnum-i-1,nsavs)+1,hh,1,j,k),hh)

                 if(pdrawtype .eq. 1) then
                    v=primpdistp(hh,savedpricesi(hh,k,mod(itnum-i-1,nsavs)+1),1)
                 else
                    v=1.d0
                    do l=1,niv
                       v=v*normpdf(pvecnext(l),impdisthh(1,hh,l),impdisthh(2,hh,l))
                    enddo
                 endif

                 if(v .eq. 0) then
                    print*,'error in weighting for grid construction'
                    print*,hh,j,i,k,pveclast(1),pvecnext(1),wtsavbig(mod(itnum-i-1,nsavs)+1,hh,1,j,k),v
                    stop
                 endif

                 sumw = sumw + wtsavbig(mod(itnum-i-1,nsavs)+1,hh,1,j,k)

                 wtsavbig(mod(itnum-i-1,nsavs)+1,hh,1,j,k) = wtsavbig(mod(itnum-i-1,nsavs)+1,hh,1,j,k)/v
              enddo
           enddo

           if(sumw .eq. 0.d0 .and. ncutiv .gt. 0 .and. itnum .gt. 1) then
              if(hhcutiv(hh) .gt. 0) then
                 hhcutiv(hh) = hhcutiv(hh) - 1
                 hhlim2(1) = hh
                 hhlim2(2) = hh
                 call getivs(bsavebig(:,:,mod(itnum-1,nsavs)+1),n,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,.true.,0,hhlim2,.false.)
                 doloop = .true.
                 exit
              else
                 print*,'error: sum of weights are zero for point',savedprices(hh,j,:,mod(itnum-1,nsavs)+1)
                 do i=1,min(itnum,nsavs)-1
                    do k=1,nrpts
                       do l=1,niv
                          write(*,'(F7.2)',advance='NO') savedprices(hh,k,l,mod(itnum-i-1,nsavs)+1)
                       enddo
                    enddo
                    write(*,*)
                 enddo
                 print*,hh,j
                 print*,ivcoef(1:(hhcutiv(hh)+4),:,:,hh)
                 print*,cutmat(1:2,:,hh)
                 print*,ivsd(:,:,hh)
                 print*,impdisthh(:,:,hh)
                 print*,'transition prob checks'
                 do i=1,ncheck
                    do k=1,ncheck
                       s2 = impdisthh(1,hh,:)+(-1.d0 + 2.d0*dble(i-1)/dble(ncheck-1))*3.d0*impdisthh(2,hh,:)
                       s3 = impdisthh(1,hh,:)+(-1.d0 + 2.d0*dble(j-1)/dble(ncheck-1))*3.d0*impdisthh(2,hh,:)
                       call ptprobshn(s2,s3,pvcheck(i,j),hh,.false.,-1)
                       print*,hh,s2,s3,pvcheck(i,j),pvcheck(i,j) .le. 1e-40
                    enddo
                 enddo
                 stop
              endif
           else
              doloop = .false.
           endif
        enddo

     enddo

  enddo

  !$OMP END DO
  !$OMP END PARALLEL

  cstats = 0.d0

  if(itnum .le. vfitchange) then
     nvfit = maxvfit
  else
     nvfit = 2
  endif

  !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(vfpass,hh,diff,ri,loop1,c,cbig,v1,c2,ck,i,inv1,inv2,st,k,v,s,j,l,ut,x1,x2,p1,p2,util,c1d,c2d,ctemp,erflag) NUM_THREADS(nthread)
  !$OMP DO
  do hh=1,nhhs

     diff = 10.d0

     ri = 1

     loop1 = 1

     do while(ri .lt. nvfit)

        c = bsavebig(:,hh,mod(itnum-1,nsavs)+1)

        call transcoef(n,npbig,c,coefbig,notest,expcoefs,poscoefs,negcoefs,cbig)

        if(ri .eq. 1)then

           !draw errors

           v1 = 1.d0

           c2(:) = bsavebig(:,hh,mod(itnum-1,nsavs)+1)

           ck(1,1:n) = c2(:)

           call getclosest(n,ntil,hh,ck,kernsavhh(:,hh))

           ! compute regression coefficients

           do st=1,nmulti

              do i=1,nipts

                  do k=1,nrpts

                     call getbell(0,0,hh,st,k,i,v,erflag)  !integration happens in this routine

                     vfpass(i,k,st) = v

                  enddo

               enddo

           enddo

        endif

        diff = 0.d0

        do st=1,nmulti

           ctemp = cbig

           if(st .eq. 2 .and. twopoint)  ctemp((numprods+2):(numprods+3)) = 0.d0

           do i=1,nipts

              inv1 = dble(i-1)

              do k=1,nrpts

                 ! do the size utility

                 v = 0.d0

                 do s=1,nmdraws

                    ut = 0.d0

                    do j=1,bigj+1

                       do l=1,niv

                          if(cadmissable(j,l)) then

                             x1 = dble(j-1)

                             p1 = savedprices(hh,k,l,mod(itnum-1,nsavs)+1)*x1/dbj

                             call optcons(x1,inv1,v1(s),p1,k,npbig,ctemp,c1d,util,0,vfpass(:,k,st))

                             ut(2) = ut(2) + exp(util)
                             uttemp(j) = util

                             if(j .eq. 1) ut(1) = util

                             if(ri .eq. nvfit-1) then
                                cstats(i,int(x1)+1,st,k,l,hh) = cstats(i,int(x1)+1,st,k,l,hh) + c1d/dble(nmdraws)
                             endif

                          endif

                       enddo

                    enddo

                    v = v + hhweek(nhhvars-2,1,hh)*log(ut(2))+(1.d0-hhweek(nhhvars-2,1,hh))*ut(1)

                 enddo
                 v = v/dble(nmdraws)

                 vsav(hh,i,k,st,1) = v

                 if(ut(2) .eq. 0.d0) then
                    print*,'0 log(cv) in vf update',hh,i,k
                    stop
                 endif

              enddo

           enddo

        enddo

        ri=ri+1

     enddo
     !convergence, store for next time

     vsav1(mod(itnum-1,nsavs)+1,hh,:,:,:) = vsav(hh,:,:,:,1)

  enddo

  !$OMP END DO
  !$OMP END PARALLEL

end subroutine vfupdate



!Unobserved Coupon Model
!Structural Estimation Version


subroutine hbstruct(np,npbig,coefbig,fixedcoefs,notest,expcoefs,poscoefs,negcoefs,s0)

  !hierarchical Bayes code
  !for static model with coupon use loops

  use params
  implicit none

  integer np, npbig       !number of model parameters. 7 for the quadratic model and 9 for the log model
  doubleprecision coefbig(npbig), negcoefs(npbig), coefout(npbig)
  integer notest(npbig), expcoefs(npbig), poscoefs(npbig), fixedcoefs(npbig)
  integer, parameter :: nconv = 5000   !draws til convergence
  integer, parameter :: nkeep = 20000  !draws to do after convergence
  integer, parameter :: nskip = 10     !keep every nth draw
  integer, parameter :: nprint = 25   !how often to print results
  integer, parameter :: ntypes = 1    !number of different types

  integer, parameter :: savestuff = 100000    !how often do we write out everything
  integer, parameter :: restart = 1       !where are we starting the sequence

  integer iseed                 !random number seed
  integer hhlimits(2)

  doubleprecision bmeans(np),bstd(np),s2,s3,s4
  doubleprecision pvcheck(2,2)

  doubleprecision asave(np,nkeep/nskip)
  doubleprecision bsave(np,nhhs),bsave2(np,nhhs)
  doubleprecision dsave(np,np,nkeep/nskip)
  doubleprecision pindexsav(nkeep/nskip)
  doubleprecision p0save(nkeep/nskip)

  doubleprecision amean(np), dmean(np,np)
  doubleprecision astd(np), dstd(np,np)
  doubleprecision p0, p0mean, p0std
  doubleprecision fpcsave

  doubleprecision bo(np),s0(np)

  doubleprecision rho(6)          !jump values for mh steps
  !1st is heterogeneous coeffs
  !2nd is fixed
  !3rd is initial inv
  !4th is vs
  !5th is coupons
  !6th is price expectations coefs


  integer i, naccept(6), j, hh, iter,k,l,s1,t,m, st

  integer ivtypes(ncutiv+1)

  integer, parameter :: nfixedsav = 20   !how far back we'll go with fixed a/r
  integer nfixedaccept(nfixedsav,2)        !accept/reject for fixed draws
  integer numzeros(2)
  integer tfixedaccept(2)                   !total acceptances so far for fixed params

  character(20) pnamesbig(npbig)   !names of variables - everything
  character(20) pnames(np)   !names of variables - only those we estimate

  doubleprecision x, xmax, u(1)

  real(4) t1, tdelta

  doubleprecision dcond, ivpred

  integer errstatus

  character(10) temp1c, temp2c

  doubleprecision csout(maxinv+bigj+1,2)

  !optimization
  integer visits(wksample)
  doubleprecision b0out(np),f0out,stddev,ivcoefavg(ncutiv+4,niv,niv,3),ivsdavg(niv,niv,3),vprobs(maxpurchases),udummy(wksample)
  doubleprecision v1(niv,niv,nhhs)

  doubleprecision avginv(wkstart+maxpurchases), avgcons(wkstart+maxpurchases)
  doubleprecision avginvbytype(2,wkstart+maxpurchases), avgconsbytype(2,wkstart+maxpurchases)

  doubleprecision mtemp(niv), sdtemp(niv), pvecnext(niv), pveclast(niv)

  doubleprecision avubytype(2,wkstart+maxpurchases,2),avubytype2(2,wkstart+maxpurchases,2,2)

  doubleprecision wtstats(5,2)

  doubleprecision bstats(2,np)  !draws on statistics - mean, sd

  logical bad

  external DRNUN

  j=0
  do i=1,npbig
     if(notest(i) == 0) then
        j=j+1
        bo(j) = coefbig(i)
     endif
  enddo

  allocate(visitsave(wksample,nhhs),vlist(wksample,nhhs),vlisti(wksample,nhhs),lvlisti(nhhs))

  !intialize grid of pms and the household
  !transition probabilities on the price process

  allocate(avutils(wkstart+maxpurchases),avutilsav(wkstart+maxpurchases,nconv+nkeep),avutilhh(wkstart+maxpurchases,nhhs))
  allocate(avutilhh2(wkstart+maxpurchases,nhhs,2))
  allocate(invsave(wkstart+maxpurchases,nhhs),conssave(wkstart+maxpurchases,nhhs))

  write(*,*) 'Drawing Random Grid Points.'

  allocate(savedpricesi(nhhs,nrpts,nsavs))
  allocate(savedprices(nhhs,nrpts,niv,nsavs))
  allocate(wtsavbig(nsavs,nhhs,1,nrpts,nrpts),wtsavhhbig(nsavs,nhhs,wkstart+maxpurchases,nrpts))


  allocate(ain(np))        !parameter means
  allocate(bin(np,nhhs))   !individual level parmaters
  allocate(din(np,np))  !parameter variances

  allocate(aout(np))        !parameter means
  allocate(bout(np,nhhs))   !individual level parmaters
  allocate(dout(np,np))  !parameter variances

  allocate(propv(np,np), propvi(np,np), propvic(np,np))

  allocate(scin(nhhs),scout(nhhs))
  p0 = 0.95
  scin = 1

  allocate(bsavebig(np,nhhs,nsavs))

  ain(1:np) = bo

  do i=1,nhhs
     bin(:,i) = ain
  enddo

  bsavebig = 0.d0
  bsavebig(:,:,1) = bin

  din = 0.d0
  dout = 0.d0

  j=0
  do i=1,npbig

     if(notest(i) == 0) then

        j=j+1
        if (fixedcoefs(i) .eq. 0) then
           din(j,j) = 1.d0
        endif
     endif

  enddo

  !starting initial inventories

  allocate(initialinv(numprods,nhhs))

  do i=1,nhhs

     call DRNUN(1,u)

     s2 = 1.d0

     initialinv(1,i) = 0

  enddo

  !starting vs

  allocate(verrorsav(numprods,maxpurchases,nhhs),verrorsavold(numprods,maxpurchases,nhhs))

  verrorsav = 1.d0

  write(*,*) 'Beginning MCMC Loops.'
  allocate(vsav(nhhs,nipts,nrpts,nmulti,1))
  allocate(vsav1(nsavs,nhhs,nipts,nrpts,nmulti))
  allocate(vfupdtemp(nipts,nrpts))
  allocate(ivcoef(ncutiv+4,niv,niv,nhhs),cutmat(ncutiv+1,niv,nhhs),ivsd(niv,niv,nhhs),ivweek(wkstart+maxpurchases,nhhs,niv),ivsave(wkstart+maxpurchases,nhhs,niv))
  allocate(dsighh(nhhs))
  allocate(cstats(nipts,bigj+1,nmulti,nrpts,niv,nhhs))

  allocate(probsave(nhhs))


  allocate(kernsavhh(nsavs,nhhs))

  vsav = 0.d0
  vfupdtemp = 0.d0
  vsav1 = 0.d0

  allocate(acmat(2,nhhs))
  allocate(acmatbig(nhhs,wkstart+maxpurchases-wksample,nipts))
  call hhaccesscount(j)

  write(*,*) 'number of accesses:',j

  allocate(vfsav(2,j))

  allocate(vfupd(nhhs))

  allocate(iterarray(nhhs))

  ! check inclusive value coefficient stability

  itnum = 1
  do hh=1,nhhs
     call DRNUN(wksample,udummy)

     s1=0.d0
     k=1
     do while(hhstorevisits(1,k,hh) .gt. 0)
        s1=s1+dble(hhstorevisits(2,k,hh))
        k=k+1
     enddo
     k=k-1
     vprobs(1:k) = dble(hhstorevisits(2,1:k,hh))/s1

     do i=2,k
        vprobs(i) = vprobs(i) + vprobs(i-1)
     enddo

     do i=1,wksample

        !if we don't see a store visit, draw it out
        if(initweek(sthhind,i,hh) .gt. 0) then
           visits(i) = int(initweek(sthhind,i,hh))
        else
           call DRNUN(1,u)
           if(u(1) .lt. hhweek(nhhvars-2,1,hh) .or. i .eq. 1) then
              do j=1,k
                 if(udummy(i) .lt. vprobs(j)) then
                    visits(i) = hhstorevisits(1,j,hh)
                    exit
                 endif
              enddo
           else
              visits(i) = -abs(visits(i-1))
           endif
        endif

     enddo

     visitsave(:,hh) = visits

  enddo

  allocate(ivinclude(nhhs,niv,niv),hhcutiv(nhhs))

  ivinclude = .false.

  if(niv .eq. 5) then
     do i=1,niv
        ivinclude(:,i,i) = .true.
     enddo
     ivinclude(:,1,5) = .true.
     ivinclude(:,1,2) = .true.
     ivinclude(:,5,1) = .true.
     ivinclude(:,5,2) = .true.
     do i=2,4
        ivinclude(:,i,1) = .true.
        ivinclude(:,i,5) = .true.
     enddo
  else if (niv .eq. 4) then
     if(ncutiv .eq. 0) then
        do i=1,niv
           ivinclude(:,i,i) = .true.
        enddo
        ivinclude(:,1,2) = .true.
        do i=2,4
           ivinclude(:,i,1) = .true.
        enddo
     else
        do i=1,niv
           ivinclude(:,i,i) = .true.
        enddo
     endif
  endif

  if(ncutiv .eq. 0) then
     hhcutiv = ncutiv
  else
     hhcutiv = ncutiv
     hhlimits(1) = 1
     hhlimits(2) = nhhs
     call getivs(bin,np,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,.false.,0,hhlimits,.true.)
     do hh=1,nhhs
        if(ncutiv .ge. 1) then
           bad = .true.
           do i=1,2
             do j=1,2
               pveclast = impdisthh(1,hh,:)+dble(3-2*i)*3.d0*impdisthh(2,hh,:)
               pvecnext = impdisthh(1,hh,:)+dble(3-2*j)*3.d0*impdisthh(2,hh,:)
               call ptprobshn(pvecnext,pveclast,pvcheck(i,j),hh,.false.,-1)
               bad = bad .and. pvcheck(i,j) .eq. 0
             enddo
           enddo
           if(bad) then
              hhcutiv(hh) = ncutiv-1
              print*,'less iv coefficients for hh',hh
           endif
        endif
     enddo
  endif


  iterarray = 1

  bsave = 0.d0
  bsave2 = 0.d0

  nfixedaccept = 0

  rho = 0.15

  rho(2) = 5.0e-3

  rho(3) = 0.5d0

  rho(6) = 5.0e-3

  numzeros = 0

  naccept = 0

  if(restart .eq. 1) then

    open(unit=25,file=pathname//prodname//'/hb/alldraws_a.dat')  !this won't work if we restart, but restart is broken on windows anyway for now...
    open(unit=26,file=pathname//prodname//'/hb/alldraws_d.dat')
    open(unit=27,file=pathname//prodname//'/hb/alldraws_i.dat')
    open(unit=28,file=pathname//prodname//'/hb/alldraws_p.dat')
    open(unit=29,file=pathname//prodname//'/hb/alldraws_c.dat')
    open(unit=30,file=pathname//prodname//'/hb/alldraws_u.dat')
    open(unit=31,file=pathname//prodname//'/hb/alldraws_ll.dat')
    open(unit=32,file=pathname//prodname//'/hb/alldraws_u_types.dat')
    open(unit=33,file=pathname//prodname//'/hb/alldraws_types.dat')
    open(unit=34,file=pathname//prodname//'/hb/alldraws_i_types.dat')
    open(unit=35,file=pathname//prodname//'/hb/alldraws_c_types.dat')
    open(unit=36,file=pathname//prodname//'/hb/alldraws_futil_types.dat')
    open(unit=37,file=pathname//prodname//'/hb/alldraws_ivutil_types.dat')
    open(unit=38,file=pathname//prodname//'/hb/alldraws_bmeans.dat')
    open(unit=39,file=pathname//prodname//'/hb/alldraws_bvar.dat')

  else

    open(unit=25,file=pathname//prodname//'/hb/alldraws_a.dat',position='APPEND')  !this won't work if we restart, but restart is broken on windows anyway for now...
    open(unit=26,file=pathname//prodname//'/hb/alldraws_d.dat',position='APPEND')
    open(unit=27,file=pathname//prodname//'/hb/alldraws_i.dat',position='APPEND')
    open(unit=28,file=pathname//prodname//'/hb/alldraws_p.dat',position='APPEND')
    open(unit=29,file=pathname//prodname//'/hb/alldraws_c.dat',position='APPEND')
    open(unit=30,file=pathname//prodname//'/hb/alldraws_u.dat',position='APPEND')
    open(unit=31,file=pathname//prodname//'/hb/alldraws_ll.dat',position='APPEND')
    open(unit=32,file=pathname//prodname//'/hb/alldraws_u_types.dat',position='APPEND')
    open(unit=33,file=pathname//prodname//'/hb/alldraws_types.dat',position='APPEND')
    open(unit=34,file=pathname//prodname//'/hb/alldraws_i_types.dat',position='APPEND')
    open(unit=35,file=pathname//prodname//'/hb/alldraws_c_types.dat',position='APPEND')
    open(unit=36,file=pathname//prodname//'/hb/alldraws_futil_types.dat',position='APPEND')
    open(unit=37,file=pathname//prodname//'/hb/alldraws_ivutil_types.dat',position='APPEND')
    open(unit=38,file=pathname//prodname//'/hb/alldraws_bmeans.dat',position='APPEND')
    open(unit=39,file=pathname//prodname//'/hb/alldraws_bvar.dat',position='APPEND')

  endif


  if (restart .gt. 1) then

     !read in the file if we're restarting a crashed run

     write(*,*) 'Restarting at point', restart

     write(*,*) 'Reading in from restart file'

     open(unit=58,file=pathname//prodname//'/hb/restartfile1.dat')

     read(58,*) vsav

     close(unit=58,status = 'keep')

     open(unit=58,file=pathname//prodname//'/hb/restartfile1.5.dat')

     do j=1,nsavs
       read(58,*) vsav1(j,:,:,:,:)
     enddo

     close(unit=58,status = 'keep')

     open(unit=58,file=pathname//prodname//'/hb/restartfile2.dat')

     read(58,*) ain
     read(58,*) bin
     read(58,*) din

     read(58,*) asave
     read(58,*) bsave
     read(58,*) dsave

     close(unit=58,status = 'keep')

     open(unit=58,file=pathname//prodname//'/hb/restartfile3.dat')

     read(58,*) bsavebig

     close(unit=58,status = 'keep')

     open(unit=58,file=pathname//prodname//'/hb/restartfile4.dat')

     read(58,*) vfsav
     read(58,*) vfupd
     read(58,*) iterarray

     close(unit=58,status = 'keep')

     open(unit=58,file=pathname//prodname//'/hb/restartfile5.dat')

     read(58,*) rho
     read(58,*) nfixedaccept,numzeros
     read(58,*) savedpricesi
     read(58,*) wtsavbig
     read(58,*) wtsavhhbig

     read(58,*) vfupdtemp
     read(58,*) initialinv

     read(58,*) rseedsave

     close(unit=58,status='keep')

     call RANDOM_SEED(PUT=rseedsave)

     print*,'Error status:',errstatus

     write(*,*) 'Finished reading restart file'

     open(unit=1,file=pathname//prodname//'/hb/hblog.dat',access='APPEND')

     write(1,*) 'Restart'

     if (i .gt. nconv) then

        write(*,*) 'Reconstructing bigbsaves.dat file.'

        open(unit=2,file=pathname//prodname//'/hb/bigbsaves.dat')

        open(unit=57,file=pathname//prodname//'/hb/bigbsavestemp.dat')

        !since we may have stopped before the end of bigbsavescoup we can't just
        !append to the file

        do i=1,restart-1

           if (i .gt. nconv .and. mod(i,nskip) .eq. 0) then
              do j=1,nhhs
                 read(2,*) bin(:,j)
                 write(57,*) bin(:,j)
              enddo
           endif

        enddo

        close(unit=2,status='keep')

        close(unit=57,status='keep')

        open(unit=57,file=pathname//prodname//'/hb/bigbsavestemp.dat')

        open(unit=2,file=pathname//prodname//'/hb/bigbsaves.dat')

        do i=1,restart-1

           if (i .gt. nconv .and. mod(i,nskip) .eq. 0) then
              do j=1,nhhs
                 read(57,*) bin(:,j)
                 write(2,*) bin(:,j)
              enddo
           endif

        enddo

        close(unit=57,status='DELETE')

     else

        open(unit=2,file=pathname//prodname//'/hb/bigbsaves.dat')

     endif

     write(*,*) 'Starting gibbs sampler.'

  else

     open(unit=1,file=pathname//prodname//'/hb/hblog.dat')

     open(unit=2,file=pathname//prodname//'/hb/bigbsaves.dat')

     open(unit=3,file=pathname//prodname//'/hb/vffile.dat')

  endif

  pnames = ' '
  pnamesbig = ' '

  open(unit=101,file=pathname//prodname//'/upcnames.txt')

  do i=1,numprods
     read(101,*) pnamesbig(i)
  enddo

  close(unit=101,status='keep')

  pnamesbig(numprods+1) = 'Consump'
  pnamesbig(numprods+2) = 'Inv0'
  pnamesbig(numprods+3) = 'Inv1'
  pnamesbig(numprods+4) = 'Price'
  pnamesbig(numprods+5) = 'Feature'
  pnamesbig(numprods+6) = 'Display'
  pnamesbig(numprods+7) = 'CC'

  k=0
  do i=1,npbig
     if(notest(i) .eq. 0) then
        k=k+1
        pnames(k) = pnamesbig(i)
     endif
  enddo

  open(unit=4,file=pathname//prodname//'/hb/vfsavfile.txt')

  t1 = secnds(0.0)

  tfixedaccept = 0

  do i=restart,nconv+nkeep

     t1 = secnds(0.0)

     itnum = i

     if(allfixed .eq. 0) then
        call drawbstr(np,naccept(1),rho(1),npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,fixedcoefs)
        bin = bout

        call drawast(np,bo,s0,npbig,fixedcoefs,notest)

        ain = aout

        if(diagvar) then
           call drawdconststr(np,3,1.d0,npbig,fixedcoefs,notest)
        else
           call drawdconststr(np,np+3,1.d0,npbig,fixedcoefs,notest)
        endif

        din = dout

     endif

     if(allrandom .eq. 0) then

        call drawbfixed(np,naccept(2),rho(2),bo,s0,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,fixedcoefs)

        tfixedaccept(1) = tfixedaccept(1) + naccept(2)

        bin = bout
     endif

     if(allrandom .eq. 0) then

        k=0
        do j=1,npbig
           if(notest(j) == 0) then
              k=k+1
              if (fixedcoefs(j) .eq. 1) ain(k) = bin(k,1)
           endif
        enddo

        if (i .gt. nfixedsav) then

           nfixedaccept(1:nfixedsav-1,1) = nfixedaccept(2:nfixedsav,1)

           nfixedaccept(nfixedsav,1) = naccept(2)

           if (mod(i,nfixedsav) .eq. 0) then

              naccept(2) = 0

              do j=1,nfixedsav

                 naccept(2) = naccept(2) + nfixedaccept(j,1)

              enddo

              if (dble(naccept(2))/dble(nfixedsav) .lt. 0.3) then

                 if(.true. .eqv. .false.) then
                    if (rho(2) - 1.0e-3 .gt. 0.d0) then
                       rho(2) = rho(2) - 1.0e-3
                    else
                       if (rho(2) .gt. 1.0e-10) rho(2) = 0.1d0*rho(2)
                    endif

                    if (naccept(2) .eq. 0) then
                       numzeros(1) = numzeros(1) + 1
                       if (numzeros(1) .ge. 20) then
                          numzeros(1) = 0
                          rho(2) = 5.0e-3
                       endif
                    else
                       numzeros(1) = 0
                    endif
                 else
                    rho(2) = rho(2) - 0.1d0*rho(2)
                 endif

              else

                 if(.true. .eqv. .false.) then
                    if (rho(2) - 1.0e-3 .gt. 0.d0) then
                       rho(2) = rho(2) + 1.0e-3
                    else
                       rho(2) = 10.d0*rho(2)
                    endif
                 else
                    rho(2) = rho(2) + 0.1d0*rho(2)
                 endif

              endif

           endif

        else

           nfixedaccept(i,1) = naccept(2)

        endif

     endif

     if(sctypes .or. twopoint) then
        call drawsc(np,1.d0,1.d0,p0,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,fixedcoefs)
     endif

     tfixedaccept(2) = tfixedaccept(2) + naccept(6)

     if (i .gt. nfixedsav) then

        nfixedaccept(1:nfixedsav-1,2) = nfixedaccept(2:nfixedsav,2)

        nfixedaccept(nfixedsav,2) = naccept(6)

        if (mod(i,nfixedsav) .eq. 0) then

           naccept(6) = 0

           do j=1,nfixedsav

              naccept(6) = naccept(6) + nfixedaccept(j,2)

           enddo

           if (dble(naccept(6))/dble(nfixedsav) .lt. 0.3) then

              if (rho(6) - 1.0e-3 .gt. 0.d0) then
                rho(6) = rho(6) - 1.0e-3
              else
                if (rho(6) .gt. 1.0e-10) rho(6) = 0.1d0*rho(6)
              endif

              if (naccept(6) .eq. 0) then
                numzeros(2) = numzeros(2) + 1
                if (numzeros(2) .ge. 20) then
                    numzeros(2) = 0
                    rho(6) = 5.0e-3
                endif
              else
                numzeros(2) = 0
              endif

           else

              if (rho(6) - 1.0e-3 .gt. 0.d0) then
                rho(6) = rho(6) + 1.0e-3
              else
                rho(6) = 10.d0*rho(6)
              endif

           endif

        endif

     else

        nfixedaccept(i,2) = naccept(6)

     endif

     avutilsav(:,itnum) = avutils

     call vfupdate(np,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs)

     write(25,'(I10)') i
     do j=1,np
       write(25,'(F20.10)',advance='NO') ain(j)
     enddo
     write(25,*)

     write(26,'(I10)') i
     do j=1,np
       write(26,'(F20.10)',advance='NO') din(j,j)
     enddo
     write(26,*)

     avginv = 0.d0

     do hh=1,nhhs
       avginv = avginv + invsave(:,hh)/dble(nhhs)
     enddo

     write(27,'(I10)') i
     do j=1,wkstart+maxpurchases
        write(27,'(F20.10)',advance='NO') avginv(j)
     enddo
     write(27,*)

     if(sctypes .or. twopoint) then
       write(28,'(I10)') i
       write(28,'(F20.10)',advance='NO') p0
       write(28,*)
     endif

     avgcons = 0.d0

     do hh=1,nhhs
       avgcons = avgcons + conssave(:,hh)/dble(nhhs)
     enddo

     write(29,'(I10)') i
     do j=1,wkstart+maxpurchases
        write(29,'(F20.10)',advance='NO') avgcons(j)
     enddo
     write(29,*)

     write(30,'(I10)') i
     do j=1,wkstart+maxpurchases
       write(30,'(F20.10)',advance='NO') avutilsav(j,itnum)
     enddo
     write(30,*)

     if(twopoint .or. sctypes) then

        avubytype = 0
        avubytype2 = 0

        do hh=1,nhhs
           do j=1,wkstart+maxpurchases
              avubytype(scin(hh)+1,j,1) = avubytype(scin(hh)+1,j,1) + avutilhh(j,hh)
              avubytype(scin(hh)+1,j,2) = avubytype(scin(hh)+1,j,2) + 1.d0
              avubytype2(scin(hh)+1,j,1,1) = avubytype2(scin(hh)+1,j,1,1) + avutilhh2(j,hh,1)
              avubytype2(scin(hh)+1,j,2,1) = avubytype2(scin(hh)+1,j,2,1) + avutilhh2(j,hh,2)
           enddo
        enddo

        avubytype(:,:,1) = avubytype(:,:,1)/avubytype(:,:,2)
        avubytype2(:,:,1,1) = avubytype2(:,:,1,1)/avubytype(:,:,2)
        avubytype2(:,:,2,1) = avubytype2(:,:,2,1)/avubytype(:,:,2)

        write(32,'(I10)') i
        write(36,'(I10)') i
        write(37,'(I10)') i
        do k=1,2
           do j=1,wkstart+maxpurchases
              write(32,'(F20.10)',advance='NO') avubytype(k,j,1)
              write(36,'(F20.10)',advance='NO') avubytype2(k,j,1,1)
              write(37,'(F20.10)',advance='NO') avubytype2(k,j,2,1)
           enddo
        enddo
        write(32,*)
        write(36,*)
        write(37,*)

        avginvbytype = 0.d0

        do hh=1,nhhs
           do j=1,wkstart+maxpurchases
              avginvbytype(scin(hh)+1,j) = avginvbytype(scin(hh)+1,j) + invsave(j,hh)
           enddo
        enddo
        avginvbytype = avginvbytype/avubytype(:,:,2)

        write(34,'(I10)') i
        do l=1,2
           do j=1,wkstart+maxpurchases
              write(34,'(F20.10)',advance='NO') avginvbytype(l,j)
           enddo
        enddo
        write(34,*)

        avgconsbytype = 0.d0

        do hh=1,nhhs
           do j=1,wkstart+maxpurchases
              avgconsbytype(scin(hh)+1,j) = avgconsbytype(scin(hh)+1,j) + conssave(j,hh)
           enddo
        enddo
        avgconsbytype = avgconsbytype/avubytype(:,:,2)

        write(35,'(I10)') i
        do l=1,2
           do j=1,wkstart+maxpurchases
              write(35,'(F20.10)',advance='NO') avgconsbytype(l,j)
           enddo
        enddo
        write(35,*)

     endif

     s2 = 0.d0
     do j=1,nhhs
        s2=s2+probsave(j)
     enddo
     write(31,*) s2

     write(33,'(I10)') i
     do hh=1,nhhs
       write(33,'(I3)',advance='NO') scin(hh)
     enddo
     write(33,*)

     xmax = 0

     bmeans = 0.d0
     bstats(2,:) = 0.d0
     do hh=1,nhhs
        call transcoef(np,npbig,bin(:,hh),coefbig,notest,expcoefs,poscoefs,negcoefs,coefout)
        j=0
        do k=1,npbig
           if(notest(k) == 0) then
              j=j+1
              bmeans(j) = bmeans(j) + coefout(k)
              bstats(2,j) = bstats(2,j) + coefout(k)*coefout(k)
              if (i .gt. nconv) bsave(j,hh) = bsave(j,hh) + coefout(k)
           endif
        enddo
     enddo

     bstats(2,:) = ( dble(nhhs)*bstats(2,:) - bmeans*bmeans )/dble(nhhs*(nhhs-1))
     bmeans = bmeans/dble(nhhs)
     bstats(1,:) = bmeans

     write(38,'(I10)') i
     do j=1,np
        write(38,'(F20.10)',advance='NO') bstats(1,j)
     enddo
     write(38,*)

     write(39,'(I10)') i
     do j=1,np
        write(39,'(F20.10)',advance='NO') bstats(2,j)
     enddo
     write(39,*)

     if (mod(i,nprint) .eq. 0) then
        write(*,*)
        write(*,*) 'Information for Iteration',i
        write(*,'(A10)',advance='NO') ' '
        write(*,'(3A15)',advance='YES') 'Mean','Variance','Pop Mean'
        do j=1,np
          write(*,'(A10)',advance='NO') pnames(j)
          write(*,'(3F15.5)',advance='YES') ain(j),din(j,j),bmeans(j)
        enddo
        if(sctypes .or. twopoint) then
           s2 = 0.d0
           do j=1,nhhs
              s2=s2+scin(j)/dble(nhhs)
           enddo
           write(*,*) 'Fraction of 0 storage cost hhs',s2,p0
        endif
        !stop
        s2 = 0.d0
        do j=1,nhhs
           s2=s2+probsave(j)
        enddo
        write(*,*) 'log-likelihood at last draw:',s2
        if(allrandom .eq. 0) then
          naccept(2) = 0
          do j=1,nfixedsav
             naccept(2) = naccept(2) + nfixedaccept(j,1)
          enddo
        endif
        naccept(6) = 0
        do j=1,nfixedsav
           naccept(6) = naccept(6) + nfixedaccept(j,2)
        enddo
        write(*,'(A14)',advance='NO') '# acceptances:'
        write(*,'(6I5)',advance='YES') naccept
        write(*,'(A4)',advance='NO') 'Rho:'
        write(*,'(6F7.4)',advance='YES') rho
        write(*,*) 'frac fixed accept:',tfixedaccept,dble(tfixedaccept)/dble(i)
        write(*,*) 'numzeros',numzeros
        write(*,*) 'Avg Value Functions:'
        !val function stats
        do st=1,nmulti
           do j=1,nipts
              if(j .eq. 1 .or. j .eq. 10 .or. j .eq. 21) then
                 s2=0.d0
                 do k=1,nrpts
                    do hh=1,nhhs
                       s2=s2+vsav(hh,j,k,st,1)
                    enddo
                 enddo
                 s2 = s2/dble(nhhs*nrpts)
                 write(*,'(1I3)',advance='NO') j
                 write(*,'(F7.2)',advance='NO') s2
              endif
           enddo
           write(*,*)
        enddo
        s2=0.d0
        do j=1,maxpurchases
          s2=s2+avutilsav(wkstart+j,itnum)/dble(maxpurchases)
        enddo
        write(*,*) 'Average Flow Utilities',s2
        write(*,*)

        write(*,*) 'Statistics on importance weights'
        wtstats = 0.d0
        wtstats(3,1) = -1e10
        wtstats(4,1) = 1e10
        do hh=1,nhhs
           do j=1,nrpts
              do m=1,min(itnum,nsavs)-1
                 do k=1,nrpts
                    s2 = wtsavbig(mod(itnum-m-1,nsavs)+1,hh,1,j,k)
                    wtstats(1,2) = wtstats(1,2) + 1.d0
                    wtstats(1,1) = wtstats(1,1) + s2
                    wtstats(2,1) = wtstats(2,1) + s2**2
                    if(s2 .gt. wtstats(3,1)) wtstats(3,1) = s2
                    if(s2 .lt. wtstats(4,1)) wtstats(4,1) = s2
                    if(s2 .eq. 0.d0) wtstats(5,1) = wtstats(5,1) + 1.d0
                 enddo
              enddo
           enddo
        enddo
        s2 = wtstats(1,2)
        write(*,*) 'mean, sd, max, min, frac0',wtstats(1,1)/s2,sqrt(s2*wtstats(2,1)-wtstats(1,1)**2)/s2,wtstats(3:4,1),wtstats(5,1)/wtstats(1,2)
        write(*,*)

        mtemp = 0.d0
        sdtemp = 0.d0
        do j=1,wkstart+maxpurchases
           do hh=1,nhhs
              mtemp = mtemp + ivweek(j,hh,:)
              sdtemp = sdtemp + ivweek(j,hh,:)*ivweek(j,hh,:)
           enddo
        enddo
        write(*,'(A45)',advance='YES') 'Inclusive value mean, s.d., importance dist'
        do k=1,niv
           write(*,'(F7.3)',advance='NO') mtemp(k)/dble((wkstart+maxpurchases)*nhhs)
        enddo
        write(*,*)
        do k=1,niv
           write(*,'(F7.3)',advance='NO') sqrt( (dble((wkstart+maxpurchases)*nhhs)*sdtemp(k)-mtemp(k)*mtemp(k))/(dble((wkstart+maxpurchases)*nhhs)*dble((wkstart+maxpurchases)*nhhs-1))  )
        enddo
        write(*,*)
        do k=1,niv
           write(*,'(F7.3)',advance='NO') impdist(1,k)
        enddo
        write(*,*)
        do k=1,niv
           write(*,'(F7.3)',advance='NO') impdist(2,k)
        enddo
        write(*,*)
        do hh=1,nhhs
           call invertmat(ivsd(:,:,hh),v1(:,:,hh),niv)
        enddo
        ivcoefavg = 0.d0
        ivsdavg = 0.d0
        do l=1,niv
           do k=1,niv
              do hh=1,nhhs
                 ivcoefavg(:,l,k,1) = ivcoefavg(:,l,k,1) + ivcoef(:,l,k,hh)/dble(nhhs)
                 ivsdavg(l,k,1) = ivsdavg(l,k,1) + v1(l,k,hh)/dble(nhhs)
              enddo
              call pmin(ivsd(l,k,:),nhhs,ivsdavg(l,k,2),hh)
              call pmax(ivsd(l,k,:),nhhs,ivsdavg(l,k,3),hh)
              do j=1,ncutiv+4
                 call pmin(ivcoef(j,l,k,:),nhhs,ivcoefavg(j,l,k,2),hh)
                 call pmax(ivcoef(j,l,k,:),nhhs,ivcoefavg(j,l,k,3),hh)
              enddo
           enddo
        enddo
        write(*,*) 'iv coefficients'
        do l=1,niv
           write(*,'(I2)',advance='NO') l
           do k=1,3
              if(k .eq. 1) then
                 write(*,'(A5)',advance='NO') 'avg:'
              else if (k .eq. 2) then
                 write(*,'(A7)',advance='NO') 'min:'
              else
                 write(*,'(A7)',advance='NO') 'max:'
              endif
              if(ncutiv .eq. 0) then
                 write(*,'(F7.2)',advance='NO') ivcoefavg(1,1,l,k)
                 do hh=1,niv
                    if(ivinclude(1,l,hh)) then
                       write(*,'(F7.2)',advance='NO') ivcoefavg(2,hh,l,k)
                    endif
                 enddo
              else
                 do hh=1,niv
                    if(ivinclude(1,l,hh)) then
                       do m=1,ncutiv+4
                          write(*,'(F10.2)',advance='NO') ivcoefavg(m,hh,l,k)
                       enddo
                    endif
                 enddo
              endif
              write(*,*)
           enddo
        enddo
        write(*,*) 'iv Variance Matrix'
        do j=1,niv
           do k=1,niv
              write(*,'(F10.2)',advance='NO') ivsdavg(j,k,1)
           enddo
           write(*,*)
        enddo

        ivtypes = 0
        do hh=1,nhhs
           ivtypes(hhcutiv(hh)+1) = ivtypes(hhcutiv(hh)+1) + 1
        enddo
        write(*,*) 'Fraction of individuals at each level'
        do k=1,ncutiv+1
           write(*,'(I3)',advance='NO') k-1
           write(*,'(A1)',advance='NO') ':'
           write(*,'(F7.2)',advance='NO') dble(ivtypes(k))/dble(nhhs)
        enddo
        write(*,*)

        tdelta = secnds(t1)
        write(*,*) 'Elapsed Seconds:',tdelta

        write(1,*)
        write(1,*) 'Information for Iteration',i
        write(1,'(A10)',advance='NO') ' '
        write(1,'(3A20)',advance='YES') 'Mean','Variance','Pop Mean'
        do j=1,np
           write(1,'(A10)',advance='NO') pnames(j)
           write(1,'(3F20.10)',advance='YES') ain(j),din(j,j),bmeans(j)
        enddo
        if(.not. diagvar) then
           write(1,*)
           write(1,*) 'Variance Matrix'
           do j=1,np
              do k=1,np
                 if(k .lt. np) then
                    write(1,'(F20.10)',advance='NO') din(j,k)
                 else
                    write(1,'(F20.10)',advance='YES') din(j,k)
                 endif
              enddo
           enddo
           write(1,*)
        endif
        if(sctypes .or. twopoint) then
           s2 = 0.d0
           do j=1,nhhs
              s2=s2+scin(j)/dble(nhhs)
           enddo
           write(1,*) 'Fraction of 0 storage cost hhs',s2,p0
        endif
        s2 = 0.d0
        do j=1,nhhs
           s2=s2+probsave(j)
        enddo
        write(1,*) 'log-likelihood at last draw:',s2
        naccept(2) = 0
        do j=1,nfixedsav
           naccept(2) = naccept(2) + nfixedaccept(j,1)
        enddo
        write(1,'(A14)',advance='NO') '# acceptances:'
        write(1,'(6I5)',advance='YES') naccept
        write(1,'(A4)',advance='NO') 'Rho:'
        write(1,'(6F7.4)',advance='YES') rho
        write(1,*) 'frac fixed accept:',tfixedaccept,dble(tfixedaccept)/dble(i)
        write(1,*) 'numzeros',numzeros
        write(1,*) 'Initial Inventories:'
        j=1
        do k=0,maxinv
           l=0
           do hh=1,nhhs
              if(initialinv(j,hh) == k) l=l+1
           enddo
           write(1,*) 'State:',k,l
        enddo
        write(1,*)
        write(1,*) 'Value Functions'
        !val function stats
        do st=1,nmulti
          do j=1,nipts
             s2=0.d0
             do k=1,nrpts
                do hh=1,nhhs
                   s2=s2+vsav(hh,j,k,st,1)
                enddo
             enddo
             s2 = s2/dble(nhhs*nrpts)
             write(1,'(1I3)',advance='NO') j
             write(1,'(F15.5)',advance='NO') s2
          enddo
        enddo
        write(1,*) 'Consumption from inventory'
        csout = 0.d0
        do st=1,nmulti
           do m=1,nipts
              do k=1,nrpts
                 do j=1,bigj+1
                    t = m+j-1
                    do l=1,niv
                       if(allfixed == 1) then
                          csout(t,2) = csout(t,2)+1
                          csout(t,1) = csout(t,1)+cstats(m,j,st,k,l,1)
                       else
                          do hh=1,nhhs
                             csout(t,2) = csout(t,2)+1
                             csout(t,1) = csout(t,1)+cstats(m,j,st,k,l,hh)
                          enddo
                       endif
                    enddo
                 enddo
              enddo
           enddo
        enddo
        csout(:,1) = csout(:,1)/csout(:,2)
        do t=1,maxinv+bigj+1
           write(1,*) 'Total purchase plus inv level ',t-1,csout(t,1)
        enddo
        write(1,*)
        write(1,*)
        write(1,'(A45)',advance='NO') 'Inclusive value mean, s.d., importance dist'
        do k=1,niv
           write(1,'(F7.3)',advance='NO') mtemp(k)/dble((wkstart+maxpurchases)*nhhs)
        enddo
        write(1,*)
        do k=1,niv
           write(1,'(F7.3)',advance='NO') sqrt( (dble((wkstart+maxpurchases)*nhhs)*sdtemp(k)-mtemp(k)*mtemp(k))/(dble((wkstart+maxpurchases)*nhhs)*dble((wkstart+maxpurchases)*nhhs-1))  )
        enddo
        write(1,*)
        do k=1,niv
           write(1,'(F7.3)',advance='NO') impdist(1,k)
        enddo
        write(1,*)
        do k=1,niv
           write(1,'(F7.3)',advance='NO') impdist(2,k)
        enddo
        write(1,*)

        write(1,*) 'iv coefficients'
        do l=1,niv
           write(1,'(I3)',advance='NO') l
           do k=1,3
              if(k .eq. 1) then
                 write(1,'(A4)',advance='NO') 'avg:'
              else if (k .eq. 2) then
                 write(1,'(A4)',advance='NO') 'min:'
              else
                 write(1,'(A4)',advance='NO') 'max:'
              endif
              do hh=1,niv
                 do j=1,ncutiv+4
                    if(ivcoefavg(j,hh,l,1) .ne. 0.d0) write(1,'(F10.2)',advance='NO') ivcoefavg(j,hh,l,k)
                 enddo
              enddo
              write(1,*)
           enddo
        enddo
        write(1,*) 'iv Variance Matrix'
        do j=1,niv
           do k=1,niv
              write(1,'(F10.2)',advance='NO') ivsdavg(j,k,1)
           enddo
           write(1,*)
        enddo

        write(1,*) 'Fraction of individuals at each level'
        do k=1,ncutiv+1
           write(1,'(I3)',advance='NO') k-1
           write(1,'(A1)',advance='NO') ':'
           write(1,'(F7.2)',advance='NO') dble(ivtypes(k))/dble(nhhs)
        enddo
        write(1,*)
        write(1,*)
        write(1,*)

        !value functions
        if(mod(i,100) .eq. 0) then
           write(4,*)
           write(4,*) 'Iteration',i
           do st=1,nmulti
              do j=1,nipts
                  write(4,'(1I3)') j
                  do k=1,nrpts
                     s2 = 0.d0
                     do hh=1,nhhs
                        s2=s2+vsav(hh,j,k,st,1)
                     enddo
                     s2=s2/dble(nhhs)
                     write(4,'(F7.2)',advance='NO') s2
                  enddo
                  write(4,*)
               enddo
           enddo
        endif

     endif


     if (i .eq. nconv) then
        write(*,*)
        write(*,*) 'Convergence Achieved - Saving Draws'
        write(1,*)
        write(1,*) 'Convergence Achieved - Saving Draws'
     endif

     if (i .gt. nconv .and. mod(i,nskip) .eq. 0) then
        do j=1,nhhs
           write(2,*) bin(:,j)
        enddo

        do j=1,nhhs
           write(2,*) initialinv(:,j)
        enddo

        do j=1,nhhs
           do t=1,maxpurchases
              write(2,*) verrorsav(:,t,j)
           enddo
        enddo

        asave(:,(i-nconv)/nskip) = ain
        dsave(:,:,(i-nconv)/nskip) = din

        if(sctypes .or. twopoint) p0save((i-nconv)/nskip) = p0

     endif

     if (mod(i,savestuff) .eq. 0) then

        !write everything to a file we can reuse

        write(*,*) 'Writing to the restart file'

        open(unit=58,file=pathname//prodname//'/hb/restartfile1.dat')

        write(58,*) vsav

        close(unit=58,status = 'keep')

        open(unit=58,file=pathname//prodname//'/hb/restartfile1.5.dat')

        do j=1,nsavs
          write(58,*) vsav1(j,:,:,:,:)
        enddo

        close(unit=58,status = 'keep')

        open(unit=58,file=pathname//prodname//'/hb/restartfile2.dat')

        write(58,*) ain
        write(58,*) bin
        write(58,*) din

        write(58,*) asave
        write(58,*) bsave
        write(58,*) dsave

        close(unit=58,status = 'keep')

        open(unit=58,file=pathname//prodname//'/hb/restartfile3.dat')

        write(58,*) bsavebig

        close(unit=58,status = 'keep')

        open(unit=58,file=pathname//prodname//'/hb/restartfile4.dat')

        write(58,*) vfsav
        write(58,*) vfupd
        write(58,*) iterarray

        close(unit=58,status = 'keep')

        open(unit=58,file=pathname//prodname//'/hb/restartfile5.dat')

        write(58,*) rho
        write(58,*) nfixedaccept,numzeros

        write(58,*) savedpricesi
        write(58,*) wtsavbig
        write(58,*) wtsavhhbig

        write(58,*) vfupdtemp
        write(58,*) initialinv

        call RANDOM_SEED(GET=rseedsave)

        write(58,*) rseedsave

        close(unit=58,status='keep')

        write(*,*) 'Writing Finished'

     endif

  enddo

  close(unit=3,status='keep')

  close(unit=4,status='keep')

  close(unit=25,status='keep')
  close(unit=26,status='keep')
  close(unit=27,status='keep')
  close(unit=28,status='keep')
  close(unit=29,status='keep')
  close(unit=30,status='keep')
  close(unit=31,status='keep')
  close(unit=32,status='keep')
  close(unit=33,status='keep')
  close(unit=34,status='keep')
  close(unit=35,status='keep')
  close(unit=36,status='keep')
  close(unit=37,status='keep')
  close(unit=38,status='keep')
  close(unit=39,status='keep')


  bsave=bsave/dble(nkeep)
  bsave2=bsave2/dble(nkeep)

  amean = 0.d0
  astd = 0.d0

  dmean = 0.d0
  dstd = 0.d0

  p0mean = 0.d0
  p0std = 0.d0

  do i=1,nkeep/nskip

     amean = amean + asave(:,i)

     dmean = dmean + dsave(:,:,i)

     if(sctypes .or. twopoint) p0mean = p0mean + p0save(i)

  enddo

  amean = amean/dble(nkeep/nskip)

  dmean = dmean/dble(nkeep/nskip)

  if(sctypes .or. twopoint) p0mean = p0mean/dble(nkeep/nskip)

  do i=1,nkeep/nskip

     astd = astd + (asave(:,i) - amean)**2

     dstd = dstd + (dsave(:,:,i) - dmean)**2

     if(sctypes .or. twopoint) p0std = p0std + (p0save(i) - p0mean)**2

  enddo

  astd = sqrt(astd/dble(nkeep/nskip-1))

  dstd = sqrt(dstd/dble(nkeep/nskip-1))

  if(sctypes .or. twopoint) p0std = sqrt(p0std/dble(nkeep/nskip-1))

  bmeans = 0.d0

  do j=1,nhhs
     bmeans = bmeans + bsave(:,j)
  enddo

  bmeans = bmeans/dble(nhhs)

  bstd = 0.d0

  do j=1,nhhs
     bstd = bstd + (bsave(:,j) - bmeans)**2
  enddo

  bstd = sqrt(bstd/dble(nhhs-1))


  write(1,*)
  write(1,*) 'Parameter Means, Variances'


  do i=1,np
     write(1,'(A20)',advance='NO') pnames(i)
     write(1,'(4F20.10)',advance='YES') amean(i),astd(i),dmean(i,i),dstd(i,i)
  enddo

  write(1,*)
  write(1,*) 'Population Parameter Means, Std. Dev'

  do i=1,np
     write(1,'(A20)',advance='NO') pnames(i)
     write(1,'(4F20.10)',advance='YES') bmeans(i),bstd(i)
  enddo

  write(1,*)
  write(1,*) 'Variance Matrix For All Parameters'

  do i=1,np
     do j=1,np-1
        write(1,'(F10.5)',advance='NO') dmean(i,j)
     enddo
     write(1,'(F10.5)',advance='YES') dmean(i,np)
     do j=1,np-1
        write(1,'(A2)',advance='NO') ' ('
        write(1,'(F6.3)',advance='NO') dstd(i,j)
        write(1,'(A2)',advance='NO') ') '
     enddo
     write(1,'(A2)',advance='NO') ' ('
     write(1,'(F6.3)',advance='NO') dstd(i,np)
     write(1,'(A2)',advance='YES') ') '
  enddo

  if(sctypes .or. twopoint) then
     write(1,*) 'Storage cost probability mean, std dev',p0mean,p0std
  endif

  close(unit=1,status='keep')

  open(unit=1,file=pathname//prodname//'/hb/hbmeans.dat')

  do i=1,nhhs
     do j=1,np-1
        write(1,'(F10.3)',advance='NO') bsave(j,i)
     enddo
     write(1,'(F10.3)',advance='YES') bsave(np,i)
  enddo

  close(unit=1,status='keep')

  close(unit=2,status='keep')

  open(unit=1,file=pathname//prodname//'/hb/heirsaves.dat')

  write(1,*) asave
  write(1,*) dsave
  if(sctypes .or. twopoint) write(1,*) p0save

  close(unit=1,status='keep')


  call convertheir(np,nkeep,nskip)


end subroutine hbstruct




!subroutines to draw from distributions


subroutine drawbstr(n,naccept,rho,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,fixedcoefs)

  !draw new household b
  !using MH algorithm and a/r
  !use the logit formula with unobserved coupons
  !structural version - also updates the expected value function
  !this only updates population-varying coefficients


  use params
  implicit none

  integer n, i, naccept, j, iter, k, nphi, hh
  doubleprecision rho

  integer npbig
  doubleprecision coefbig(npbig), negcoefs(npbig)
  integer notest(npbig), expcoefs(npbig), poscoefs(npbig), fixedcoefs(npbig)

  doubleprecision pin(nhhs), pout(nhhs)        !household logit values
  doubleprecision normdraws(n,nhhs), udraws(nhhs), r(1,1)
  integer invnew(numprods,nhhs), invold(numprods,nhhs)

  doubleprecision nidraws(numprods,nhhs)

  doubleprecision r1,sum1

  !for DLFTDS

  doubleprecision dch(n,n)      !cholesky parameter variances
  doubleprecision dinv(n,n)
  doubleprecision amean(n)
  doubleprecision bnew(n,1), bold(n,1)

  doubleprecision, allocatable :: dsmall(:,:), dcsmall(:,:), dismall(:,:), ndsmall(:,:), boutsmall(:,:), binsmall(:,:)
  integer nvarying, i1, i2, j1, j2

  doubleprecision futilhh(wkstart+maxpurchases,nhhs,2), invhh(wkstart+maxpurchases,nhhs), conshh(wkstart+maxpurchases,nhhs)

  real(4) t1,tdelta

  logical baddraw

  external DRNNOA, DRNUN

  call hhlogitstr(n,pin,1,1,1,1,.true.,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,scin)
  futilhh(:,:,1) = avutilhh
  invhh = invsave
  conshh = conssave
  
  amean = 0.d0

  dch = 0.d0
  dinv = 0.d0

  if(diagvar) then

     do i=1,nhhs

        call DRNNOA(n,normdraws(1:n,i))
        call DRNNOA(numprods,nidraws(:,i))

     enddo

     call DRNUN(nhhs,udraws)

     if(allfixed == 0) then

        j=0
        do i=1,npbig
           if(notest(i) == 0) then
              j=j+1
              amean(j) = ain(j)
              if(fixedcoefs(i) .eq. 0) then
                 dch(j,j) = sqrt(din(j,j))
              else
                 normdraws(j,:) = 0.d0
              endif
           endif
        enddo

     endif

     do i=1,nhhs

        bout(:,i) = bin(:,i) + sqrt(rho)*matmul(dch(:,:),normdraws(:,i))

     enddo

  else

     nvarying = 0
     do i=1,npbig
        if(notest(i) .eq. 0 .and. fixedcoefs(i) .eq. 0) nvarying = nvarying + 1
     enddo

     allocate(dsmall(nvarying,nvarying),dcsmall(nvarying,nvarying),dismall(nvarying,nvarying),ndsmall(nhhs,nvarying))
     allocate(boutsmall(nvarying,nhhs),binsmall(nvarying,nhhs))

     i1=0
     i2=0
     do i=1,npbig
        if(notest(i) .eq. 0) i2=i2+1
        if(notest(i) .eq. 0 .and. fixedcoefs(i) .eq. 0) then
           i1=i1+1
           binsmall(i1,:) = bin(i2,:)
           j1=0
           j2=0
           do j=1,npbig
              if(notest(j) .eq. 0) j2=j2+1
              if(notest(j) .eq. 0 .and. fixedcoefs(j) .eq. 0) then
                 j1=j1+1
                 dsmall(i1,j1) = din(i2,j2)
              endif
           enddo
        endif
     enddo

     dcsmall = 0.d0
     dismall = 0.d0

     call chol(dsmall,dcsmall,nvarying)
     call invertmat(dsmall,dismall,nvarying)

     do i=1,nhhs

        call DRNNOA(nvarying,ndsmall(i,:))

     enddo

     call DRNUN(nhhs,udraws)

     boutsmall = binsmall + sqrt(rho)*transpose(matmul(ndsmall,dcsmall))
     bout = bin

     i1=0
     i2=0
     do i=1,npbig
        if(notest(i) .eq. 0) i2=i2+1
        if(notest(i) .eq. 0 .and. fixedcoefs(i) .eq. 0) then
           i1=i1+1
           bout(i2,:) = boutsmall(i1,:)
        endif
     enddo

  endif

  call hhlogitstr(n,pout,0,2,1,1,.false.,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,scin)
  futilhh(:,:,2) = avutilhh

  if(allfixed == 0) then

     if(diagvar) then
        do i=1,n
           if (dch(i,i) .gt. 0) dinv(i,i) = 1.d0/(dch(i,i)**2)
        enddo
     else
        i1=0
        i2=0
        do i=1,npbig
           if(notest(i) .eq. 0) i2=i2+1
           if(notest(i) .eq. 0 .and. fixedcoefs(i) .eq. 0) then
              i1=i1+1
              j1=0
              j2=0
              do j=1,npbig
                 if(notest(j) .eq. 0) j2=j2+1
                 if(notest(j) .eq. 0 .and. fixedcoefs(j) .eq. 0) then
                    j1=j1+1
                    dinv(i2,j2) = dismall(i1,j1)
                 endif
              enddo
           endif
        enddo

     endif

  endif

  naccept = 0

  avutils = 0.d0

  do i=1,nhhs

     bnew(:,1) = bout(:,i) - amean(:)
     bold(:,1) = bin(:,i)  - amean(:)

     baddraw = max(exp(bout(1,i)),exp(bout(2,i))) .gt. 10

     if(.not. baddraw) then

        if(allfixed .eq. 0) then

           r = pout(i) - pin(i) - 0.5d0*(matmul(transpose(bnew),matmul(dinv(:,:),bnew)) &
                & - matmul(transpose(bold),matmul(dinv(:,:),bold)) )

        else

           r = pout(i) - pin(i)

        endif

        if (r(1,1) .lt. 100) then

           r = exp(r)

        else

           r = 10.d0

        endif

     endif

     if ((udraws(i) .gt. r(1,1) .and. r(1,1) .lt. 1.d0) .or. baddraw) then

        !reject - don't keep the updated value function

        initialinv(:,i) = invold(:,i)
        bout(:,i) = bin(:,i)
        vfupd(i) = 0
        vfsav(1,acmat(1,i)+1:acmat(2,i)) = vfsav(2,acmat(1,i)+1:acmat(2,i))
        do j=1,wkstart+maxpurchases
           avutils(j) = avutils(j) + futilhh(j,i,1)/dble(nhhs)
        enddo

        invsave(:,i) = invhh(:,i)
        conssave(:,i) = conshh(:,i)

        if(allrandom .eq. 1) probsave(i) = pin(i)

     else

        !accept - store updated value function and new betas
        !for future value function interpolation

        naccept = naccept + 1
        iterarray(i) = iterarray(i) + 1
        vfupd(i) = 1

        if(allrandom .eq. 1) probsave(i) = pout(i)

        do j=1,wkstart+maxpurchases
           avutils(j) = avutils(j) + futilhh(j,i,2)/dble(nhhs)
        enddo

     endif

  enddo

  if (dble(naccept)/dble(nhhs) .lt. 0.3) then

     rho = rho - 0.1d0*rho

  elseif (dble(naccept)/dble(nhhs) .gt. 0.3) then

     rho = rho + 0.1d0*rho

  endif

  bsavebig(:,:,mod(itnum-1,nsavs)+1) = bout

end subroutine drawbstr



subroutine drawbfixed(n,naccept,rho,bo,s0,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,fixedcoefs)

  !draw new household b
  !using MH algorithm and a/r
  !use the logit formula with unobserved coupons
  !structural version - also updates the expected value function
  !this only updates fixed coefficients


  use params
  implicit none

  integer n, i, naccept, j, iter, k, baddraw, ctype, hh
  doubleprecision rho
  doubleprecision bo(n),s0(n),g(1)

  integer npbig
  doubleprecision coefbig(npbig), negcoefs(npbig)
  integer notest(npbig), expcoefs(npbig), poscoefs(npbig), fixedcoefs(npbig)

  doubleprecision pin(nhhs), pout(nhhs)        !household logit values
  doubleprecision normdraws(n), udraws(1), r(1,1)

  doubleprecision r1,s1

  !for DLFTDS

  doubleprecision dch(n,n)      !cholesky parameter variances
  doubleprecision dinv(n,n), hx, hy
  doubleprecision amean(n),v(2)
  doubleprecision bnew(n,1), bold(n,1)

  doubleprecision vscale(n), factor

  integer, parameter :: proptype = 1
  !proposal type:
  !1 random walk
  !2 multivariate t
  !3 multivariate normal

  integer, parameter :: chinu = 6  !degrees of freedom in t dist
  doubleprecision chidraws(chinu,n), chidenom(n), futilhh(wkstart+maxpurchases,nhhs,2), invhh(wkstart+maxpurchases,nhhs), conshh(wkstart+maxpurchases,nhhs)



  call DRNNOA(n,normdraws)


  call DRNUN(1,udraws)


  dch = 0.d0

  !nothing before item 4 is fixed

  if(proptype == 1) then

     j=0
     do i=1,npbig
        if(notest(i) == 0) then
           j=j+1
           if (fixedcoefs(i) .eq. 1) dch(j,j) = 1.d0
        endif
     enddo

  elseif(proptype == 2) then

     j=0
     do i=1,npbig
        if(notest(i) == 0) then
           j=j+1
           if (fixedcoefs(i) .eq. 1) dch(i,i) = 1.d0
           call DRNNOA(chinu,chidraws(:,j))
           s1 = 0.d0
           do k=1,chinu
              s1 = s1 + chidraws(k,j)*chidraws(k,j)
           enddo
           chidenom(j) = s1/dble(chinu)
        endif
     enddo

  elseif(proptype==3) then

     j=0
     do i=1,npbig
        if(notest(i) == 0) then
           j=j+1
           dch(j,j) = min(propvic(j,j),0.005d0)
           dinv(j,j) = max(propvi(j,j),0.005d0)
           if(fixedcoefs(i) == 0) dch(j,j) = 0.d0
        endif
     enddo

  elseif(proptype==4) then

     j=0
     do i=1,npbig
        if(notest(i) == 0) then
           j=j+1
           if (fixedcoefs(i) .eq. 1) then
              dch(j,j) = 1.d0
              if(i == 1 .or. i == 2) then
                 dch(j,j) = dch(j,j)*exp(2.d0*bin(j,1))
              endif
           endif
        endif
     enddo

  endif

  baddraw = 0

  do i=1,nhhs

     if(proptype == 1) then

        bout(:,i) = bin(:,i) + sqrt(rho)*matmul(dch,normdraws)
        vscale = 1.d0

     elseif(proptype == 2) then

        !this assumes diagonal dch
        factor = 1.107436
        vscale(1) = 0.0176672403
        vscale(2) = vscale(1)
        vscale(3) = 0.0058649666
        vscale(4) = 4.660932e-03
        vscale(5) = 1
        vscale(6) = 1.341993e-02
        vscale(7) = 1.755809e-04
        bout(:,i) = bin(:,i) + matmul(dch,normdraws)*sqrt(vscale)/chidenom

     elseif(proptype == 3) then

        hx = 0.d0
        k=0
        do j=1,npbig
           if(notest(j) == 0) then
              k=k+1
              if(fixedcoefs(j) == 1) then
                 hx = hx + exp(abs(bin(k,i))-5.d0)
              endif
           endif
        enddo

        bout(:,i) = bin(:,i) + hx*matmul(dch,normdraws)*rho

        hy = 0.d0
        k=0
        do j=1,npbig
           if(notest(j) == 0) then
              k=k+1
              if(fixedcoefs(j) == 1) then
                 hy = hy + exp(abs(bout(k,i))-5.d0)
              endif
           endif
        enddo

        k=0
        do j=1,npbig
           if(notest(j) == 0) then
              k=k+1
              if(fixedcoefs(j) == 0) then
                 bout(k,i) = bin(k,i)
              endif
           endif
        enddo

     elseif(proptype == 4) then

        bout(:,i) = bin(:,i) + sqrt(rho)*matmul(dch,normdraws)

        if(max(exp(bout(1,i)),exp(bout(2,i))) .gt. 5) baddraw = 1

     endif

  enddo

  naccept = 0

  r = 0.d0

  if (baddraw .eq. 0) then
     if(allfixed .eq. 0) then
        call hhlogitstr(n,pin,1,1,1,1,.false.,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,scin)

     else
        call hhlogitstr(n,pin,1,1,1,1,.true.,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,scin)
     endif
     futilhh(:,:,1) = avutilhh
     invhh = invsave
     conshh = conssave
     call hhlogitstr(n,pout,0,2,1,1,.false.,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,scin)
     futilhh(:,:,2) = avutilhh

     r=0.d0
     do i=1,nhhs
        r=r+ pin(i)
     enddo

     r=0.d0
     do i=1,nhhs
        r=r+ pout(i)
     enddo

     r = 0.d0

     do i=1,nhhs

        r = r + pout(i) - pin(i)

        if(i .eq. 1) then

           do j=1,n
              if(s0(j) .ne. -1 .and. dch(j,j) .gt. 0) then
                 r = r - 0.5*( (bout(j,i)-bo(j))/s0(j) )**2 + 0.5*( (bin(j,i)-bo(j))/s0(j) )**2
              endif
           enddo

           k=0
           do j=1,npbig
              if(notest(j) == 0) then
                 k=k+1
                 if(fixedcoefs(j) == 0) then
                    bnew(k,1) = 0.d0
                    bold(k,1) = 0.d0
                 else
                    bnew(k,1) = bout(k,i) - bo(i)
                    bold(k,1) = bout(k,i) - bo(i)
                 endif
              endif
           enddo

           if(proptype == 2) then


           elseif(proptype==3) then
              k=0
              do j=1,npbig
                 if(notest(j) == 0) then
                    k=k+1
                    if(fixedcoefs(j) == 1) then
                       r = r + (log(hx)-log(hy)) - 0.5d0*normdraws(k)*normdraws(k)*( (hx*hx)/(hy*hy)-1.d0)
                    endif
                 endif
              enddo

           elseif(proptype==4) then
              r = r + 2.d0*(bout(1,1)-bin(1,1)+bout(2,1)-bin(2,1))
           endif

        endif

     enddo

     if (r(1,1) .lt. 100) then

        r = exp(r)

     else

        r = 10.d0

     endif

  endif

  if ((udraws(1) .gt. r(1,1) .and. r(1,1) .lt. 1.d0) .or. baddraw .eq. 1) then

     !reject - don't keep the updated value function
     !note - don't change vfupd since it may be 1 or 0 from the
     !individual-level mh routine

     do i=1,nhhs
        bout(:,i) = bin(:,i)
        if (baddraw .eq. 0) then
           vfsav(1,acmat(1,i)+1:acmat(2,i)) = vfsav(2,acmat(1,i)+1:acmat(2,i))
	      endif
     enddo
     probsave = pin

     avutils = 0.d0
     do hh=1,nhhs
        do i=1,wkstart+maxpurchases
           avutils(i) = avutils(i) + futilhh(i,hh,1)/dble(nhhs)
        enddo
     enddo

     invsave = invhh
     conssave = conshh

  else

     !accept - store updated value function and new betas
     !for future value function interpolation

     naccept = naccept + 1
     probsave = pout

     avutils = 0.d0
     do hh=1,nhhs
        do i=1,wkstart+maxpurchases
           avutils(i) = avutils(i) + futilhh(i,hh,2)/dble(nhhs)
        enddo
     enddo

  endif

  !update bsavebig

  bsavebig(:,:,mod(itnum-1,nsavs)+1) = bout

end subroutine drawbfixed




! draw a new set of storage costs

subroutine drawsc(n,alpha0,beta0,p0,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,fixedcoefs)

  use params
  implicit none

  integer n, j, iter, k, baddraw, ctype, i, hh
  doubleprecision rho
  doubleprecision bo(n),s0(n),g(1), p0, alpha0, beta0

  integer npbig
  doubleprecision coefbig(npbig), negcoefs(npbig)
  integer notest(npbig), expcoefs(npbig), poscoefs(npbig), fixedcoefs(npbig)

  doubleprecision pin(nhhs), pout(nhhs)        !household logit values
  doubleprecision normdraws(n), udraws(nhhs), r

  doubleprecision r1,s1,alpha1,beta1

  doubleprecision avgp(2)

  integer sctemp(nhhs)


  sctemp = 0

  call hhlogitstr(n,pin,1,1,1,1,.false.,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,sctemp)

  sctemp = 1

  call hhlogitstr(n,pout,1,1,0,1,.false.,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,sctemp)

  call DRNUN(nhhs,udraws)

  alpha1 = alpha0
  beta1 = beta0

  avgp = 0.d0

  do i=1,nhhs
     avgp(1) = avgp(1) + pout(i)
     avgp(2) = avgp(2) + pin(i)
     r = exp(pout(i))*p0/(exp(pout(i))*p0 + exp(pin(i))*(1.d0-p0))
     if(udraws(i) .lt. r) then
        scin(i) = 1
     else
        scin(i) = 0
     endif
     alpha1 = alpha1 + dble(scin(i))
     beta1 = beta1 + dble(1-scin(i))
  enddo

  avgp = avgp/dble(nhhs)

  call hhlogitstr(n,pin,1,1,1,1,.false.,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,scin)
  avutils = 0.d0
  do hh=1,nhhs
     do i=1,wkstart+maxpurchases
        avutils(i) = avutils(i) + avutilhh(i,hh)/dble(nhhs)
     enddo
  enddo

  probsave = pin

  call drnbeta(1,alpha1,beta1,p0)


end subroutine drawsc


subroutine hhaccesscount(accnum)

  !this runs through all houshold purchases and figures out
  !each time we call getbell
  !each of these will be an index where we can store an
  !old getbell

  !accnum2 should only be accessed by brand
  !or the memory requirement will explode

  use params
  implicit none

  integer accnum
  integer hh,i,j,k

  accnum = 0
  acmatbig = 0
  k=0

  do hh=1,nhhs

     acmat(1,hh) = accnum

     do i=wksample+1,wkstart

        if(initweek(nhhvars,i,hh) .eq. 0) exit
        do j=1,nipts
          k=k+1
          acmatbig(hh,i-wksample,j) = k
        enddo

        accnum = accnum + nipts

     enddo

     do i=1,maxpurchases

        if(hhweek(nhhvars,i,hh) .eq. 0) exit
        do j=1,nipts
          k=k+1
          acmatbig(hh,i+wkstart-wksample,j) = k
        enddo

        accnum = accnum + nipts

     enddo

     acmat(2,hh) = accnum

  enddo

end subroutine hhaccesscount


subroutine hhlogitstr(n,p,bothflag,bflag,getweights,getvfs,drawstore,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,scflag)

  !calculate the logit probs
  !household by household
  !keep log sum
  !include the unobserved coupons
  !this structural version also calculates the expected value
  !function using the subroutine getbell

  !we will use the function with counterfactuals as well to compute the coli for tuna

  use params
  implicit none

  integer n
  integer bothflag   !update both els of vfsav
  integer bflag      !are we using bin or bout
  integer getweights     !calculate the price weights
  integer getvfs         !are we calling getbell: don't need to when drawing initial inventories

  integer npbig
  doubleprecision coefbig(npbig), negcoefs(npbig)
  integer notest(npbig), expcoefs(npbig), poscoefs(npbig)

  integer scflag(nhhs), hhlimits(2), hhlim2(2), erflag

  logical doloop

  doubleprecision p(nhhs),c(n),ck(2,n), pcoeff
  doubleprecision cbig(npbig), ctemp(npbig) !cbig(n+3)
  doubleprecision u, v, cv, sumw
  doubleprecision vf,lpd,vfkexp,vfk

  doubleprecision currentp(numprods),inv(numprods),pvecnext(niv),pveclast(niv),vfpass(nipts),upass(2*wkstart,nhhs)

  integer hh,i,j,accnum,inflag,it,k,st,stt(2,numprods),l,ii,lv,abucket,npos,ivchosen

  integer nchoices, pstateind

  doubleprecision utildischh, x1, x2, c1d(bigj+1), c2d, c1c, c2c, normpdf, p1, p2, x1c, x2c, utq, flowcv, utildisc, utl
  doubleprecision futilhh(maxpurchases,nhhs), f2(2), utils(bigj+1), ptemp(3*numprods)

  logical badcdraw, badc, drawstore

  integer visits(wksample)
  doubleprecision vlistp(3*numprods,wksample)
  doubleprecision u1(1), vprobs(maxpurchases), s1, dbj, ubrand
  doubleprecision tunits

  logical pstatefilled(nobspr), found, allzero

  real(4) t1, tdelta

  external utildischh,utildisc



  p = 0.d0

  accnum = 0

  dbj = dble(bigj)

  if(drawstore) then
     vlist = 0
     vlisti = 0
     do hh=1,nhhs
        call DRNUN(2*wksample,upass(:,hh))
        ! here we replicate code so there's some inefficiency

        s1=0.d0
        k=1
        do while(hhstorevisits(1,k,hh) .gt. 0)
           s1=s1+dble(hhstorevisits(2,k,hh))
           k=k+1
        enddo
        k=k-1
        vprobs(1:k) = dble(hhstorevisits(2,1:k,hh))/s1

        do i=2,k
           vprobs(i) = vprobs(i) + vprobs(i-1)
        enddo

        do i=1,wksample

           !if we don't see a store visit, draw it out
           if(initweek(sthhind,i,hh) .gt. 0) then
              visits(i) = int(initweek(sthhind,i,hh))
           else
              ! first, figure out if you visit a store or not
              if(upass(wksample+i,hh) .lt. hhweek(nhhvars-2,1,hh) .or. i .eq. 1) then
                 do j=1,k
                    if(upass(i,hh) .lt. vprobs(j)) then
                       visits(i) = hhstorevisits(1,j,hh)
                       exit
                    endif
                 enddo
              else
                 visits(i) = -abs(visits(i-1))
              endif
           endif

        enddo

        visitsave(:,hh) = visits

        ! index value functions - should still work for hendel nevo style model since everything is a fn of price and this is individual level

        vlistp = 0.d0
        lv = 0
        do i=1,wksample
           ! the search here could be slow - we might want to figure out a way to not search over all prices
           k=1
           j=0
           l = abs(visits(i))
           if(visits(i) .lt. 0) then
              ptemp = 0.d0
              ptemp(1:numprods) = initprice(1:numprods,l,i)
           else
              ptemp = initprice(:,l,i)
           endif
           do while(.not. allzero(vlistp(1:numprods,k),numprods))
              found = .true.
              do ii=1,numprods
                 found = found .and. vlistp(ii,k) .eq. ptemp(ii)
              enddo
              if(usefeatures) then
                 do ii=1,numprods
                    found = found .and. vlistp(ii+numprods,k) .eq. ptemp(ii+numprods)
                 enddo
              endif
              if(usedisplay) then
                 do ii=1,numprods
                    found = found .and. vlistp(ii+2*numprods,k) .eq. ptemp(ii+2*numprods)
                 enddo
              endif
              if(found) then
                 j=k
                 vlist(i,hh) = k
                 exit
              endif
              k=k+1
           enddo
           if(j .eq. 0) then
              vlistp(1:(3*numprods),k) = initprice(1:(3*numprods),l,i)
              lv = lv + 1
              vlist(i,hh) = k
              vlisti(lv,hh) = k
              if(vlisti(lv,hh) .gt. nobspr .or. vlisti(lv,hh) .le. 0) then
                 print*,"vlisti bug",vlisti(lv,hh),k,i,hh
                 print*,"other stuff",vlist(i,hh),visits(i),pptrstatei(l,i),vlistp(:,k)
                 print*,"store prices",sasprices(l,i,:)
                 stop
              endif

           endif

        enddo

        lvlisti(hh) = lv

     enddo
  endif

  hhlimits(1) = 1
  hhlimits(2) = nhhs
  if (bflag .eq. 1) then
     call getivs(bin,n,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,.true.,1,hhlimits,.true.)
  else
     call getivs(bout,n,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,.true.,1,hhlimits,.true.)
  endif

  !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(hh,badc,c,ck,i,cbig,inv,accnum,inflag,nchoices,cv,c1c,c2c,x1,x2,p1,p2,u,v,c1d,c2d,x1c,x2c,vf,flowcv,vfpass,pcoeff,pstatefilled,pstateind,j,k,l,pveclast,pvecnext,ubrand,ctemp,abucket,ivchosen,tunits,npos,hhlim2,erflag,doloop,sumw,utils,f2) NUM_THREADS(nthread)
  !$OMP DO
  do hh=1,nhhs

     !skip if it's a bad draw on initial inventories; this shouldn't happen outside of the loop that draws initial inventories
     !if it does, the access number thing will get screwed up

     badc = .false.

     if(.not. badc) then

        if (bflag .eq. 1) then
           c = bin(:,hh)
        else
           c = bout(:,hh)
        endif

        ck(1,1:n) = c(:)

        call transcoef(n,npbig,c,coefbig,notest,expcoefs,poscoefs,negcoefs,cbig)

        pcoeff = cbig(numprods+4)

        !store value functions

        ! in the hendel nevo model the weights are functions of individual level parameters, so they need to be done a bit differently
        ! remember we need to compute the weights for all price states the household visits
        ! to optimize on speed we can check and only fill the same indexes once

        if(getweights .eq. 1) then

           doloop = .true.

           do while(doloop)

              pstatefilled = .false.

              do j=1,wkstart+maxpurchases

                 if(j .le. wksample) then
                    pstateind = vlist(j,hh)
                 else if (j .le. wkstart) then
                    pstateind = wksample+int(initweek(nhhvars,j,hh))
                    if(pstateind .eq. wksample) then
                       cycle
                    endif
                 else
                    pstateind = wksample+int(hhweek(nhhvars,j-wkstart,hh))
                    if(pstateind .eq. wksample) then
                       doloop = .false.
                       exit
                    endif
                 endif

                 if(pstateind .eq. 0 .or. pstateind .gt. wkstart+maxpurchases) then
                    print*,'error in pstateind',pstateind
                    if(j .le. wksample) then
                       print*,'hh, t',hh,j,vlist(j,hh)
                    else if ( j .le. wkstart) then
                       print*,'hh, t',hh,j,initweek(:,j,hh)
                    else
                       print*,'hh, t',hh,j,hhweek(:,j-wkstart,hh)
                    endif
                    stop
                 endif

                 if(.not. pstatefilled(pstateind)) then

                    pstatefilled(pstateind) = .true.

                    sumw = 0.d0
                    do i=1,min(itnum,nsavs)-1
                       
                       do k=1,nrpts

                          pveclast = ivweek(j,hh,:)

                          pvecnext = savedprices(hh,k,:,mod(itnum-i-1,nsavs)+1)

                          call ptprobshn(pveclast,pvecnext,wtsavhhbig(mod(itnum-i-1,nsavs)+1,hh,pstateind,k),hh)

                          if(pdrawtype .eq. 1) then
                             v=primpdistp(hh,savedpricesi(hh,k,mod(itnum-i-1,nsavs)+1),1)
                          else
                             v=1.d0
                             do l=1,niv
                                v=v*normpdf(pvecnext(l),impdisthh(1,hh,l),impdisthh(2,hh,l))
                             enddo
                          endif

                          sumw = sumw + wtsavhhbig(mod(itnum-i-1,nsavs)+1,hh,pstateind,k)

                          wtsavhhbig(mod(itnum-i-1,nsavs)+1,hh,pstateind,k) = wtsavhhbig(mod(itnum-i-1,nsavs)+1,hh,pstateind,k)/v

                       enddo

                    enddo

                    if(sumw .eq. 0.d0 .and. hhcutiv(hh) .gt. 0 .and. itnum .gt. 1) then
                       hhcutiv(hh) = hhcutiv(hh) - 1
                       hhlim2(1) = hh
                       hhlim2(2) = hh
                       if (bflag .eq. 1) then
                          call getivs(bin,n,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,.true.,1,hhlim2,.false.)
                       else
                          call getivs(bout,n,npbig,coefbig,notest,expcoefs,poscoefs,negcoefs,.true.,1,hhlim2,.false.)
                       endif
                       doloop = .true.
                       exit
                    else if(sumw .eq. 0.d0 .and. hhcutiv(hh) .eq. 0 .and. itnum .gt. 1) then
                       print*,'sum of weights are 0 in hhlogitcoupstr'
                       stop
                    else if(sumw .gt. 0.d0 .or. itnum .eq. 1) then
                       doloop = .false.
                    endif

                 endif

              enddo

           enddo

        endif

        if(getvfs .eq. 1) then

           call getclosest(n,ntil,hh,ck,kernsavhh(:,hh))

           do i=wksample+1,wkstart

              if(initweek(nhhvars,i,hh) .eq. 0) exit

              do j=1,nipts

                 accnum = acmatbig(hh,i-wksample,j)

                 if(twopoint) then
                    call getbell(acmatbig(hh,i-wksample,j),bflag,hh,scflag(hh)+1,wksample+int(initweek(nhhvars,i,hh)),j,vf,erflag)
                 else
                    call getbell(acmatbig(hh,i-wksample,j),bflag,hh,1,wksample+int(initweek(nhhvars,i,hh)),j,vf,erflag)
                 endif

                 if(erflag .eq. 1 .and. itnum .gt. 1) then
                    print*,'bug: 0 weights in hhlogitcoupstr',i,hh
                    stop
                 endif

                 vfsav(1,acmatbig(hh,i-wksample,j)) = vf

                 if (bothflag .eq. 1) then
                    vfsav(2,acmatbig(hh,i-wksample,j)) = vf
                 endif

              enddo

           enddo

           call siminitinv(hh,cbig,npbig,initialinv(:,hh),bflag,drawstore,upass,scflag(hh))

           do i=1,maxpurchases

              if(hhweek(nhhvars,i,hh) .eq. 0) exit

              do j=1,nipts

                 accnum = acmatbig(hh,i+wkstart-wksample,j)

                 if(twopoint) then
                    call getbell(acmatbig(hh,i+wkstart-wksample,j),bflag,hh,scflag(hh)+1,wksample+int(hhweek(nhhvars,i,hh)),j,vf,erflag)
                 else
                    call getbell(acmatbig(hh,i+wkstart-wksample,j),bflag,hh,1,wksample+int(hhweek(nhhvars,i,hh)),j,vf,erflag)
                 endif

                 if(erflag .eq. 1 .and. itnum .gt. 1) then
                    print*,'bug: 0 weights in hhlogitcoupstr',i,hh
                    stop
                 endif

                 vfsav(1,acmatbig(hh,i+wkstart-wksample,j)) = vf

                 if (bothflag .eq. 1) then
                    vfsav(2,acmatbig(hh,i+wkstart-wksample,j)) = vf
                 endif

              enddo

           enddo

        endif

        inv = dble(initialinv(:,hh))

        ctemp = cbig

        if(twopoint .and. scflag(hh) .eq. 1) ctemp((numprods+2):(numprods+3)) = 0.d0

        do i=1,maxpurchases

           invsave(wkstart+i,hh) = 0.d0

           if(hhweek(nhhvars-1,i,hh) .gt. 0) then

              inflag = 0

              abucket = 0

              if(hhweek(sthhind,i,hh) .eq. 0) then
                 nchoices = 1
                 abucket = 1
                 ivchosen = 1
                 tunits = 0.d0
              else
                 nchoices = bigj+1
                 npos = 0
                 tunits = 0.d0
                 do j=1,3
                    if(hhweek(2*j-1,i,hh) .gt. 0) npos=npos+1
                    tunits = tunits + hhweek(2*j,i,hh)
                 enddo
                 if(npos .eq. 1 .or. npos .eq. 0) then
                    abucket = 1
                    ivchosen = 1
                 else if(npos .eq. 2) then
                    ivchosen=1
                    do j=1,lalpha(npos)
                       if( int(tunits*alfasmall(npos,j,1)*100) .eq. int(hhweek(2,i,hh)*100) .or. int(tunits*alfasmall(npos,j,1)*100) .eq. int(hhweek(4,i,hh)*100) ) then
                          if(abucket .ne. 0) then
                             print*,'bug in npos=2',tunits,hhweek(2,i,hh),hhweek(4,i,hh),alfasmall(npos,:,1)
                             stop
                          endif

                          abucket = j
                          ivchosen = ivchosen + j
                       endif
                    enddo
                 else if (npos .eq. 3) then
                    abucket = 1
                    ivchosen = 2 + lalpha(2)
                 endif
              endif

              cv = 0.d0

              c1c = -1
              c2c = -1
              if(int(hhweek(nhhvars-1,i,hh)) .eq. 0) then
                 print*,"error in hhweek(nhhvars-1,,)",hhweek(nhhvars-1,i,hh),i,hh
                 stop
              endif

              if(twopoint .or. scflag(hh) .eq. 1) then
                 vfpass = vfsav(1,(1+acmat(1,hh)+nipts*(pstateptr(int(hhweek(nhhvars-1,i,hh)),hh)-1)):(acmat(1,hh)+nipts*(pstateptr(int(hhweek(nhhvars-1,i,hh)),hh)-1)+nipts) )
              else
                 vfpass = 0.d0
              endif

              flowcv = 0.d0

              ! brand likelihood  (ctemp vs cbig doesn't matter here)

              if(hhweek(1,i,hh) .gt. 0) then
                 ubrand = 0.d0
                 do l=1,csetnels(ivchosen)
                    j = int(hhweek(2*l-1,i,hh))
                    ubrand = ubrand + (hhweek(2*l,i,hh)/tunits)*(cbig(j) - cbig(numprods+4)*hhweek(sthhind+j,i,hh) + cbig(numprods+5)*hhweek(sthhind+numprods+j,i,hh) + cbig(numprods+6)*hhweek(sthhind+2*numprods+j,i,hh))
                    if(hhweek(sthhind+j,i,hh) .eq. 0) then
                       print*,'error - chosen product has 0 price',i,hh,l
                       stop
                    endif
                 enddo
              endif

              ! consumption likelihood

              futilhh(i,hh) = 0.d0
              f2 = 0.d0

              do j=1,nchoices

                 do l=1,niv

                    !get optimal consumption

                    if(cadmissable(j,l)) then

                       x1 = dble(j-1)

                       p1 = ivweek(i+wkstart,hh,l)*x1/dbj

                       if(l .eq. 1) then
                          call optcons(x1,inv(1),verrorsav(1,i,hh),0.d0,pstateptr(int(hhweek(nhhvars-1,i,hh)),hh),npbig,ctemp,c1d(j),utils(j),hh,vfpass)
                       endif
                       
                       if(j .gt. 1) then
                          futilhh(i,hh) = futilhh(i,hh) + exp(utl(c1d(j),x1,inv(1),verrorsav(1,i,hh),npbig,ctemp) + p1 + ctemp(numprods+7))
                       else
                          futilhh(i,hh) = futilhh(i,hh) + exp(utl(c1d(j),x1,inv(1),verrorsav(1,i,hh),npbig,ctemp) + p1)
                       endif
                       f2(1) = f2(1) + exp(utl(c1d(j),x1,inv(1),verrorsav(1,i,hh),npbig,ctemp))
                       f2(2) = f2(2) + exp(p1)

                       if(tunits .eq. x1 .and. l .eq. ivchosen) then

                          v = utils(j)+p1
                          c1c = c1d(j)
                          x1c = x1

                       endif

                       cv = cv + exp(utils(j)+p1)

                    endif

                 enddo

              enddo

              futilhh(i,hh) = dbj*log(futilhh(i,hh))/(100.d0*ctemp(numprods+4))
              avutilhh(wkstart+i,hh) = futilhh(i,hh)
              avutilhh2(wkstart+i,hh,1) = dbj*log(f2(1))/(100.d0*ctemp(numprods+4))
              avutilhh2(wkstart+i,hh,2) = dbj*log(f2(2))/(100.d0*ctemp(numprods+4))

              invsave(wkstart+i,hh) = inv(1)
              conssave(wkstart+i,hh) = c1c
              inv(1) = inv(1) - c1c + x1c

              if(nchoices .gt. 1) then
                 p(hh) = p(hh) + v - log(cv)
                 if(hhweek(1,i,hh) .gt. 0) then
                    p(hh) = p(hh) + ubrand - ivweek(i+wkstart,hh,ivchosen)
                 endif
                 if(cv .eq. 0.d0) then
                    print*,'0 log(cv)',i,hh,v,log(cv),int(hhweek(1:sthhind,i,hh)),inv(1),c1c,x1c
                    print*,cbig,scflag(hh)
                    write(*,'(A3)',advance='NO') 'VF:'
                    do j=1,nipts-1
                       if(mod(j,10) == 0) then
                          write(*,'(F7.2)',advance='YES') vfpass(j)
                       else
                          write(*,'(F7.2)',advance='NO') vfpass(j)
                       endif
                    enddo
                    write(*,'(F7.2)',advance='YES') vfpass(j)
                    stop
                 endif
              endif

              if(c1c .eq. -1 .or. inv(1) .lt. 0) then
                 print*,'error in choices'
                 print*,hh
                 print*,tunits,ivchosen,npos,cadmissable(int(tunits)+1,ivchosen)
                 print*,int(hhweek(1:sthhind,i,hh))
                 print*,c1c,x1c
                 print*,inv(1)
                 stop
              endif

           endif

        enddo
     endif

  enddo
  !$OMP END DO
  !$OMP END PARALLEL

end subroutine hhlogitstr


! forward simulation of initial inventories

subroutine siminitinv(hh,cbig,n,inv,bflag,drawstore,upass,scflag)

  use params
  implicit none

  integer hh, inv(2)
  integer visits(wksample)
  doubleprecision vlistp(3*numprods,wksample), upass(wksample,nhhs)
  doubleprecision u1(1), vprobs(maxpurchases), s1
  doubleprecision vfpass(nipts,wksample), vfhn(nipts)
  doubleprecision vfpass1(nipts)
  doubleprecision vdraw
  integer scflag

  integer i,j,k,ii


  integer n
  integer bflag

  doubleprecision cbig(n), ctemp(n)
  doubleprecision u, v, cv
  doubleprecision vf,vfkexp,vfk

  integer inflag,it,l,npos, erflag

  integer nchoices

  doubleprecision x1, x2, c1d(bigj+1), c2d, c1c, c2c, normpdf, p1, p2, x1c, x2c
  doubleprecision utildisc, dbj
  doubleprecision tunits, futilhh, utl, f2(2)

  logical drawstore, found, allzero

  external utildisc

  inv = 0

  s1=0.d0
  k=1
  do while(hhstorevisits(1,k,hh) .gt. 0)
    s1=s1+dble(hhstorevisits(2,k,hh))
    k=k+1
  enddo
  k=k-1
  vprobs(1:k) = dble(hhstorevisits(2,1:k,hh))/s1

  do i=2,k
    vprobs(i) = vprobs(i) + vprobs(i-1)
  enddo

  ! fill in store visits

  vdraw = 1.d0

  ! in the hendel nevo model we need to compute inclusive values after drawing the store ids, so there we'll do it beforehand
  visits = visitsave(:,hh)

  if(scflag .eq. 1 .or. twopoint) then
     do i=1,lvlisti(hh)

        do j=1,nipts
           if(vlisti(i,hh) .le. 0 .or. vlisti(i,hh) .gt. nobspr) then
              print*,'bug in vlisti',vlisti(i,hh),i,lvlisti(hh),hh,j
              stop
           endif

           if(twopoint) then
              call getbell(int(1),bflag,hh,scflag+1,vlisti(i,hh),j,vf,erflag)
           else
              call getbell(int(1),bflag,hh,1,vlisti(i,hh),j,vf,erflag)
           endif

           vfpass(j,i) = vf

        enddo

     enddo
  else
     vfpass = 0.d0
  endif

  ctemp = cbig

  if(twopoint .and. scflag .eq. 1) ctemp((numprods+2):(numprods+3)) = 0.0

  !impute consumption, inventories from prior purchases

  dbj = dble(bigj)
  do i=1,wkstart

    inflag = 0

    cv = 0.d0

    c1c = -1
    c2c = -1

    tunits = 0.d0
    do j=1,3
       tunits = tunits + initweek(2*j,i,hh)
    enddo

    x1c = tunits

    if(i .le. wksample) then

       if(visits(i) .lt. 0) then
          nchoices = 1
       else
          nchoices = bigj+1
       endif

       !get optimal consumption
       ! note that outside of the effect on the value function, the inclusive value
       ! doesn't affect consumption, so the inclusive value here can be anything
       ! the state of the iv is indexed in vlist(i,hh)
       
       call optcons(tunits,dble(inv(1)),vdraw,ivweek(i,hh,1)*tunits/dbj,vlist(i,hh),n,ctemp,c1c,v,hh,vfpass(:,vlist(i,hh)))

    else

       if(initweek(sthhind,i,hh) .le. 0) then
          nchoices = 1
       else
          nchoices = bigj+1
       endif

       if(twopoint .or. scflag .eq. 1) then
          vfpass1 = vfsav(1,(1+acmat(1,hh)+nipts*(pstateptr(int(initweek(nhhvars-1,i,hh)),hh)-1)):(acmat(1,hh)+nipts*(pstateptr(int(initweek(nhhvars-1,i,hh)),hh)-1)+nipts) )
       else
          vfpass = 0.d0
       endif

       call optcons(tunits,dble(inv(1)),vdraw,ivweek(i,hh,1)*tunits/dbj,pstateptr(int(initweek(nhhvars-1,i,hh)),hh),n,ctemp,c1c,v,hh,vfpass1)

    endif

    ! this needed if we get utility at all states

    futilhh = 0.d0
    f2 = 0.d0

    do j=1,nchoices

       do l=1,niv

          !get optimal consumption

          if(cadmissable(j,l)) then

             x1 = dble(j-1)

             p1 = ivweek(i,hh,l)*x1/dbj

             if(l .eq. 1) then
                if(i .le. wksample) then
                   call optcons(x1,dble(inv(1)),vdraw,0.d0,vlist(i,hh),n,ctemp,c1d(j),v,hh,vfpass(:,vlist(i,hh)))
                else
                   call optcons(x1,dble(inv(1)),vdraw,0.d0,pstateptr(int(initweek(nhhvars-1,i,hh)),hh),n,ctemp,c1d(j),v,hh,vfpass1)
                endif
             endif

             if(j .gt. 1) then
                futilhh = futilhh + exp(utl(c1d(j),x1,dble(inv(1)),vdraw,n,ctemp) + p1 + ctemp(numprods+7))
             else
                futilhh = futilhh + exp(utl(c1d(j),x1,dble(inv(1)),vdraw,n,ctemp) + p1)
             endif
             f2(1) = f2(1) + exp(utl(c1d(j),x1,dble(inv(1)),vdraw,n,ctemp))
             f2(2) = f2(2) + exp(p1)

          endif

       enddo

    enddo

    futilhh = dbj*log(futilhh)/(100.d0*ctemp(numprods+4))
    avutilhh(i,hh) = futilhh
    avutilhh2(i,hh,1) = dbj*log(f2(1))/(100.d0*ctemp(numprods+4))
    avutilhh2(i,hh,2) = dbj*log(f2(2))/(100.d0*ctemp(numprods+4))

    invsave(i,hh) = inv(1)
    conssave(i,hh) = c1c
    inv(1) = inv(1) - c1c + x1c

    if(c1c .eq. -1  .or. inv(1) .lt. 0) then
       print*,'error in choices (inv simulation)'
       print*,hh,i
       print*,int(initweek(1:sthhind,i,hh))
       print*,c1c,x1c
       print*,inv(1)
       print*,'****'
       print*,vlist(i,hh)
       print*,visits(i)
       print*,initprice(:,abs(visits(i)),i)
       stop
    endif

  enddo


end subroutine siminitinv





subroutine drawast(n,bo,s0,npbig,fixedcoefs,notest)

  !draw next iteration of parameter means

  use params
  implicit none

  integer n, i, j, npbig

  doubleprecision normdraws(n)

  doubleprecision dch(n,n)      !cholesky parameter variances

  doubleprecision bmean(n)      !mean of b

  doubleprecision bo(n), s0(n)  !prior parameters

  integer fixedcoefs(npbig), notest(npbig)

  ! stuff for wishart model
  doubleprecision, allocatable :: dsmall(:,:), dchsmall(:,:), eps(:,:), s0big(:,:)
  doubleprecision, allocatable :: b0big(:,:), bmeanbig(:,:), boutbig(:,:), s0i(:,:)
  doubleprecision, allocatable :: denomi(:,:), cdenomi(:,:), di(:,:), denom1(:,:)
  integer nvarying, i1, j1, i2, j2, hh

  !external DRNNOA, DLFTDS


  if(diagvar) then

     call DRNNOA(n,normdraws)

     dch = sqrt(din)

     bmean = 0.d0

     do i=1,nhhs

        do j=1,n

           bmean(j) = bmean(j) + bin(j,i)

        enddo

     enddo

     do j=1,n

        if(s0(j) == -1) then

           bmean(j) = bmean(j)/dble(nhhs)

           aout(j) = bmean(j) + dch(j,j)*normdraws(j)/sqrt(dble(nhhs))

        else

           bmean(j) = (bo(j)/s0(j)+bmean(j)/din(j,j))/(1.d0/s0(j)+dble(nhhs)/din(j,j))

           aout(j) = bmean(j) + sqrt(1.d0/(1.d0/s0(j)+dble(nhhs)/din(j,j)))*normdraws(j)

        endif

     enddo

  else

     nvarying = 0
     do i=1,npbig
        if(notest(i) .eq. 0 .and. fixedcoefs(i) .eq. 0) nvarying = nvarying + 1
     enddo
     allocate(dsmall(nvarying,nvarying),dchsmall(nvarying,nvarying),eps(1,nvarying))
     i1 = 0
     i2 = 0
     do i=1,npbig
        if(notest(i) .eq. 0) i2=i2+1
        if(notest(i) .eq. 0 .and. fixedcoefs(i) .eq. 0) then
           i1=i1+1
           j1=0
           j2=0
           do j=1,npbig
              if(notest(j) .eq. 0) j2=j2+1
              if(notest(j) .eq. 0 .and. fixedcoefs(j) .eq. 0) then
                 j1=j1+1
                 dsmall(i1,j1) = din(i2,j2)
              endif
           enddo
        endif
     enddo

     allocate(s0big(nvarying,nvarying),b0big(nvarying,1),s0i(nvarying,nvarying),bmeanbig(nvarying,1),boutbig(nvarying,1))
     s0big = 0.d0
     bmeanbig = 0.d0
     i1=0
     i2=0
     do i=1,npbig
        if(notest(i) .eq. 0) i2=i2+1
        if(notest(i) .eq. 0 .and. fixedcoefs(i) .eq. 0) then
           i1=i1+1
           s0big(i1,i1) = s0(i2)
           b0big(i1,1) = bo(i2)
           do hh=1,nhhs
              bmeanbig(i1,1) = bmeanbig(i1,1) + bin(i2,hh)/dble(nhhs)
           enddo
        endif
     enddo

     allocate(denomi(nvarying,nvarying),di(nvarying,nvarying),denom1(nvarying,nvarying))

     s0i = 0.d0
     di = 0.d0

     call invertmat(s0big,s0i,nvarying)
     call invertmat(dsmall,di,nvarying)

     denom1 = s0big+di*(dble(nhhs))

     denomi = 0.d0
     call invertmat(denom1,denomi,nvarying)

     dchsmall = 0.d0
     call chol(denomi,dchsmall,nvarying)

     allocate(cdenomi(nvarying,nvarying))

     cdenomi = 0.d0
     call chol(denomi,cdenomi,nvarying)

     call DRNNOA(nvarying,eps(1,:))

     boutbig = matmul(s0i,b0big) + matmul(di,bmeanbig)
     boutbig = matmul(denomi,boutbig) + transpose(matmul(eps,cdenomi))

     i1 = 0
     i2 = 0
     do i=1,nvarying
        if(notest(i) .eq. 0) i2 = i2 + 1
        if(notest(i) .eq. 0 .and. fixedcoefs(i) .eq. 0) then
           i1=i1+1
           aout(i2) = boutbig(i1,1)
        endif
     enddo
  endif

end subroutine drawast


subroutine drawdconststr(n,vo,so,npbig,fixedcoefs,notest)

  !draw next iteration of D
  !constrained version

  use params
  implicit none

  integer n, i, j, hh, i1, j1, i2, j2

  integer npbig
  integer fixedcoefs(npbig), notest(npbig)

  doubleprecision s
  integer vo
  doubleprecision so

  ! stuff for nondiagonal matrix
  integer nvarying
  doubleprecision, allocatable :: eye(:,:), bigs(:,:), w(:,:), xx(:,:), xx1(:,:), nd(:,:), bigsc(:,:)
  doubleprecision, allocatable :: bigsi(:,:)

  doubleprecision normdraws(n,nhhs+vo)

  external DRNNOA

  dout = 0.d0

  if(diagvar) then

     ! draw diagonals from inverse gamma

     do i=1,nhhs+vo
        call DRNNOA(n,normdraws(:,i))
     enddo

     do i=1,n

        s = dble(vo)*so + dot_product(bin(i,:)-ain(i),bin(i,:)-ain(i))

        dout(i,i) = s/dot_product(normdraws(i,:),normdraws(i,:))

     enddo

     j=0
     do i=1,npbig
        if(notest(i) == 0) then
           j=j+1
           if (fixedcoefs(i) .eq. 1) dout(j,j) = 0.d0
        endif
     enddo

  else

     ! draw D from inverse Wishart

     nvarying = 0
     do i=1,npbig
        if(notest(i) .eq. 0 .and. fixedcoefs(i) .eq. 0) nvarying = nvarying + 1
     enddo
     allocate(eye(nvarying,nvarying),bigs(nvarying,nvarying),w(nvarying,nvarying),xx(nvarying,nhhs+vo),xx1(nvarying,nvarying))
     allocate(nd(nvarying,nhhs+vo),bigsc(nvarying,nvarying),bigsi(nvarying,nvarying))

     do i=1,nhhs+vo
        call DRNNOA(nvarying,nd(:,i))
     enddo

     eye = 0.d0
     do i=1,nvarying
        eye(i,i) = 1.d0
     enddo

     i1 = 0
     i2 = 0
     do i=1,npbig
        if(notest(i) .eq. 0) i2=i2+1
        if(notest(i) .eq. 0 .and. fixedcoefs(i) .eq. 0) then
           i1=i1+1
           j1=0
           j2=0
           do j=1,npbig
              if(notest(j) .eq. 0) j2=j2+1
              if(notest(j) .eq. 0 .and. fixedcoefs(j) .eq. 0) then
                 j1=j1+1
                 bigs(i1,j1) = dot_product(bin(i2,:)-ain(i2),bin(j2,:)-ain(j2)) + dble(vo)*eye(i1,j1)
              endif
           enddo
        endif
     enddo

     call invertmat(bigs,bigsi,nvarying)

     bigsc = 0.d0
     call chol(bigsi,bigsc,nvarying)
     xx = matmul(transpose(bigsc),nd)

     xx1 = matmul(xx,transpose(xx))
     w = 0.d0
     call invertmat(xx1,w,nvarying)

     i1 = 0
     i2 = 0
     do i=1,npbig
        if(notest(i) .eq. 0) i2=i2+1
        if(notest(i) .eq. 0 .and. fixedcoefs(i) .eq. 0) then
           i1 = i1+1
           j1 = 0
           j2 = 0
           do j=1,npbig
              if(notest(j) .eq. 0) j2=j2+1
              if(notest(j) .eq. 0 .and. fixedcoefs(j) .eq. 0) then
                 j1=j1+1
                 dout(i2,j2) = w(i1,j1)
              endif
           enddo
        endif
     enddo

  endif

end subroutine drawdconststr







!           UTILITY FUNCTIONS











doubleprecision function utl(c,x,inv,v,n,coef)

  !flow utility - logarithmic function

  use params
  implicit none

  integer n
  doubleprecision c,x,v,inv,coef(n)

  doubleprecision v1e, v2e

  utl = coef(numprods+1)*log(c+v) + coef(numprods+2)*((inv+x-scwt*c)/invdiv) &
       & + coef(numprods+3)*( ((inv+x-scwt*c)/invdiv)**2 )

end function utl


doubleprecision function utill(c,x,inv,v,p,n,coef,vf)

  !utility plus value function


  use params
  implicit none

  integer n

  doubleprecision c,x,v,inv,coef(n),vf(nipts)

  integer p  !price state

  doubleprecision utl

  utill = utl(c,x,inv,v,n,coef) + delta*vf(int(inv-c+x)+1)

end function utill








! find optimal consumption


subroutine optcons(x1,inv,v,p1,p,n,coef,c,util,hh,vf)

  !get your optimal consumption
  !since we're assuming consumption is
  !discrete, this is easy to find


  use params
  implicit none

  integer n,p,c2lb,hh

  doubleprecision x,inv,v,coef(n),c,u,x1,p1,vf(nipts)

  integer ic1,ic2

  doubleprecision utill,cmax,util,utold


  x = x1

  util = -1000000
  utold = util

  cmax = 0
  if(int(inv+x) .gt. maxinv) cmax = dble(int(inv+x)-maxinv)
  do ic1=1,int(inv+x)+1

     !you can't consume above the maximum inventory threshold
     if(int(inv+x)-(ic1-1) .le. maxinv) then

        c=dble(ic1-1)
        u=utill(c,x,inv,v,p,n,coef,vf)
        if(u .gt. utold) then
           util = u
           cmax = c
           utold = u
        endif

     endif

  enddo

  c = cmax

  util = util + p1
  if(x .gt. 0) util = util + coef(numprods+7)

  if(c .lt. 0) then
     print*,"c bug (outside)",c,int(inv+x)+1,util
     stop
  endif

end subroutine optcons


! post estimation work routine

subroutine convertheir(np,nkeep,nskip)

  use params
  implicit none

  integer np, nkeep, nskip,i,j,k

  doubleprecision asave(np,nkeep/nskip)
  doubleprecision dsave(np,np,nkeep/nskip)

  !turn heirarchical draws into a file to load into R

  open(unit=1,file=pathname//prodname//'/hb/heirsaves.dat')

  read(1,*) asave
  read(1,*) dsave

  close(unit=1,status='keep')

  open(unit=1,file=pathname//prodname//'/hb/hsavesforR.dat')

  do i=1,nkeep/nskip

     do j=1,np

        write(1,'(F20.10)',advance='NO') asave(j,i)

     enddo

     write(1,*)

  enddo

  do i=1,nkeep/nskip

     do j=1,np

        do k=1,np

           write(1,'(F20.10)',advance='NO') dsave(k,j,i)

        enddo

        write(1,*)

     enddo

  enddo

  close(unit=1,status='keep')

end subroutine convertheir
