#include <Rcpp.h>
#include <cmath>

// transform parameters (like the fortran code transcoef)

void tform(const Rcpp::NumericVector &hhcoefs, const int d,
                          const int hh, const int nparam, const int nhhs,
                          const int ndraws, const int npbig,
                          const Rcpp::NumericVector &negcoefs,
                          const Rcpp::IntegerVector &expcoefs,
                          const Rcpp::IntegerVector &poscoefs,
                          const Rcpp::NumericVector &coeffull,
                          const Rcpp::IntegerVector &notest,
                          Rcpp::NumericVector &coefout) {

  int j=0;

  for(int i=0;i<npbig;i++) {

    if(notest(i)) {
      coefout(i) = coeffull(i);
    } else {
      coefout(i) = hhcoefs(j+nparam*(hh+nhhs*d));
      j++;
    }

    if(expcoefs(i)) {
      coefout(i) = exp(coefout(i)*negcoefs(i));
    }
    if(poscoefs(i) && coefout(i) <= 0) {
      coefout(i) = 0;
    }

  }

}


// compute inclusive values given individual level
// parameters

// this one does the computation for canned tuna where we only
// have a single inclusive value
// hhcoefs should be number of params by number of households by
// number of draws

// [[Rcpp::export]]
Rcpp::List computeivcpptuna(Rcpp::NumericVector hhcoefs, Rcpp::List inputs,
                            Rcpp::NumericVector hhweek) {

  int nhhs = inputs["nhhs"];
  int nparam = inputs["nparam"];
  int nproduct = inputs["nproduct"];
  int nwparam = inputs["nwparam"];
  int nweek = inputs["nweek"];
  int ndraws = inputs["ndraws"];
  int nskip = inputs["nskip"];   //if we don't skip this is REALLY SLOW
  int bigj = inputs["bigj"];

  SEXP nc1 = inputs["negcoefs"];
  Rcpp::NumericVector negcoefs(nc1);

  SEXP ec1 = inputs["expcoefs"];
  Rcpp::IntegerVector expcoefs(ec1);

  SEXP pc1 = inputs["poscoefs"];
  Rcpp::IntegerVector poscoefs(pc1);

  SEXP cf1 = inputs["coeffull"];
  Rcpp::NumericVector coeffull(cf1);

  SEXP ne1 = inputs["notest"];
  Rcpp::IntegerVector notest(ne1);

  int npbig = notest.size();

  Rcpp::NumericMatrix ivout(nhhs,nweek);
  Rcpp::NumericMatrix utout(nhhs,nweek);

  for(int hh=0;hh<nhhs;hh++) {
    Rcpp::NumericVector cbig(npbig);
    for(int d=0;d<ndraws;d += nskip) {
      tform(hhcoefs,d,hh,nparam,nhhs,ndraws,npbig,
            negcoefs,expcoefs,poscoefs,coeffull,
            notest,cbig);
      for(int n=0;n<nweek;n++) {
        double iv = 0;
        for(int k=0;k<nproduct;k++) {
          double price = hhweek(5+k+nwparam*(n+nweek*hh));
          double feat = hhweek(5+nproduct+k+nwparam*(n+nweek*hh));
          double disp = hhweek(5+2*nproduct+k+nwparam*(n+nweek*hh));
          if(price > 0) {
            iv += exp( cbig(k) - cbig(nproduct+3)*price +
                       cbig(nproduct+4)*feat+cbig(nproduct+5)*disp );
          }
          if(iv > 0) {
            iv = log(iv);
          }
        }
        double util = 0;
        if(iv != 0) {
          for(int k=1;k<bigj+1;k++) {
            util += exp( ((double)k/(double)bigj)*iv );
          }
        } else {
          util = 1;
        }
        util = log(util);
        ivout(hh,n) += iv;
        utout(hh,n) += util*((double)bigj)/(100.0*cbig(nproduct+3));
      }
    }
    for(int n=0;n<nweek;n++) {
      ivout(hh,n) /= (double)ndraws/(double)nskip;
      utout(hh,n) /= (double)ndraws/(double)nskip;
    }
  }


  Rcpp::List res;
  res("iv") = ivout;
  res("ut") = utout;

  return(res);

}


// version for canned soup

// [[Rcpp::export]]
Rcpp::List computeivcppsoup(Rcpp::NumericVector hhcoefs, Rcpp::List inputs,
                            Rcpp::NumericVector hhweek) {

  int nhhs = inputs["nhhs"];
  int nparam = inputs["nparam"];
  int nproduct = inputs["nproduct"];
  int nwparam = inputs["nwparam"];
  int nweek = inputs["nweek"];
  int ndraws = inputs["ndraws"];
  int nskip = inputs["nskip"];   //if we don't skip this is REALLY SLOW
  int bigj = inputs["bigj"];
  int niv = inputs["niv"];
  int stind = inputs["stind"];
  stind--;

  SEXP nc1 = inputs["negcoefs"];
  Rcpp::NumericVector negcoefs(nc1);

  SEXP ec1 = inputs["expcoefs"];
  Rcpp::IntegerVector expcoefs(ec1);

  SEXP pc1 = inputs["poscoefs"];
  Rcpp::IntegerVector poscoefs(pc1);

  SEXP cf1 = inputs["coeffull"];
  Rcpp::NumericVector coeffull(cf1);

  SEXP ne1 = inputs["notest"];
  Rcpp::IntegerVector notest(ne1);

  SEXP css1 = inputs["csetsize"];
  Rcpp::IntegerVector csetsize(css1);

  SEXP csn1 = inputs["csetnels"];
  Rcpp::IntegerVector csetnels(csn1);

  SEXP csm1 = inputs["choicesetmat"];
  Rcpp::IntegerVector choicesetmat(csm1);

  SEXP csd1 = inputs["choicesetdims"];
  Rcpp::IntegerVector csd(csd1);

  SEXP alfa1 = inputs["alphabig"];
  Rcpp::NumericVector alphabig(alfa1);

  SEXP cad1 = inputs["cadmissable"];
  Rcpp::IntegerMatrix cad(cad1);

  int npbig = notest.size();

  Rcpp::NumericVector ivout(nhhs*nweek*niv);
  Rcpp::NumericMatrix utout(nhhs,nweek);

  for(int hh=0;hh<nhhs;hh++) {
    Rcpp::NumericVector cbig(npbig);
    for(int d=0;d<ndraws;d += nskip) {
      tform(hhcoefs,d,hh,nparam,nhhs,ndraws,npbig,
            negcoefs,expcoefs,poscoefs,coeffull,
            notest,cbig);
      for(int n=0;n<nweek;n++) {
        double iv[niv];
        double pvec[nproduct];
        double fvec[nproduct];
        double dvec[nproduct];
        for(int k=0;k<nproduct;k++) {
          pvec[k] = hhweek(7+k+nwparam*(n+nweek*hh));
          fvec[k] = hhweek(7+nproduct+k+nwparam*(n+nweek*hh));
          dvec[k] = hhweek(7+2*nproduct+k+nwparam*(n+nweek*hh));
        }
        for(int k=0;k<niv;k++) {
          iv[k] = 0;
          double denom = 0;
          for(int j=0;j<csetsize(k);j++) {

            int dontskip = 1;
            for(int l=0;l<csetnels(k);l++) {
              int pindx = choicesetmat(l+csd(0)*(j + csd(1)*k))-1;
              dontskip = dontskip && pvec[pindx] > 0;
            }

            if(dontskip) {
              double u=0;
              for(int l=0;l<csetnels(k);l++) {
                int pindx = choicesetmat(l+csd(0)*(j + csd(1)*k))-1;
                u += alphabig(l+csd(0)*(j + csd(1)*k))*(cbig(pindx) - cbig(nproduct+3)*pvec[pindx] + cbig(nproduct+4)*fvec[pindx] + cbig(nproduct+5)*dvec[pindx]);
              }
              denom += exp(u);
            }
          }
          if(denom > 0) {
            iv[k] = log(denom);
          }
        }

        int nchoices = hhweek(stind+nwparam*(n+nweek*hh)) == 0 ? 1 : bigj+1;
        double util = 0;
        for(int j=1;j<nchoices;j++) {
          for(int l=0;l<niv;l++) {
            if(cad(j,l)) {
              util += exp( ((double)j/(double)bigj)*iv[l] );
            }
          }
        }

        if(util > 0) {
          util = log(util);
        }

        for(int k=0;k<niv;k++) {
          ivout(hh+nhhs*(n+nweek*k)) += iv[k];
        }
        utout(hh,n) += util*((double)bigj)/(100.0*cbig(nproduct+3));
      }
    }
    for(int n=0;n<nweek;n++) {
      for(int k=0;k<niv;k++) {
        ivout(hh+nhhs*(n+nweek*k)) /= (double)ndraws/(double)nskip;
      }
      utout(hh,n) /= (double)ndraws/(double)nskip;
    }
  }

  Rcpp::IntegerVector dims(3);
  dims(0) = nhhs;
  dims(1) = nweek;
  dims(2) = niv;

  ivout.attr("dim") = dims;

  Rcpp::List res;
  res("iv") = ivout;
  res("ut") = utout;

  return(res);

}
