# look at sale frequencies, etc, across the markets and over time to see if we can get anything

rm(list=ls())

product = "tuna"
inputpath = "C:/Users/MattOsborne/Dropbox/price index paper/input data/"                   #raw data
outputpath = "C:/Users/MattOsborne/Dropbox/price index paper/step 1 output files/"         #intermediate output
tablepath = "C:/Users/MattOsborne/Dropbox/price index paper/output tables and figures/"    #tables for paper
figpath = "C:/Users/MattOsborne/Dropbox/price index paper/output tables and figures/"      #tables for paper

load(paste(outputpath,"erim_",product,".RData",sep=""))

stdata = stdata[-c(1234:1539,1819:2049),]

# print out summary stats of household purchase data

cat("Number of households in Sioux Falls:", length(unique(hhpurch$hhid[hhpurch$market==1])), "\n")
cat("Number of households in Springfield:", length(unique(hhpurch$hhid[hhpurch$market==2])), "\n")

cat("Number of purchases in Sioux Falls:", length(hhpurch$hhid[hhpurch$market==1]), "\n")
cat("Number of purchases in Springfield:", length(hhpurch$hhid[hhpurch$market==2]), "\n")

stdata$price = stdata$price/100

allstids = unique(stdata$storeid)
allstids = allstids[order(allstids)]

# market 2 store brand UPCs: "36800817600" "36800817700" stores 1 through 15. although there are 2 UPCs the product descriptions are the same
# 70038313060 stores 17 through 31

brandnames = c("MARTEL","B BEE","RICHELIEU","3 DIAMONDS","THREE PEARLS","GEISHA","CARNATION","CKN OF SEA","FEATHERWEIGHT",
               "BRST O' CKN","STAR-KIST","CTL BR","GENERIC")

oilwtr = c("OIL","WTR","DT")

chk = c("CHK","SLD","GRTD")

lt = c("LT","WHT")

bdatabig = branddata
bdatabig$brandid = 0

j=0
for(i in brandnames) {
  j=j+1
  bdatabig$brandid[grep(i,bdatabig$desc)] = j
}

# add in 2 more brands for the market 2 store brands
j=j+1
bdatabig$brandid[bdatabig$upc %in% c(36800817600, 36800817700)] = j

j=j+1
bdatabig$brandid[bdatabig$upc == 70038313060] = j

brandnames = c(brandnames,"CTL BR (mkt 2 sb 1)","CTL BR (mkt 2 sb 2)")

bdatabig$oilwtr = 0

j=0
for(i in oilwtr) {
  j=j+1
  bdatabig$oilwtr[grep(i,bdatabig$desc)] = j
}

bdatabig$chksld = 0

j=0
for(i in chk) {
  j=j+1
  bdatabig$chksld[grep(i,bdatabig$desc)] = j
}

bdatabig$lt = 0
bdatabig$lt[grep("lt",bdatabig$desc)] = 1

bdatabig$wtcode <- NULL
bdatabig$wtdesc <- NULL
bdatabig$eqfactor <- NULL
bdatabig$rsln <- NULL

# merge brand info data into purchase data

hhpurch$filler <- NULL

hhpurch = merge(hhpurch,bdatabig,all.x=TRUE)
hhpurch$brandname = brandnames[hhpurch$brandid]

hhpurch$uprice = hhpurch$price/pmax(100,hhpurch$units)  # there is 1 observation where units = 0

# make table of market shares and size shares

bnamest = c("Martel","Bumblebee","Richelieu","3 Diamonds","Three Pearls","Geisha","Carnation","Chicken of the Sea","Featherweight",
               "Breast O' Chicken","Starkist","Store Brand","Generic")

mktshares = matrix(0,nrow=length(bnamest),ncol=3)

for(i in 1:length(brandnames)) {
  for(j in 1:2) {
    if(i <= length(bnamest)) {
      mktshares[i,j] = sum(hhpurch$brandid == i & hhpurch$market == j)/sum(hhpurch$market==j)
    } else {
      mktshares[length(bnamest)-1,j] = mktshares[length(bnamest)-1,j] + sum(hhpurch$brandid == i & hhpurch$market == j)/sum(hhpurch$market==j)
    }
  }
}

for(i in 1:length(bnamest)) {
  mktshares[i,3] = (mktshares[i,1]*sum(hhpurch$market==1)+mktshares[i,2]*sum(hhpurch$market==2))/nrow(hhpurch)
}

o = order(mktshares[,3],decreasing=TRUE)
bnamest = bnamest[o]
mktshares = mktshares[o,]

sharetable = bnamest[mktshares[,3] > 0.01]

sigfig = 1

for(i in 1:length(sharetable)) {
  sharetable[i] = paste(sharetable[i]," & ",round(100*mktshares[i,1],sigfig)," \\\\",sep="")
}

write.table(sharetable,file=paste(tablepath,"share",product,".tex",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

print(table(hhpurch$wtamt)/nrow(hhpurch))

allweek = unique(hhpurch$week)
allweek = allweek[order(allweek)]

stdata = merge(stdata,bdatabig,all.x=TRUE)
stdata = stdata[order(stdata$storeid,stdata$week,stdata$brandid,stdata$oilwtr,stdata$upc),]

stprices = array(0,c(length(allstids),length(allweek),length(brandnames),2,2))

# remove purchases that are not in the categories used for estimation
# also remove a few strange upcs.  ie Starkist select (157 purchases), 2nd is starkist 6pk (29 purchases)

badupc = c(79100001340,17800070160)

badpurch = hhpurch$chksld != 1 | hhpurch$oilwtr == 3 | hhpurch$wtamt != 6500 | hhpurch$upc %in% badupc | hhpurch$price == 0

hhpurch = hhpurch[!badpurch,]

badst = stdata$chksld != 1 | stdata$oilwtr == 3 | stdata$lt > 0 | stdata$wtamt != 6500 | stdata$upc %in% badupc

stdata = stdata[!badst,]

hhpurch = hhpurch[order(hhpurch$hhid,hhpurch$week,hhpurch$brandid,hhpurch$oilwtr,hhpurch$day),]

# load the SAS price files

pf1 = read.csv(paste(inputpath,product,"/pf1.csv",sep=""),stringsAsFactors=FALSE)
pf2 = read.csv(paste(inputpath,product,"/pf2.csv",sep=""),stringsAsFactors=FALSE)

pf1$DATE = as.Date(pf1$DATE,origin="1960-01-01")
pf1$WEEK = as.Date(pf1$WEEK,origin="1960-01-01")

pf1 = merge(pf1,bdatabig,all.x=TRUE,by.x="UPC",by.y="upc")

save(pf1,file=paste(outputpath,"dailyprices1_",product,".RData",sep=""))

pf1 = pf1[!(pf1$chksld != 1 | pf1$oilwtr == 3 | pf1$lt > 0 | pf1$wtamt != 6500 | pf1$UPC %in% badupc),]

pf2$DATE = as.Date(pf2$DATE,origin="1960-01-01")
pf2$WEEK = as.Date(pf2$WEEK,origin="1960-01-01")

pf2 = merge(pf2,bdatabig,all.x=TRUE,by.x="UPC",by.y="upc")

pf2 = pf2[!(pf2$chksld != 1 | pf2$oilwtr == 3 | pf2$lt > 0 | pf2$wtamt != 6500 | pf2$UPC %in% badupc),]

d2w = read.csv(paste(inputpath,"dateweektranslation.csv",sep=""),stringsAsFactors=FALSE)
d2w$mindate = as.Date(d2w$mindate,format="%m/%d/%Y")
d2w$maxdate = as.Date(d2w$maxdate,format="%m/%d/%Y")
save(d2w,file=paste(outputpath,"d2w.RData",sep=""))

pf1$erimweek = 0
for(i in 1:nrow(d2w)) {
  pf1$erimweek[pf1$DATE >= d2w$mindate[i] & pf1$DATE <= d2w$maxdate[i]] = d2w$erimweek[i]
}

pf2$erimweek = 0
for(i in 1:nrow(d2w)) {
  pf2$erimweek[pf1$DATE >= d2w$mindate[i] & pf1$DATE <= d2w$maxdate[i]] = d2w$erimweek[i]
}

# aggregate to the weekly level

pf1w = aggregate(pf1$DPR,by=list(pf1$SID,pf1$erimweek,pf1$UPC),FUN=min)

colnames(pf1w) = c("SID","erimweek","UPC","price")

uw = aggregate(pf1$UNITS,by=list(pf1$SID,pf1$erimweek,pf1$UPC),FUN=sum)

colnames(uw) = c("SID","erimweek","UPC","UNITS")

pf1w$UNITS = uw$UNITS

rp = aggregate(pf1$REG,by=list(pf1$SID,pf1$erimweek,pf1$UPC),FUN=min)

colnames(rp) = c("SID","erimweek","UPC","REG")

pf1w$REG = rp$REG

pfweeks = aggregate(pf1$DATE,by=list(pf1$SID,pf1$erimweek,pf1$UPC),FUN=min)

colnames(pfweeks) = c("SID","erimweek","UPC","WEEK")

pf1w$WEEK = pfweeks$WEEK

pf1w = merge(pf1w,bdatabig,all.x=TRUE,by.x="UPC",by.y="upc")

pf1w = pf1w[order(pf1w$SID,pf1w$erimweek,pf1w$UPC),]

pf1w = pf1w[order(pf1w$SID,pf1w$UPC,pf1w$erimweek),]

pf1w = pf1w[pf1w$erimweek > 0 & pf1w$erimweek < 198724,]


pf2w = aggregate(pf2$DPR,by=list(pf2$SID,pf2$erimweek,pf2$UPC),FUN=min)

colnames(pf2w) = c("SID","erimweek","UPC","price")

uw = aggregate(pf2$UNITS,by=list(pf2$SID,pf2$erimweek,pf2$UPC),FUN=sum)

colnames(uw) = c("SID","erimweek","UPC","UNITS")

pf2w$UNITS = uw$UNITS

rp = aggregate(pf2$REG,by=list(pf2$SID,pf2$erimweek,pf2$UPC),FUN=min)

colnames(rp) = c("SID","erimweek","UPC","REG")

pf2w$REG = rp$REG

pfweeks = aggregate(pf2$DATE,by=list(pf2$SID,pf2$erimweek,pf2$UPC),FUN=min)

colnames(pfweeks) = c("SID","erimweek","UPC","WEEK")

pf2w$WEEK = pfweeks$WEEK

pf2w = merge(pf2w,bdatabig,all.x=TRUE,by.x="UPC",by.y="upc")

pf2w = pf2w[order(pf2w$SID,pf2w$erimweek,pf2w$UPC),]

pf2w = pf2w[pf2w$erimweek > 0 & pf2w$erimweek < 198724,]

# compare sale frequencies, etc

brands = unique(c(pf1w$brandid,pf2w$brandid))

brands = brands[order(brands)]

scut = matrix(0,nrow=length(brands),ncol=2)
scuthh = matrix(0,nrow=length(brands),ncol=2)

j=0
for(br in brands) {
  j=j+1

  scut[j,1] = quantile(pf1w$price[pf1w$price > 0 & pf1w$brandid == br],probs=c(0.25))
  scut[j,2] = quantile(pf2w$price[pf2w$price > 0 & pf2w$brandid == br],probs=c(0.25))
  scuthh[j,1] = quantile(hhpurch$uprice[hhpurch$brandid == br & hhpurch$market == 1],probs=c(0.5))
  scuthh[j,2] = quantile(hhpurch$uprice[hhpurch$brandid == br & hhpurch$market == 2],probs=c(0.5))

}

spct = array(0.1,c(length(brands),2))  # percent where we define sale

# compute promotion frequencies

pf1w$sale = FALSE
pf2w$sale = FALSE

j=0

mweek = allweek[1]
inds1 = pf1w$erimweek >= mweek
inds2 = pf2w$erimweek >= mweek

for(br in brands) {
  j=j+1
  pf1w$sale[pf1w$brandid == br & pf1w$price <= pf1w$REG*(1-spct[j,1])] = TRUE
  pf2w$sale[pf2w$brandid == br & pf2w$price <= pf2w$REG*(1-spct[j,2])] = TRUE
  cat("% of time ",brandnames[br]," is on sale in market 1: ",mean(pf1w$sale[pf1w$brandid == br & inds1]),"\n",sep="")
  cat("% of time ",brandnames[br]," is on sale in market 2: ",mean(pf2w$sale[pf2w$brandid == br & inds2]),"\n",sep="")
}

salepcts = matrix(0,nrow=3,ncol=2)
salepcts[1,1] = mean(pf1w$sale[pf1w$brandid == 11 & inds1])
salepcts[1,2] = mean(pf2w$sale[pf2w$brandid == 11 & inds2])
salepcts[2,1] = mean(pf1w$sale[pf1w$brandid == 8 & inds1])
salepcts[2,2] = mean(pf2w$sale[pf2w$brandid == 8 & inds2])
salepcts[3,1] = mean(pf1w$sale[pf1w$brandid %in% c(12,14,15) & inds1])
salepcts[3,2] = mean(pf2w$sale[pf2w$brandid %in% c(12,14,15) & inds2])

# promotion depth

for(br in brands) {
  cat("avg sale, nonsale price for brand ",brandnames[br],": ",mean(pf1w$price[pf1w$brandid == br & pf1w$sale & inds1]),
      ", ",mean(pf1w$price[pf1w$brandid == br & !pf1w$sale & inds1])," for market 1.\n",sep="")
  cat("avg sale, nonsale price for brand ",brandnames[br],": ",mean(pf2w$price[pf2w$brandid == br & pf2w$sale & inds2]),
      ", ",mean(pf2w$price[pf2w$brandid == br & !pf2w$sale & inds2])," for market 2.\n",sep="")
}

avgprices = array(0,c(3,3,2))

blist = list(11,8,c(12,14,15))
for(i in 1:3) {
  avgprices[i,1,1] = mean(pf1w$price[pf1w$brandid %in% blist[[i]] & inds1])
  avgprices[i,2,1] = mean(pf1w$price[pf1w$brandid %in% blist[[i]] & pf1w$sale & inds1])
  avgprices[i,3,1] = mean(pf1w$price[pf1w$brandid %in% blist[[i]] & !pf1w$sale & inds1])

  avgprices[i,1,2] = mean(pf2w$price[pf2w$brandid %in% blist[[i]] & inds2])
  avgprices[i,2,2] = mean(pf2w$price[pf2w$brandid %in% blist[[i]] & pf2w$sale & inds2])
  avgprices[i,3,2] = mean(pf2w$price[pf2w$brandid %in% blist[[i]] & !pf2w$sale & inds2])

}

# sale frequencies for the brands used in the model estimation

upckeep = c(17800000120,17800000240,79100002450,79100002800,75450022620,77600152580)

nweek = length(allweek) - sum(allweek < mweek)
nmonth = as.integer(nweek/4)+1
month2week = kronecker(1:nmonth,rep(1,4))[1:nweek]

salepctmonth = array(0,c(nmonth,2))

for(i in 1:nmonth) {
  salepctmonth[i,1] = mean(pf1w$sale[pf1w$UPC %in% upckeep & pf1w$erimweek %in% allweek[sum(allweek < mweek)+which(month2week == i)]])
  salepctmonth[i,2] = mean(pf2w$sale[pf2w$UPC %in% upckeep & pf2w$erimweek %in% allweek[sum(allweek < mweek)+which(month2week == i)]])
}

# bimonthly

salepctbimonth = array(0,c(as.integer(nmonth/2)+1,2))

for(i in 1:as.integer(nmonth/2)) {
  salepctbimonth[i,1] = mean(pf1w$sale[pf1w$UPC %in% upckeep & pf1w$erimweek %in% allweek[sum(allweek < mweek)+which(month2week %in% c(2*i-1,2*i))]])
  salepctbimonth[i,2] = mean(pf2w$sale[pf2w$UPC %in% upckeep & pf2w$erimweek %in% allweek[sum(allweek < mweek)+which(month2week %in% c(2*i-1,2*i))]])
}

if(nmonth%%2 != 0) {
  salepctbimonth[as.integer(nmonth/2)+1,1] = mean(pf1w$sale[pf1w$UPC %in% upckeep & pf1w$erimweek %in% allweek[sum(allweek < mweek)+which(month2week == as.integer(nmonth/2)+1)]])
  salepctbimonth[as.integer(nmonth/2)+1,2] = mean(pf2w$sale[pf2w$UPC %in% upckeep & pf2w$erimweek %in% allweek[sum(allweek < mweek)+which(month2week == as.integer(nmonth/2)+1)]])
}

pf1wu = pf1w[pf1w$UPC %in% upckeep,]

pf1wu = pf1wu[order(pf1wu$SID,pf1wu$UPC,pf1wu$erimweek),]

wkindexes = match(pf1wu$erimweek,allweek)

# compute conditional probabilities of promotions

saleowncprobs = list(matrix(0,nrow=2,ncol=2),matrix(0,nrow=2,ncol=2))
salecompcprobs = list(matrix(0,nrow=2,ncol=2),matrix(0,nrow=2,ncol=2))

brids = c(8,11)

lead.pf1w = pf1w[-1,]
lag.pf1w = pf1w[-nrow(pf1w),]

# create stacked data

upclist = c(17800000120, 17800000240, 79100002450, 79100002800)
upcbrand = c(8,8,11,11)

allstid = unique(pf1w$SID)

pf1wstack = data.frame(cbind(kronecker(allstid,rep(1,length(allweek))),rep(allweek,length(allstid))))
colnames(pf1wstack) = c("SID","erimweek")

for(i in 1:length(upclist)) {
  temp = pf1w[pf1w$UPC == upclist[i],c("SID","erimweek","sale")]
  colnames(temp)[3] = paste("sale",i,sep="")
  pf1wstack = merge(pf1wstack,temp,all.x=TRUE)
}

pf1wstack = pf1wstack[order(pf1wstack$SID,pf1wstack$erimweek),]

for(i in 1:length(brids)) {
  pf1wstack[[paste("brsale",i,sep="")]] = NA
  for(j in which(upcbrand==brids[i])) {
    inds = is.na(pf1wstack[[paste("brsale",i,sep="")]])
    pf1wstack[[paste("brsale",i,sep="")]][inds] = pf1wstack[[paste("sale",j,sep="")]][inds]
    inds = !is.na(pf1wstack[[paste("brsale",i,sep="")]]) & !is.na(pf1wstack[[paste("sale",j,sep="")]])
    pf1wstack[[paste("brsale",i,sep="")]][inds] = pf1wstack[[paste("brsale",i,sep="")]][inds] | pf1wstack[[paste("sale",j,sep="")]][inds]
  }
}

for(i in 1:length(brids)) {
  inds = is.na(pf1wstack[[paste("brsale",i,sep="")]])
  pf1wstack = pf1wstack[!inds,]
}

lead.stack = pf1wstack[-1,]
lag.stack = pf1wstack[-nrow(pf1wstack),]


for(i in 2:length(allweek)) {
  for(j in 1:length(brids)) {
    inds = lead.pf1w$erimweek == allweek[i] & lag.pf1w$erimweek == allweek[i-1] &
        lead.pf1w$SID == lag.pf1w$SID & lead.pf1w$UPC == lag.pf1w$UPC & lead.pf1w$brandid == brids[j]
    saleowncprobs[[j]][1,1] = saleowncprobs[[j]][1,1] + sum(lead.pf1w$sale[inds] & lag.pf1w$sale[inds])
    saleowncprobs[[j]][1,2] = saleowncprobs[[j]][1,2] + sum(!lead.pf1w$sale[inds] & lag.pf1w$sale[inds])
    saleowncprobs[[j]][2,1] = saleowncprobs[[j]][2,1] + sum(lead.pf1w$sale[inds] & !lag.pf1w$sale[inds])
    saleowncprobs[[j]][2,2] = saleowncprobs[[j]][2,2] + sum(!lead.pf1w$sale[inds] & !lag.pf1w$sale[inds])

    otherbrand = 3-j

    inds = lead.stack$erimweek == allweek[i] & lag.stack$erimweek == allweek[i-1] &
        lead.stack$SID == lag.stack$SID

    salecompcprobs[[j]][1,1] = salecompcprobs[[j]][1,1] + sum(lead.stack[[paste("brsale",j,sep="")]][inds] & lag.stack[[paste("brsale",otherbrand,sep="")]][inds])
    salecompcprobs[[j]][1,2] = salecompcprobs[[j]][1,2] + sum(!lead.stack[[paste("brsale",j,sep="")]][inds] & lag.stack[[paste("brsale",otherbrand,sep="")]][inds])
    salecompcprobs[[j]][2,1] = salecompcprobs[[j]][2,1] + sum(lead.stack[[paste("brsale",j,sep="")]][inds] & !lag.stack[[paste("brsale",otherbrand,sep="")]][inds])
    salecompcprobs[[j]][2,2] = salecompcprobs[[j]][2,2] + sum(!lead.stack[[paste("brsale",j,sep="")]][inds] & !lag.stack[[paste("brsale",otherbrand,sep="")]][inds])

  }
}

for(j in 1:length(brids)) {
  saleowncprobs[[j]] = saleowncprobs[[j]]/rowSums(saleowncprobs[[j]])
  salecompcprobs[[j]] = salecompcprobs[[j]]/rowSums(salecompcprobs[[j]])
}

cprobtabown = c("Sale","Nonsale")

cprobtabcomp = c("Sale","Nonsale")

sigfig = 2
for(i in 1:2) {
  for(j in 1:2) {
    for(k in 1:2) {
      cprobtabown[i] = paste(cprobtabown[i]," & ",round(saleowncprobs[[j]][i,k],sigfig),sep="")
      cprobtabcomp[i] = paste(cprobtabcomp[i]," & ",round(salecompcprobs[[j]][i,k],sigfig),sep="")
    }
  }
  cprobtabown[i] = paste(cprobtabown[i],"\\\\")
  cprobtabcomp[i] = paste(cprobtabcomp[i],"\\\\")
}

write.table(cprobtabown,paste(tablepath,"tuna_saleprobs.tex",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)
write.table(cprobtabcomp,paste(tablepath,"tuna_compprobs.tex",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

# make graphs and tables of prices

# price and quantity graphs: the 2 most popular stores in each market: 7 and 30

inds = pf1w$SID == 29 & pf1w$brandid == 11 & pf1w$oilwtr == 2

op <- par(mfrow=c(2,1))
plot(pf1w$WEEK[inds],pf1w$price[inds]*100,type="l",ylab="Price (cents)",xlab="Week",main="Price and Quantity of Starkist 6 oz in Water",col=c("blue"))
plot(pf1w$WEEK[inds],pf1w$UNITS[inds],type="l",ylab="Quantity",xlab="Week",col=c("blue"))
par(op)
dev.copy2eps(file=paste(figpath,"pricequantitymkt1.eps",sep=""))
dev.copy2pdf(file=paste(figpath,"pricequantitymkt1.pdf",sep=""))

inds = pf2w$SID == 30 & pf2w$brandid == 11 & pf2w$oilwtr == 2

op <- par(mfrow=c(2,1))
plot(pf2w$WEEK[inds],pf2w$price[inds]*100,type="l",ylab="Price (cents)",xlab="Week",main="Price and Quantity of Starkist",col=c("blue"))
plot(pf2w$WEEK[inds],pf2w$UNITS[inds],type="l",ylab="Quantity",xlab="Week",col=c("blue"))
par(op)
dev.copy2eps(file=paste(figpath,"pricequantitymkt2.eps",sep=""))
dev.copy2pdf(file=paste(figpath,"pricequantitymkt2.pdf",sep=""))

# save output files for step 2

save(pf1w,pf2w,file=paste(outputpath,"weeklyprices_",product,".RData",sep=""))
save(bdatabig,file=paste(outputpath,"bdatabig_",product,".RData",sep=""))
