# load in the erim data to look at it
# this should replicate the price construction from the fortran code

rm(list=ls())

product = "sssoup"
inputpath = "C:/Users/MattOsborne/Dropbox/price index paper/input data/"            # put raw data here
outputpath = "C:/Users/MattOsborne/Dropbox/price index paper/step 1 output files/"  # intermediate output files go here


hhpurch = read.fwf(file=paste(inputpath,product,"/",product,"_f1.dat",sep=""),width=c(2,2,6,1,1,8,13,4,1,5,7,7,3,3,5,3,5,11,4,rep(1,10),3),stringsAsFactors=FALSE)

colnames(hhpurch) = c("cellident","market","week","day","tripnum","hhid","upc","storeid","storetype","units","price","weight","couponcode",
                      "storecouponunits","storecouponvalue","mancouponunits","mancouponvalue","eqfactor","rsln",
                      "eadc","fadc","iadc","odc","adcode","adtype","inadcc","popcode","spcode","cf","filler")

stdata = read.fwf(file=paste(inputpath,product,"/",product,"_f4.dat",sep=""),width=c(2,4,13,6,rep(1,8),7,2,2),stringsAsFactors=FALSE)
colnames(stdata) = c("market","storeid","upc","week","eadc","fadc","iadc","odc","adcode","inadcc","popcode","spcode",
                     "price","pmultiple","filler")
branddata = read.fwf(file=paste(inputpath,product,"/",product,"_f5.dat",sep=""),width=c(13,30,2,6,7,3,11,4),stringsAsFactors=FALSE)
colnames(branddata) = c("upc","desc","wtcode","wtdesc","wtamt","multipack","eqfactor","rsln")

save(hhpurch,stdata,branddata,file=paste(outputpath,"erim_",product,".RData",sep=""))

hhvisit = read.fwf(file=paste(inputpath,product,"/",product,"_f2.dat",sep=""),width=c(2,4,1,8,6,7),stringsAsFactors=FALSE)
colnames(hhvisit) = c("market","storeid","storetype","hhid","week","dollars")
save(hhvisit,file=paste(outputpath,"visits_",product,".RData",sep=""))

# make a small fix so flavors don't get mixed up

branddata$desc[branddata$upc == 5100001021] = "CAMP CD VEG SP                "

stdata$price = stdata$price/100

allstids = unique(stdata$storeid)
allstids = allstids[order(allstids)]

stchain = cbind(allstids,rep(0,length(allstids)),rep(0,length(allstids)))
stchain = stchain[order(stchain[,1]),]
colnames(stchain) = c("storeid","market","chain")

# assign markets

for(i in 1:length(allstids)) {
  stchain[i,2] = max(stdata$market[stdata$storeid==allstids[i]])
}

# assign chains
stchain[1,3] = 1
stchain[2,3] = 4
stchain[3,3] = 2
stchain[4,3] = 2
stchain[5,3] = 1
stchain[6,3] = 1
stchain[7,3] = 2
stchain[8,3] = 1
stchain[9,3] = 1
stchain[10,3] = 3
stchain[11,3] = 1
stchain[12,3] = 1
stchain[13,3] = 1
stchain[14,3] = 1
stchain[15,3] = 5
stchain[16,3] = 2
stchain[17,3] = 5
stchain[18,3] = 2
stchain[19,3] = 2
stchain[20,3] = 5
stchain[21,3] = 3
stchain[22,3] = 3
stchain[23,3] = 5
stchain[24,3] = 3
stchain[25,3] = 5
stchain[26,3] = 3
stchain[27,3] = 3
stchain[28,3] = 5
stchain[29,3] = 4
stchain[30,3] = 4
stchain[31,3] = 3
stchain[32,3] = 1
stchain[33,3] = 4
stchain[34,3] = 2
stchain[35,3] = 4
stchain[36,3] = -1

# look at purchase shares

brandnames = c("CAMP","CTL BR","SWNSN")

flavor = c("CRM OF MSH","CD TOM","CH NDL","CRM OF CH","CRM OF CELRY","VEG BF","BN/BCN","CHEDDAR CHEESE","CLAM CHOW",
           "CH/RC","CRM OF POT","HMSTY CH NDL","VEGETARIAN VEG","CK O-F BN/HM","CH BR","CD VEG SP","CH GMB",
           "SPLT PEA/HM & BCN")

bdatabig = branddata
bdatabig$brandid = 0

j=0
for(i in brandnames) {
  j=j+1
  bdatabig$brandid[grep(i,bdatabig$desc)] = j
}

bdatabig$flavor = 0

j=0
for(i in flavor) {
  j=j+1
  bdatabig$flavor[grep(i,bdatabig$desc)] = j
}

# identify ready mix packets (note that the restrictions on package size will probably cut out most of these)

packet = union(grep("I-MX",bdatabig$desc) , grep("R-MX",bdatabig$desc))

bdatabig$packet = 0
bdatabig$packet[packet] = 1

bdatabig$wtcode <- NULL
bdatabig$wtdesc <- NULL
bdatabig$eqfactor <- NULL
bdatabig$rsln <- NULL

# merge brand info data into purchase data

hhpurch$filler <- NULL

hhpurch = merge(hhpurch,bdatabig,all.x=TRUE)
hhpurch$brandname = "unassigned"
hhpurch$brandname[hhpurch$brandid>0] = brandnames[hhpurch$brandid[hhpurch$brandid > 0]]

hhpurch$uprice = hhpurch$price/pmax(100,hhpurch$units)  # 1 observation that is 0

print(table(hhpurch$flavor))
print(summary(lm(uprice~factor(brandid)+factor(flavor),data=hhpurch)))

# create store level prices by week

allweek = unique(hhpurch$week)
allweek = allweek[order(allweek)]

stdata = merge(stdata,bdatabig,all.x=TRUE)
stdata = stdata[order(stdata$storeid,stdata$week,stdata$brandid,stdata$flavor,stdata$upc),]

stprices = array(0,c(length(allstids),length(allweek),length(brandnames),length(flavor),2))

# remove purchases that are not in the estimation data

# I'm taking out a few upcs.  Campbell's special request chicken noodle (51000058970, 359 purchases), sp rq cream of mushroom (286 purch)
# CAMP CK RTS CK CH NDL/MSH (1190)

badupc = c(51000058970,51000060070,51000005720)

badpurch = hhpurch$brandid == 0 | hhpurch$flavor == 0 | hhpurch$weight < 10500 | hhpurch$weight > 11500 | hhpurch$packet == 1 | hhpurch$upc %in% badupc

hhpurch = hhpurch[!badpurch,]

badst = stdata$brandid == 0 | stdata$flavor == 0 | stdata$wtamt < 10500 | stdata$wtamt > 11500 | stdata$packet == 1 | stdata$upc %in% badupc

stdata = stdata[!badst,]

# first compute for each household maximum day we observe hh.

hhpurch = hhpurch[order(hhpurch$hhid,hhpurch$week,hhpurch$brandid,hhpurch$flavor,hhpurch$day),]

hhprices = aggregate(hhpurch$uprice,by=list(hhpurch$storeid,hhpurch$week,hhpurch$day,hhpurch$brandid,hhpurch$flavor,hhpurch$spcode),FUN=min)

colnames(hhprices) = c("storeid","week","day","brandid","flavor","spcode","uprice")

# for non special prices take price at end of week

hhp1 = hhprices[hhprices$spcode == "N",]

maxday = aggregate(hhp1$day,by=list(hhp1$storeid,hhp1$week,hhp1$brandid,hhp1$flavor),FUN=max)

colnames(maxday) = c("storeid","week","brandid","flavor","maxday")

hhp1 = merge(hhp1,maxday,all.x=TRUE)

hhp1 = hhp1[hhp1$day==hhp1$maxday,]

# sometimes we have multiple households who purchse on the same day from the same store - we'll avg prices here

hhp1 = aggregate(hhp1$uprice,by=list(hhp1$storeid,hhp1$week,hhp1$brandid,hhp1$flavor),FUN=mean)

colnames(hhp1) = c("storeid","week","brandid","flavor","uprice")

hhp1 = hhp1[order(hhp1$storeid,hhp1$week,hhp1$brandid,hhp1$flavor),]

# for special prices take min across the week

hhp2 = hhprices[hhprices$spcode == "Y",]

price2 = aggregate(hhp2$uprice,by=list(hhp2$storeid,hhp2$week,hhp2$brandid,hhp2$flavor),FUN=min)  #in tuna I used hhp2 but here I use prices2 because we get more observation
colnames(price2) = c("storeid","week","brandid","flavor","uprice")

hhp2 = hhp2[order(hhp2$storeid,hhp2$week,hhp2$brandid,hhp2$flavor),]

for(stid in 1:length(allstids)) {
    for(week in 1:length(allweek)) {
        for(br in 1:length(brandnames)) {
            for(i in 1:length(flavor)) {
                inds = stdata$storeid==allstids[stid] & stdata$week==allweek[week] & stdata$brandid==br & stdata$flavor==i & stdata$spcode == "N"
                if(sum(inds) > 0) {
                    stprices[stid,week,br,i,1] = min(stdata$price[inds]) # I put the min in here because as far as I can tell the dupes are store brands - there seems to be no overlap in UPCS across stores for market 1 - market 2 is stranger though
                }
                inds = stdata$storeid==allstids[stid] & stdata$week==allweek[week] & stdata$brandid==br & stdata$flavor==i & stdata$spcode == "Y"
                if(sum(inds) > 0) {
                    stprices[stid,week,br,i,2] = min(stdata$price[inds])
                }
                inds = hhp1$storeid==allstids[stid] & hhp1$week==allweek[week] & hhp1$brandid==br & hhp1$flavor==i
                if(sum(inds) > 0) {
                    stprices[stid,week,br,i,1] = hhp1$uprice[inds]
                }
                inds = price2$storeid==allstids[stid] & price2$week==allweek[week] & price2$brandid==br & price2$flavor==i
                if(sum(inds) > 0) {
                    if(stprices[stid,week,br,i,2] > 0) {
                        stprices[stid,week,br,i,2] = min(stprices[stid,week,br,i,2],price2$uprice[inds])
                    } else {
                        stprices[stid,week,br,i,2] = price2$uprice[inds]
                    }
                }
            }
        }
    }
}

save(stprices,file=paste(outputpath,"stprices_",product,".RData",sep=""))
