// short function to compute length of promotional/nonpromotional periods

#include<math.h>
#include<R_ext/PrtUtil.h>
#include<R.h>
#include<Rinternals.h>

// work function to get list element corresponding to str
// returns NULL if str not found

SEXP getListElement(SEXP list, const char *str) {
  SEXP elmt = R_NilValue, names = getAttrib(list, R_NamesSymbol);
  for (int i = 0; i < length(list); i++)
    if(strcmp(CHAR(STRING_ELT(names, i)), str) == 0) {
      elmt = VECTOR_ELT(list, i);
      break;
    }
  return elmt;
}

// compute household inventory measure

SEXP computeinv(SEXP hhallw, SEXP hhcrate) {

  double * week = REAL(coerceVector(getListElement(hhallw,"week"),REALSXP));
  int * hhid = INTEGER(coerceVector(getListElement(hhallw,"hhid"),INTSXP));
  double * units = REAL(coerceVector(getListElement(hhallw,"units"),REALSXP));
  double * hhcrate1 = REAL(hhcrate);

  int nobs = length(getListElement(hhallw,"week"));

  SEXP hhinv;
  PROTECT(hhinv = allocVector(REALSXP,nobs));
  double * rinv = REAL(hhinv);
  memset(rinv,0,sizeof(double)*nobs);
  double cinv = 0, crate = 0;

  int hh = -1;

  for(int n=0;n<nobs;n++) {
    if(n == 0 || hhid[n] != hhid[n-1]) {
      cinv = 0;
      hh++;
      crate = hhcrate1[hh];
    }
    rinv[n] = cinv;
    if(n < nobs - 1 && hhid[n] == hhid[n+1]) {
      double wkdelta = 0;
      if((int)(week[n+1]/100) == (int)(week[n]/100)) {
        wkdelta = week[n+1]-week[n];
      } else {
        double ystub = (double)((int)(week[n]/100));
        wkdelta = (ystub*100+52-week[n])+(week[n+1]-(ystub+1)*100);
      }
      cinv += units[n] - crate*wkdelta;
    }
  }

  UNPROTECT(1);
  return(hhinv);

}

// routine to fill in missing regular prices - used in soup data construction

SEXP fillregprices(SEXP stprices) {

  int nstore = INTEGER(getAttrib(stprices, R_DimSymbol))[0];
  int nweek = INTEGER(getAttrib(stprices, R_DimSymbol))[1];
  int nbrand = INTEGER(getAttrib(stprices, R_DimSymbol))[2];
  int nflavor = INTEGER(getAttrib(stprices, R_DimSymbol))[3];
  double * st = REAL(stprices);

  SEXP reg, dpr;
  PROTECT(reg = allocVector(REALSXP,nstore*nweek*nbrand*nflavor));
  PROTECT(dpr = allocVector(REALSXP,nstore*nweek*nbrand*nflavor));
  double * rreg = REAL(reg);
  double * rdpr = REAL(dpr);
  memset(rreg,0,sizeof(double)*nstore*nweek*nbrand*nflavor);
  memset(rdpr,0,sizeof(double)*nstore*nweek*nbrand*nflavor);

  for(int s=0;s<nstore;s++) {
    for(int b=0;b<nbrand;b++) {
      for(int f=0;f<nflavor;f++) {
        double plast = 0;
        for(int w=0;w<nweek;w++) {
          if(plast > 0 || st[s+nstore*(w+nweek*(b+nbrand*(f+nflavor*0)))] > 0) {
            if(st[s+nstore*(w+nweek*(b+nbrand*(f+nflavor*0)))] == 0) {
              rreg[s+nstore*(w+nweek*(b+nbrand*f))] = plast;
            } else {
              rreg[s+nstore*(w+nweek*(b+nbrand*f))] = st[s+nstore*(w+nweek*(b+nbrand*(f+nflavor*0)))];
            }
          }
          if(st[s+nstore*(w+nweek*(b+nbrand*(f+nflavor*1)))] == 0) {
            rdpr[s+nstore*(w+nweek*(b+nbrand*f))] = rreg[s+nstore*(w+nweek*(b+nbrand*f))];
          } else {
            rdpr[s+nstore*(w+nweek*(b+nbrand*f))] = st[s+nstore*(w+nweek*(b+nbrand*(f+nflavor*1)))];
          }
          if(st[s+nstore*(w+nweek*(b+nbrand*(f+nflavor*0)))] > 0)
            plast = st[s+nstore*(w+nweek*(b+nbrand*(f+nflavor*0)))];
        }
      }
    }
  }

  SEXP res, dim;

  PROTECT(dim = allocVector(INTSXP,4));
  INTEGER(dim)[0] = nstore; INTEGER(dim)[1] = nweek; INTEGER(dim)[2] = nbrand; INTEGER(dim)[3] = nflavor;
  setAttrib(reg, R_DimSymbol, dim);
  setAttrib(dpr, R_DimSymbol, dim);

  const char * resnames[] = {"REG","DPR",""};
  PROTECT(res = mkNamed(VECSXP,resnames));
  SET_VECTOR_ELT(res,0,reg);
  SET_VECTOR_ELT(res,1,dpr);

  UNPROTECT(4);
  return(res);

}
