#include<math.h>
#include<R_ext/PrtUtil.h>
#include<R.h>
#include<Rinternals.h>


SEXP getListElement(SEXP list, const char *str) {
  SEXP elmt = R_NilValue, names = getAttrib(list, R_NamesSymbol);
  for (int i = 0; i < length(list); i++)
    if(strcmp(CHAR(STRING_ELT(names, i)), str) == 0) {
      elmt = VECTOR_ELT(list, i);
      break;
    }
  return elmt;
}

// Index checking function

void indcheck(int ind, int lb, int ub, const char * msg,...) {

  if(ind < lb || ind > ub-1) {
    va_list args;
    va_start(args,msg);

    char buf[100];
    vsnprintf(buf,100,msg,args);
    error("%s. Index out of bounds.  Index value %d: Bounds: [%d,%d] \n",buf,ind,lb,ub);

    va_end(args);
  }

}


double dmax(double a, double b) {

  if(a > b) {
    return(a);
  } else {
    return(b);
  }

}

// create choice set
// note that alphas should be a list where the first el corresponds to all the possible combos for 2.

SEXP makecset(SEXP numprods, SEXP alphas, SEXP nchoices) {

  int maxchoices = 3;

  int np = *INTEGER(coerceVector(numprods,INTSXP));

  int nalfa = length(alphas);

  double * alfas[nalfa];
  int lalfa[nalfa];

  for(int i=0;i<nalfa;i++) {
    alfas[i] = REAL(VECTOR_ELT(alphas,i));
    lalfa[i] = INTEGER(getAttrib(VECTOR_ELT(alphas,i), R_DimSymbol))[0];
  }

  int nc = *INTEGER(coerceVector(nchoices,INTSXP));

  if(nc==0) error("number of possible choices must be at least 1\n");

  if(nc > maxchoices) error("choice set generation is only possible for up to %d SKUs\n",maxchoices);

  // first, we need to count how many choices there will be

  int nprotect = 0;

  // choices for 1 SKU only

  SEXP totchoices;

  PROTECT(totchoices = allocVector(VECSXP,nc));
  nprotect++;

  SET_VECTOR_ELT(totchoices,0,allocVector(INTSXP,1));

  *INTEGER(VECTOR_ELT(totchoices,0)) = np+1;

  // for 2 SKUs

  if(nc >= 2) {

    SET_VECTOR_ELT(totchoices,1,allocVector(INTSXP,lalfa[0]));

    for(int n=0;n<lalfa[0];n++) {
      int tc = 0;
      for(int i=0;i<np;i++) {
        for(int j=i+1;j<np;j++) {
          tc++;
          if(n > 0) tc++;
        }
      }
      INTEGER(VECTOR_ELT(totchoices,1))[n] = tc;
    }

  }

  // for 3 SKUs

  if(nc >= 3) {

    SET_VECTOR_ELT(totchoices,2,allocVector(INTSXP,lalfa[1]));

    for(int n=0;n<lalfa[1];n++) {
      int tc=0;
      for(int i=0;i<np;i++) {
        for(int j=i+1;j<np;j++) {
          for(int k=j+1;k<np;k++) {
            tc++;
            if(n > 0) tc++;
          }
        }
      }
      INTEGER(VECTOR_ELT(totchoices,2))[n] = tc;
    }

  }

  SEXP csetret, alpharet;
  PROTECT(csetret = allocVector(VECSXP,nc));
  nprotect++;
  PROTECT(alpharet = allocVector(VECSXP,nc));
  nprotect++;

  int tc1 = INTEGER(VECTOR_ELT(totchoices,0))[0];

  SET_VECTOR_ELT(csetret,0,allocVector(INTSXP,maxchoices*tc1));
  SET_VECTOR_ELT(alpharet,0,allocVector(REALSXP,maxchoices*tc1));

  int * csr = INTEGER(VECTOR_ELT(csetret,0));
  double * alfaret = REAL(VECTOR_ELT(alpharet,0));

  memset(csr,0,sizeof(int)*maxchoices*tc1);
  memset(alfaret,0,sizeof(double)*maxchoices*tc1);

  for(int i=0;i<np;i++) {
    csr[1+i+tc1*0] = i+1;
    alfaret[1+i+tc1*0] = 1.0;
  }

  SEXP dim1;
  PROTECT(dim1 = allocVector(INTSXP,2));
  nprotect++;
  INTEGER(dim1)[0] = tc1;
  INTEGER(dim1)[1] = maxchoices;
  setAttrib(VECTOR_ELT(csetret,0), R_DimSymbol, dim1);
  setAttrib(VECTOR_ELT(alpharet,0), R_DimSymbol, dim1);

  SEXP dim2;
  PROTECT(dim2 = allocVector(VECSXP,lalfa[0]));
  nprotect++;

  if(nc >=2) {
    SET_VECTOR_ELT(csetret,1,allocVector(VECSXP,lalfa[0]));
    SET_VECTOR_ELT(alpharet,1,allocVector(VECSXP,lalfa[0]));

    for(int n=0;n<lalfa[0];n++) {
      int tc = INTEGER(VECTOR_ELT(totchoices,1))[n];
      SET_VECTOR_ELT(VECTOR_ELT(csetret,1),n,allocVector(INTSXP,maxchoices*tc));
      SET_VECTOR_ELT(VECTOR_ELT(alpharet,1),n,allocVector(REALSXP,maxchoices*tc));
      csr = INTEGER(VECTOR_ELT(VECTOR_ELT(csetret,1),n));
      alfaret = REAL(VECTOR_ELT(VECTOR_ELT(alpharet,1),n));
      memset(csr,0,sizeof(int)*maxchoices*tc);
      memset(alfaret,0,sizeof(double)*maxchoices*tc);
      int indx=0;
      for(int i=0;i<np;i++) {
        for(int j=i+1;j<np;j++) {
          csr[indx+tc*0] = i+1;
          csr[indx+tc*1] = j+1;
          alfaret[indx+tc*0] = alfas[0][n+lalfa[0]*0];
          alfaret[indx+tc*1] = 1.0-alfas[0][n+lalfa[0]*0];
          indx++;
          if(n > 0) {
            csr[indx+tc*0] = i+1;
            csr[indx+tc*1] = j+1;
            alfaret[indx+tc*0] = 1.0-alfas[0][n+lalfa[0]*0];
            alfaret[indx+tc*1] = alfas[0][n+lalfa[0]*0];
            indx++;
          }
        }
      }
      SET_VECTOR_ELT(dim2,n,allocVector(INTSXP,2));
      INTEGER(VECTOR_ELT(dim2,n))[0] = tc;
      INTEGER(VECTOR_ELT(dim2,n))[1] = maxchoices;
      setAttrib(VECTOR_ELT(VECTOR_ELT(csetret,1),n), R_DimSymbol, VECTOR_ELT(dim2,n));
      setAttrib(VECTOR_ELT(VECTOR_ELT(alpharet,1),n), R_DimSymbol, VECTOR_ELT(dim2,n));

    }

  }
  
  SEXP dim3;
  PROTECT(dim3 = allocVector(VECSXP,lalfa[1]));
  nprotect++;

  if(nc >=3) {
    SET_VECTOR_ELT(csetret,2,allocVector(VECSXP,lalfa[1]));
    SET_VECTOR_ELT(alpharet,2,allocVector(VECSXP,lalfa[1]));

    for(int n=0;n<lalfa[1];n++) {
      int tc = INTEGER(VECTOR_ELT(totchoices,2))[n];
      SET_VECTOR_ELT(VECTOR_ELT(csetret,2),n,allocVector(INTSXP,maxchoices*tc));
      SET_VECTOR_ELT(VECTOR_ELT(alpharet,2),n,allocVector(REALSXP,maxchoices*tc));
      csr = INTEGER(VECTOR_ELT(VECTOR_ELT(csetret,2),n));
      alfaret = REAL(VECTOR_ELT(VECTOR_ELT(alpharet,2),n));
      memset(csr,0,sizeof(int)*maxchoices*tc);
      memset(alfaret,0,sizeof(double)*maxchoices*tc);
      int indx=0;
      for(int i=0;i<np;i++) {
        for(int j=i+1;j<np;j++) {
          for(int k=j+1;k<np;k++) {
            csr[indx+tc*0] = i+1;
            csr[indx+tc*1] = j+1;
            csr[indx+tc*2] = k+1;
            alfaret[indx+tc*0] = alfas[1][n+lalfa[1]*0];
            alfaret[indx+tc*1] = alfas[1][n+lalfa[1]*1];
            alfaret[indx+tc*2] = 1.0-alfas[1][n+lalfa[1]*0]-alfas[1][n+lalfa[1]*1];
            indx++;
            if(n > 0) {
              csr[indx+tc*0] = i+1;
              csr[indx+tc*1] = j+1;
              csr[indx+tc*2] = k+1;
              alfaret[indx+tc*0] = alfas[1][n+lalfa[1]*0];
              alfaret[indx+tc*1] = alfas[1][n+lalfa[1]*1];
              alfaret[indx+tc*2] = 1.0-alfas[1][n+lalfa[1]*0]-alfas[1][n+lalfa[1]*1];
              indx++;
            }
          }
        }
      }
      SET_VECTOR_ELT(dim3,n,allocVector(INTSXP,2));
      INTEGER(VECTOR_ELT(dim3,n))[0] = tc;
      INTEGER(VECTOR_ELT(dim3,n))[1] = maxchoices;
      setAttrib(VECTOR_ELT(VECTOR_ELT(csetret,2),n), R_DimSymbol, VECTOR_ELT(dim3,n));
      setAttrib(VECTOR_ELT(VECTOR_ELT(alpharet,2),n), R_DimSymbol, VECTOR_ELT(dim3,n));

    }

  }

  SEXP res;
  PROTECT(res = allocVector(VECSXP,2));
  nprotect++;

  SET_VECTOR_ELT(res,0,csetret);
  SET_VECTOR_ELT(res,1,alpharet);

  UNPROTECT(nprotect);

  return(res);

}


// compute brand level log likelihood
// if ivflag is TRUE, we return inclusive values instead of log likelihood

SEXP llbrand(SEXP coefs, SEXP hhweek, SEXP choiceset, SEXP numprods, SEXP alphas, SEXP ivflag, SEXP indcoefs, SEXP nchoices, SEXP niv1) {

  int maxchoices = *INTEGER(coerceVector(nchoices,INTSXP));

  int niv = *INTEGER(coerceVector(niv1,INTSXP));

  int nvar = INTEGER(getAttrib(hhweek, R_DimSymbol))[0];
  int nweek = INTEGER(getAttrib(hhweek, R_DimSymbol))[1];
  int nhhs = INTEGER(getAttrib(hhweek, R_DimSymbol))[2];

  int np = *INTEGER(coerceVector(numprods,INTSXP));

  int calciv = *INTEGER(ivflag);
  int icflag = *INTEGER(indcoefs);

  int lcoef = 0;

  if(icflag) {
    lcoef = INTEGER(getAttrib(coefs, R_DimSymbol))[0];
  } else {
    lcoef = length(coefs);
  }

  double * dhh = REAL(hhweek);

  double * x = REAL(coefs);

  int nalfa = length(alphas);

  double * alfasmall[nalfa];
  int lalfa[nalfa];
  int * csetl[nalfa];
  int ** csetbig[nalfa];
  double ** alfabig[nalfa];

  for(int i=0;i<nalfa;i++) {
    alfasmall[i] = REAL(VECTOR_ELT(alphas,i));
    lalfa[i] = INTEGER(getAttrib(VECTOR_ELT(alphas,i), R_DimSymbol))[0];
    csetl[i] = (int *) R_alloc(lalfa[i], sizeof(int));
    csetbig[i] = (int **) R_alloc(lalfa[i], sizeof(int*));
    alfabig[i] = (double **) R_alloc(lalfa[i], sizeof(double*));
    for(int j=0;j<lalfa[i];j++) {
      csetl[i][j] = INTEGER(getAttrib(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(choiceset,0),i+1),j), R_DimSymbol))[0];
      csetbig[i][j] = INTEGER(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(choiceset,0),i+1),j));
      alfabig[i][j] = REAL(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(choiceset,1),i+1),j));
    }
  }

  int nprotect = 0;
  SEXP ll, iv;
  PROTECT(ll=allocVector(REALSXP,nhhs));
  nprotect++;
  double * rll = REAL(ll);
  memset(rll,0,sizeof(double)*nhhs);

  double * riv;
  if(calciv) {
    PROTECT(iv=allocVector(REALSXP,nhhs*nweek*niv));
    nprotect++;
    riv = REAL(iv);
    memset(riv,0,sizeof(double)*nhhs*nweek*niv);
  }

  double u;

  for(int hh=0;hh<nhhs;hh++) {
    for(int n=0;n<nweek;n++) {

      if(calciv && dhh[6+nvar*(n+nweek*hh)] > 0) {
        double denom = 0;

        for(int j=0;j<np;j++) {

          double pvec = dhh[7+j+nvar*(n+nweek*hh)]/100.0;
          double fvec = dhh[7+np+j+nvar*(n+nweek*hh)];
          double dvec = dhh[7+2*np+j+nvar*(n+nweek*hh)];

          if(pvec > 0) {
            u = (x[j] - x[lcoef-3]*pvec + x[lcoef-2]*fvec + x[lcoef-1]*dvec);
            denom += exp(u);
          }

        }

        riv[n+nweek*(hh+nhhs*0)] = log(denom);

        int ivind = 0;

        for(int npos=2;npos < maxchoices+1;npos++) {

          int cind = npos-2;

          for(int abucket=0;abucket<lalfa[cind];abucket++) {

            ivind++;

            denom = 0;

            int nj = csetl[cind][abucket];

            for(int j=0;j<nj;j++) {

              double pvec[npos];
              double fvec[npos];
              double dvec[npos];
              int dontskip = 1;

              for(int k=0;k<npos;k++) {
                pvec[k] = dhh[7+csetbig[cind][abucket][j+nj*k]-1+nvar*(n+nweek*hh)]/100.0;
                fvec[k] = dhh[7+np+csetbig[cind][abucket][j+nj*k]-1+nvar*(n+nweek*hh)];
                dvec[k] = dhh[7+2*np+csetbig[cind][abucket][j+nj*k]-1+nvar*(n+nweek*hh)];
                dontskip = dontskip && (pvec[k] > 0);
              }

              if(dontskip) {
                u = 0;
                for(int k=0;k<npos;k++) {
                  u += alfabig[cind][abucket][j+nj*k]*(x[csetbig[cind][abucket][j+nj*k]-1] - x[lcoef-3]*pvec[k] + x[lcoef-2]*fvec[k] + x[lcoef-1]*dvec[k]);
                }
                denom += exp(u);
              }

            }

            indcheck(n+nweek*(hh+nhhs*ivind),0,nhhs*nweek*niv,"n %d; hh %d; ivind: %d",n,hh,ivind);

            riv[n+nweek*(hh+nhhs*ivind)] = log(denom);

          }

        }

      }

      if(dhh[0+nvar*(n+nweek*hh)] > 0) {

        double denom = 0, numer = 0;

        // how many units did you buy

        double tunits = 0;
        int npos = 0;

        for(int k=0;k<maxchoices;k++) {
          tunits += dhh[2*k+1+nvar*(n+nweek*hh)];
          npos += dhh[2*k+1+nvar*(n+nweek*hh)] > 0;
        }

        if(npos == 1) {

          for(int j=0;j<np;j++) {

            double pvec = dhh[7+j+nvar*(n+nweek*hh)]/100.0;
            double fvec = dhh[7+np+j+nvar*(n+nweek*hh)];
            double dvec = dhh[7+2*np+j+nvar*(n+nweek*hh)];

            if(pvec > 0) {
              u = (x[j] - x[lcoef-3]*pvec + x[lcoef-2]*fvec + x[lcoef-1]*dvec);
              denom += exp(u);
              if(j == (int)dhh[nvar*(n+nweek*hh)]-1)
                numer = u;
            }

          }

        } else if (npos >= 2) {

          // what alpha bucket do you fall into

          int abucket = -1;

          int cind = npos-2;

          if(npos == 2) {

            for(int j=0;j<lalfa[cind];j++) {
              if( (int)(tunits*alfasmall[cind][j+lalfa[cind]*0]*100) == (int)dhh[1+nvar*(n+nweek*hh)]*100 || (int)(tunits*alfasmall[cind][j+lalfa[cind]*0]*100) == (int)dhh[3+nvar*(n+nweek*hh)]*100)
                abucket = j;
            }

          } else {
            abucket = 0;
          }

          if(abucket == -1) {
            for(int j=0;j<nvar;j++)
              Rprintf(" %f",dhh[j+nvar*(n+nweek*hh)]);
            Rprintf("\n");
            error("problem with assigning choice at %d, %d\n",n,hh);
          }

          int cassign = 0;

          int nj = csetl[cind][abucket];

          for(int j=0;j<nj;j++) {

            double pvec[npos];
            double fvec[npos];
            double dvec[npos];
            int dontskip = 1;

            for(int k=0;k<npos;k++) {
              pvec[k] = dhh[7+csetbig[cind][abucket][j+nj*k]-1+nvar*(n+nweek*hh)]/100.0;
              fvec[k] = dhh[7+np+csetbig[cind][abucket][j+nj*k]-1+nvar*(n+nweek*hh)];
              dvec[k] = dhh[7+2*np+csetbig[cind][abucket][j+nj*k]-1+nvar*(n+nweek*hh)];
              dontskip = dontskip && (pvec[k] > 0);
            }

            if(dontskip) {
              int chosen = 1;
              u = 0;
              for(int k=0;k<npos;k++) {
                u += alfabig[cind][abucket][j+nj*k]*(x[csetbig[cind][abucket][j+nj*k]-1] - x[lcoef-3]*pvec[k] + x[lcoef-2]*fvec[k] + x[lcoef-1]*dvec[k]);
                chosen = chosen && ( (int)dhh[2*k+1+nvar*(n+nweek*hh)] == (int)(alfabig[cind][abucket][j+nj*k]*tunits) && csetbig[cind][abucket][j+nj*k] == (int)dhh[2*k+nvar*(n+nweek*hh)] );

              }
              if(chosen) {
                numer = u;
                cassign = 1;
              }
              denom += exp(u);

            }

          }

          if(cassign==0) {
            for(int k=0;k<npos;k++) {
              Rprintf("brand %d: %d, %d\n",k,(int)dhh[2*k+nvar*(n+nweek*hh)],(int)dhh[2*k+1+nvar*(n+nweek*hh)]);
            }
            error("error: no choice assigned in %d, %d; %d, %d, %f\n",hh,n,cind,abucket,tunits);
          }

        }

        if(numer > log(denom)) {
          Rprintf("problem in likelihood:\n");
          Rprintf("hh: %d; week: %d; npos: %d; br1: %d, u: %f; denom: %f\n",hh,n,npos,(int)dhh[nvar*(n+nweek*hh)]-1,numer,denom);
          error("");
        }

        rll[hh] += numer - log(denom);

      }

    }

    if(icflag)
      x += lcoef;

  }


  if(calciv) {
    SEXP res, dims;
    PROTECT(res = allocVector(VECSXP,2));
    nprotect++;
    PROTECT(dims = allocVector(INTSXP,3));
    nprotect++;
    INTEGER(dims)[0] = nweek;
    INTEGER(dims)[1] = nhhs;
    INTEGER(dims)[2] = niv;
    setAttrib(iv, R_DimSymbol, dims);
    SET_VECTOR_ELT(res,0,iv);
    SET_VECTOR_ELT(res,1,ll);

    UNPROTECT(nprotect);
    return(res);
  } else {
    UNPROTECT(nprotect);
    return(ll);
  }

}

// test function for inclusive values

SEXP getivtest(SEXP coefs, SEXP hhdata, SEXP choiceset, SEXP numprods, SEXP alphas, SEXP pstart1, SEXP nchoices, SEXP niv1) {

  int maxchoices = *INTEGER(coerceVector(nchoices,INTSXP));

  int niv = *INTEGER(coerceVector(niv1,INTSXP));

  int nobs = length(VECTOR_ELT(hhdata,0));

  int np = *INTEGER(coerceVector(numprods,INTSXP));

  int lcoef = 0;

  lcoef = INTEGER(getAttrib(coefs, R_DimSymbol))[0];

  double * x = REAL(coefs);

  int nalfa = length(alphas);

  int pstart = *INTEGER(coerceVector(pstart1,INTSXP));

  double * allp[np], * allf[np], * alld[np];

  for(int i=0;i<np;i++) {
    allp[i] = REAL(VECTOR_ELT(hhdata,pstart-1+i));
    allf[i] = REAL(VECTOR_ELT(hhdata,pstart-1+i+np));
    alld[i] = REAL(VECTOR_ELT(hhdata,pstart-1+i+2*np));
  }

  int * hhid = INTEGER(coerceVector(getListElement(hhdata,"hhid"),INTSXP));

  double * alfasmall[nalfa];
  int lalfa[nalfa];
  int * csetl[nalfa];
  int ** csetbig[nalfa];
  double ** alfabig[nalfa];

  for(int i=0;i<nalfa;i++) {
    alfasmall[i] = REAL(VECTOR_ELT(alphas,i));
    lalfa[i] = INTEGER(getAttrib(VECTOR_ELT(alphas,i), R_DimSymbol))[0];
    csetl[i] = (int *) R_alloc(lalfa[i], sizeof(int));
    csetbig[i] = (int **) R_alloc(lalfa[i], sizeof(int*));
    alfabig[i] = (double **) R_alloc(lalfa[i], sizeof(double*));
    for(int j=0;j<lalfa[i];j++) {
      csetl[i][j] = INTEGER(getAttrib(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(choiceset,0),i+1),j), R_DimSymbol))[0];
      csetbig[i][j] = INTEGER(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(choiceset,0),i+1),j));
      alfabig[i][j] = REAL(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(choiceset,1),i+1),j));
    }
  }

  int nprotect = 0;
  SEXP iv;

  double * riv;
  PROTECT(iv=allocVector(REALSXP,nobs*niv));
  nprotect++;
  riv = REAL(iv);
  memset(riv,0,sizeof(double)*nobs*niv);

  double u;

  for(int n=0;n<nobs;n++) {

    double denom = 0;

    for(int j=0;j<np;j++) {

      double pvec = allp[j][n];
      double fvec = allf[j][n];
      double dvec = alld[j][n];

      if(pvec > 0) {
        u = (x[j] - x[lcoef-3]*pvec + x[lcoef-2]*fvec + x[lcoef-1]*dvec);
        denom += exp(u);
      }

    }

    riv[n+nobs*0] = log(denom);

    int ivind = 0;

    for(int npos=2;npos < maxchoices+1;npos++) {

      int cind = npos-2;

      for(int abucket=0;abucket<lalfa[cind];abucket++) {

        ivind++;

        denom = 0;

        int nj = csetl[cind][abucket];

        for(int j=0;j<nj;j++) {

          double pvec[npos];
          double fvec[npos];
          double dvec[npos];
          int dontskip = 1;

          for(int k=0;k<npos;k++) {
            pvec[k] = allp[csetbig[cind][abucket][j+nj*k]-1][n];
            fvec[k] = allf[csetbig[cind][abucket][j+nj*k]-1][n];
            dvec[k] = alld[csetbig[cind][abucket][j+nj*k]-1][n];
            dontskip = dontskip && (pvec[k] > 0);
          }

          if(dontskip) {
            u = 0;
            for(int k=0;k<npos;k++) {
              u += alfabig[cind][abucket][j+nj*k]*(x[csetbig[cind][abucket][j+nj*k]-1] - x[lcoef-3]*pvec[k] + x[lcoef-2]*fvec[k] + x[lcoef-1]*dvec[k]);
            }
            denom += exp(u);
          }

        }

        indcheck(n+nobs*ivind,0,nobs*niv,"n %d; ivind: %d",n,ivind);

        riv[n+nobs*ivind] = log(denom);

      }

    }

    if(n < nobs-1 && hhid[n] != hhid[n+1])
      x += lcoef;

  }


  SEXP dims;
  PROTECT(dims = allocVector(INTSXP,2));
  nprotect++;
  INTEGER(dims)[0] = nobs;
  INTEGER(dims)[1] = niv;
  setAttrib(iv, R_DimSymbol, dims);

  UNPROTECT(nprotect);
  return(iv);

}

