# Do the identification simulation

rm(list=ls())

library(markovchain)

path = "C:/Users/MattOsborne/Dropbox/price index paper/code/clean code for paper/Simulation Code/"  #location of code
figstub = "figuresandtables"  # supplementary figures (purchase hazards, etc) go here

tablepath = "C:/Users/MattOsborne/Dropbox/price index paper/output tables and figures/"    #tables for paper

setwd(path)

fform = as.integer(3)
npar = as.integer(8)

ntypes = as.integer(1)
gamm = 1
curv = -0.25
i0 = 0.0
i1 = -0.10
alph = 0.01
cc = -0.25
csmean = 1
csdelta = 0
coeff.reg = matrix(rep(c(gamm,curv,i0,i1,alph,cc,csmean,csdelta),ntypes),nrow=npar,ncol=ntypes)
mtype = 1
coefmat = matrix(coeff.reg[,1],nrow=nrow(coeff.reg),ncol=length(coeff.reg)+1)
cdelta = 0.5

for(i in 1:length(coeff.reg)) {
  coefmat[i,i+1] = coefmat[i,i+1]*(1+cdelta)
}

# basic log specification
coefcols = c(1,2,5,6,7)

cpsave1 = vector("list",length = ncol(coefmat))
cpsave2 = vector("list",length = ncol(coefmat))
ppsave1 = vector("list",length = ncol(coefmat))
ppsave2 = vector("list",length = ncol(coefmat))
psharesave = vector("list",length = ncol(coefmat))
psharepsave = vector("list",length = ncol(coefmat))
ipsave = vector("list",length = ncol(coefmat))
qsave = vector("list",length = ncol(coefmat))

repeatedgrid = TRUE

# exogenous or endogenous consumption model
modeltype = "endog"

# time period to start when analyzing moments
tstart = 50

simvf = TRUE

numprice = 17

pstates = c(20,25,30,35,40,seq(100,200,9))

pprobs = matrix(0,nrow=length(pstates),ncol=length(pstates))

pprobs[1:8,9:11] = 0.25
pprobs[1:8,1:8] = 0.1/8
pprobs[1:8,12:17] = 0.15/6
for(i in 9:16) {
  pprobs[i,i] = 0.4
  pprobs[i,1:5] = 0.3/5
  pprobs[i,6:(i-1)] = 0.2/(i-1-6+1)
  pprobs[i,(i+1):17] = 0.1/(18-(i+1))
}

pprobs[17,17] = 0.4
pprobs[17,1:5] = 0.3/5
pprobs[17,6:16] = 0.3/11

cost = 0.6

delta = 0.95
deltafirm = 0.95

logitsd = 1

ibar = as.integer(11)

dyn.load("simulation_functions.dll")

source("simulation_functions.r")

for(bigrep in coefcols) {

  coeff = matrix(coefmat[,bigrep],nrow=npar,ncol=ntypes)

  set.seed(8371363)

  nt = 200
  nhhs = 10000
  hhtypes = as.integer(sample(1:ntypes,nhhs,TRUE,mtype))

  info = list(ntypes=ntypes,nhhs=nhhs,nt=nt,hhtypes=hhtypes,delta=delta,ibar=ibar)

  sp1 <- simprice(pstates,pprobs[,1:(length(pstates)-1)],10,nhhs,nt)
  simprices = sp1[[1]]
  hhassign = sp1[[2]]

  cdraws.u = vector("list",length=ntypes)

  cdraws = matrix(0.0,nrow=nhhs,ncol=nt)

  if(modeltype == "exog") {
    for(i in 1:ntypes) {
      cdraws.u[[i]] = matrix(runif(sum(hhtypes==i)*nt),nrow=sum(hhtypes==i),ncol=nt)
      cdraws[hhtypes==i,] = as.numeric(cdraws.u[[i]] < coeff[7,i])
    }
  }

  istart = rep(0,nhhs)

  set.seed(8371363)

  simhh <- createdata(coeff,pstates,ntypes,delta,ibar,pprobs,cdraws,simprices,fform,npar,logitsd,info=info)

  agx = apply(simhh$x,2,sum)
  aginv = apply(simhh$inv,2,sum)

                                        # aggregate purchase amount
                                        # interval between purchase
                                        # frequency of purchase
                                        # amount bought on sale
                                        # amount bought on nonsale
                                        # purchase shares per size

  totpurch = sum(simhh$x[,tstart:nt])
  sizeshare = as.data.frame(table(simhh$x[,tstart:nt]),stringsAsFactors=FALSE)
  sizeshare$Freq = sizeshare$Freq/length(simhh$x[,tstart:nt])
  colnames(sizeshare) = c("npackage","share")
  sizesharebyp = as.data.frame(table(simhh$x[,tstart:nt],sp1[[1]]$p[,tstart:nt]),stringsAsFactors=FALSE)
  colnames(sizesharebyp) = c("npackage","price","Freq")
  sizesharebyp$packshareprice = 0
  for(i in pstates) {
    sizesharebyp$packshareprice[as.numeric(sizesharebyp$price) == i] = sizesharebyp$Freq[as.numeric(sizesharebyp$price) == i]/sum(sp1[[1]]$p[,tstart:nt] == i)
  }

  lastp = matrix(-1,nrow=nhhs,ncol=nt)

  for(tt in tstart:nt) {
    lastp[simhh$x[,tt]>0,tt] = 1
    inds = lastp[,tt-1] > -1
    lastp[inds & simhh$x[,tt] == 0,tt] = lastp[inds & simhh$x[,tt] == 0,tt-1]+1
  }

  lastp1 = cbind(rep(-1,nhhs),lastp[,1:(nt-1)])


  # make plots and tables

  pdf(paste(figstub,"/pshares",paste(abs(coeff[,1]),collapse="_"),".pdf",sep=""))

  expr = as.expression(substitute(paste("Parameters: ",gamma," = ",a,", ",beta," = ",b,", ",alpha," = ",c,", ",sc[0]," = ",d,", ", sc[1]," = ",e,", ", CC, " = ", f),
      list(a=coeff[1,1],b=coeff[2,1],c=coeff[5,1],d=coeff[3,1],e=coeff[4,1],f=coeff[6,1])))

  barplot(sizeshare$share,names.arg=sizeshare$npackage, main = expr, ylab = "Share of purchases", xlab = "Number of Packages")
  dev.off()

  pdf(paste(figstub,"/psharesbyprice",paste(abs(coeff[,1]),collapse="_"),".pdf",sep=""))
  prices = c(20,40,127,199)

  xx = cbind(sizesharebyp$packshareprice[as.numeric(sizesharebyp$price) == prices[1]],sizesharebyp$packshareprice[as.numeric(sizesharebyp$price) == prices[2]],
  sizesharebyp$packshareprice[as.numeric(sizesharebyp$price) == prices[3]],sizesharebyp$packshareprice[as.numeric(sizesharebyp$price) == prices[4]])


  xvec = 0:(ibar-1)
  matplot(xvec[1:nrow(xx)],xx,type="l",ylab = "Share of purchases", xlab = "Number of Packages", main=expr)
  legend("topright",paste("p = ",prices),col=1:4,lty=1:4,inset=0.01)
  dev.off()

  sink(paste(figstub,"/stats",paste(abs(coeff[,1]),collapse="_"),".txt",sep=""))
  cat("Total quantity purchased per hh per week",mean(simhh$x[,tstart:nt]),"\n")
  cat("Purchase frequency",1-sizeshare$share[1],"\n")
  cat("Summary of Interval Between Purchases:\n")
  print(summary(as.vector(lastp1[lastp1 > -1 & simhh$x > 0])))
  sink()

  istart = rep(0,nhhs)

  # prob of buy after 1 sale

  stpd = 100

  # simulated prices to period 100, then a sale, then constant prices

  simprices1 = simprices
  simprices1$p[,stpd] = pstates[1]
  simprices1$i[,stpd] = 1
  simprices1$p[,(stpd+1):200] = pstates[9]
  simprices1$i[,(stpd+1):200] = 9

  set.seed(8371363)

  simhh1 <- createdata(coeff,pstates,ntypes,delta,ibar,pprobs,cdraws,simprices1,fform,npar,logitsd,info=info)

  # prob of buy with sale and then another in a period

  simprices2 = simprices
  simprices2$p[,stpd] = pstates[1]
  simprices2$i[,stpd] = 1
  simprices2$p[,(stpd+1):200] = pstates[9]
  simprices2$i[,(stpd+1):200] = 9
  simprices2$p[,stpd+2] = pstates[1]
  simprices2$i[,stpd+2] = 1

  set.seed(8371363)

  simhh2 <- createdata(coeff,pstates,ntypes,delta,ibar,pprobs,cdraws,simprices2,fform,npar,logitsd,info=info)

  # 10% across the board price cut

  simprices3 = simprices
  simprices3$p = simprices3$p*0.9
  pstates3 = pstates*0.9

  set.seed(8371363)

  simhh3 <- createdata(coeff,pstates3,ntypes,delta,ibar,pprobs,cdraws,simprices3,fform,npar,logitsd,info=info)
  purchprob1 = array(0,c(10,ibar))
  condprob1 = array(0,c(10,ibar))
  purchprob2 = array(0,c(10,ibar))
  condprob2 = array(0,c(10,ibar))
  for(i in 1:10) {
    z1 = table(simhh1$x[,stpd-1+i])/nhhs
    z2 = table(simhh2$x[,stpd-1+i])/nhhs
    purchprob1[i,1:length(z1)] = z1
    purchprob2[i,1:length(z2)] = z2
    z1 = table(simhh1$x[simhh1$x[,stpd] > 0,stpd+i])/sum(simhh1$x[,stpd] > 0)
    z2 = table(simhh2$x[simhh2$x[,stpd] > 0,stpd+i])/sum(simhh2$x[,stpd] > 0)
    condprob1[i,1:length(z1)] = z1
    condprob2[i,1:length(z2)] = z2
  }
  totq1 = c(mean(simhh$x[,1]),mean(simhh$x[,1]>0),sum(simhh$x)/(nhhs*nt))
  totq3 = c(mean(simhh3$x[,1]),mean(simhh3$x[,1]>0),sum(simhh3$x)/(nhhs*nt))

  pdf(paste(figstub,"/pprob1",paste(abs(coeff[,1]),collapse="_"),".pdf",sep=""))
  plot(0:(ibar-2),1-purchprob1[,1],type="l",main=expr,ylab="Purchase Prob",xlab="Period")
  dev.off()

  pdf(paste(figstub,"/cprob1",paste(abs(coeff[,1]),collapse="_"),".pdf",sep=""))
  plot(1:(ibar-1),1-condprob1[,1],type="l",main=expr,ylab="Purchase Prob",xlab="Period")
  dev.off()

  pdf(paste(figstub,"/pprob2",paste(abs(coeff[,1]),collapse="_"),".pdf",sep=""))
  plot(0:(ibar-2),1-purchprob2[,1],type="l",main=expr,ylab="Purchase Prob",xlab="Period")
  dev.off()

  pdf(paste(figstub,"/cprob2",paste(abs(coeff[,1]),collapse="_"),".pdf",sep=""))
  plot(1:(ibar-1),1-condprob2[,1],type="l",main=expr,ylab="Purchase Prob",xlab="Period")
  dev.off()

  cpsave1[[bigrep]] = condprob1
  cpsave2[[bigrep]] = condprob2
  ppsave1[[bigrep]] = purchprob1
  ppsave2[[bigrep]] = purchprob2
  psharesave[[bigrep]] = sizeshare
  psharepsave[[bigrep]] = sizesharebyp
  ipsave[[bigrep]] = summary(as.vector(lastp1[lastp1 > -1 & simhh$x > 0]))
  qsave[[bigrep]] = c(totq1,totq3)

  cat("Finished Replication",bigrep,"\n")

}

identtab = c("Baseline purchase hazard post-sale","Change purchase hazard post-sale","%Purch prob second sale","%Purch Freq","Prob 2/Prob 1","Avg Interpurchase Time","%Prob 2+ units given deal","%Price Elasticity (Average)")

# switch around the last 2 - easier to read
cc2 = coefcols
cc2[4:5] = c(7,6)

pnames = c("Base","gamma","beta","alpha","sc0","sc1","cc")

sigfig = 2

moments = matrix(0,nrow=8,ncol=length(coefcols))

for(k in 1:length(cc2)) {
  i = cc2[k]
  moments[1,k] = mean(1-cpsave1[[i]][3:(ibar-1),1])
  moments[2,k] = cpsave1[[i]][1,1]-cpsave1[[i]][2,1]
  moments[3,k] = 1-cpsave2[[i]][2,1]
  moments[4,k] = 1-psharesave[[i]]$share[1]
  moments[5,k] = psharesave[[i]]$share[3]/psharesave[[i]]$share[2]
  moments[6,k] = ipsave[[i]][4]
  moments[7,k] = 1-sum(psharepsave[[i]]$packshareprice[45:46])
  moments[8,k] = ((qsave[[i]][3]-qsave[[i]][6])/qsave[[i]][3])/0.1
}

for(i in 1:length(cc2)) {
  k=cc2[i]
  for(j in 1:length(identtab)) {
    if(i==1) {
      identtab[j] = paste(identtab[j]," & ",round(moments[j,i],sigfig),sep="")
    } else {
      identtab[j] = paste(identtab[j]," & ",round(((moments[j,i]-moments[j,1])/abs(moments[j,1]))/((abs(coefmat[k-1,k])-abs(coefmat[k-1,1]))/abs(coefmat[k-1,1])),sigfig),sep="")
    }
  }
}

for(i in 1:length(identtab)) {
  identtab[i] = paste(identtab[i]," \\\\",sep="")
}
identtab = identtab[c(5,1,2,6)]

write.table(identtab,file=paste(tablepath,"identification.tex",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)
