# functions used by the simulations

# compute stationary distribution of a matrix

getstationary <- function(x,n,method=1) {

  ee = eigen(x)

  ind = which(abs(Re(ee$values)-1) <= .Machine$double.eps)

  if(length(ind) > 0 & method == 1) {

    xout = matrix(abs(ee$vectors[,ind])/sum(abs(ee$vectors[,ind])),nrow=1,ncol=ncol(x))

  } else {

    xout = x

    for(i in 1:n) {
      xout=xout%*%x
    }

  }

  return(xout)

}

# simulate price data

simprice <- function(pstates,op,nstore,nhhs,nt,hhassign = NULL,
                     initprice = NULL) {

  np = length(pstates)

  pmat = matrix(0,nrow=np,ncol=np)
  pmat[1:np,1:(np-1)] = op

  if(np > 2) {

    pmat[1:np,np] = 1-apply(op,1,sum)

  } else {

    pmat[1:2,2] = 1-op

  }

  if(is.null(hhassign)) {

    ps = getstationary(pmat,50)

    hhassign = sample(1:nstore,size=nhhs,replace=TRUE,prob=rep(1/nstore,nstore))

    simprices <- list(p=matrix(0,nrow=nhhs,ncol=nt),i=matrix(0,nrow=nhhs,ncol=nt))
    for(st in 1:nstore) {
      simprices$i[hhassign == st,1] = sample(1:np,size=1,replace=TRUE,prob=ps[1,])
      for(i in 2:nt) {
        simprices$i[hhassign == st,i] = sample(1:np,size=1,replace=TRUE,prob=pmat[simprices$i[min(which(hhassign == st)),i-1],])
      }
    }

  } else {

    simprices <- list(p=matrix(0,nrow=nhhs,ncol=nt),i=matrix(0,nrow=nhhs,ncol=nt))
    for(st in 1:nstore) {
      simprices$i[hhassign == st,1] = sample(1:np,size=1,replace=TRUE,prob=pmat[initprice[min(which(hhassign == st))],])
      for(i in 2:nt) {
        simprices$i[hhassign == st,i] = sample(1:np,size=1,replace=TRUE,prob=pmat[simprices$i[min(which(hhassign == st)),i-1],])
      }
    }

  }

  simprices$p = matrix(pstates[simprices$i],nrow=nhhs,ncol=nt)

  return(list(simprices,hhassign))

}


# solve for value functions (done in C)

solvevf <- function(coeff,pstates,ntypes,delta,ibar,pprobs,tol,fform,npar,logitsd=1) {
  return(.Call("vfiter",coeff,ntypes,delta,ibar,pstates,as.integer(length(pstates)),pprobs,tol,as.integer(fform),as.integer(npar)))
}


# Simulate household choices (call to C code)

hhchoicesim.c <- function(coef,pstates,pprobs,vfpred,simpricesi,istartin,cdraws,fform,npar,info,logitsd=1) {

  out = .Call("hhchoicesim",coef,info$ntypes,info$delta,info$ibar,pstates,as.integer(length(pstates)),pprobs,as.vector(vfpred),as.integer(info$nhhs),as.integer(info$nt),
              as.integer(istartin),as.integer(simpricesi),runif(info$nhhs*info$nt),as.integer(info$hhtypes),as.integer(fform),as.integer(npar))


  return(list(x=matrix(out[[1]],nrow=info$nhhs,ncol=info$nt),
              inv=matrix(out[[2]],nrow=info$nhhs,ncol=info$nt),
              cons=matrix(out[[3]],nrow=info$nhhs,ncol=info$nt),
              util=matrix(out[[4]],nrow=info$nhhs,ncol=info$nt),
              iv=matrix(out[[5]],nrow=info$nhhs,ncol=info$nt)))
}


# create artificial data

createdata <- function(coef,pstates,ntypes,delta,ibar,pprobs,cdraws,simprices,fform,npar,logitsd=1,tol=1e-7,
                       info,istart = rep(0,nhhs)) {

  v = solvevf(coef,pstates,ntypes,delta,ibar,pprobs,tol,fform,npar,logitsd=logitsd)

  np = length(pstates)

  vf = array(v,c(ntypes,np,ibar))

  return(hhchoicesim.c(coef,pstates,pprobs,vf,simprices$i,istart,cdraws,fform,npar,info,logitsd))

}
